@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2010-2023 - nibble, alvaro, pancake */
+/* radare - LGPL - Copyright 2010-2024 - nibble, alvaro, pancake */
 
 #define R_LOG_ORIGIN "fcn"
 
@@ -78,7 +78,7 @@ static int read_ahead(ReadAhead *ra, RAnal *anal, ut64 addr, ut8 *buf, int len)
 		ra->cache_addr = addr;
 	}
 	int delta = addr - ra->cache_addr;
-	r_return_val_if_fail (delta >= 0, -1);
+	R_RETURN_VAL_IF_FAIL (delta >= 0, -1);
 	size_t length = sizeof (ra->cache) - delta;
 	memcpy (buf, ra->cache + delta, R_MIN (len, length));
 	return len;
@@ -89,7 +89,7 @@ R_API int r_anal_function_resize(RAnalFunction *fcn, int newsize) {
 	RAnalBlock *bb;
 	RListIter *iter, *iter2;
 
-	r_return_val_if_fail (fcn, false);
+	R_RETURN_VAL_IF_FAIL (fcn, false);
 
 	if (newsize < 1) {
 		return false;
@@ -174,7 +174,7 @@ static bool is_symbol_flag(const char *name) {
 }
 
 static bool next_instruction_is_symbol(RAnal *anal, RAnalOp *op) {
-	r_return_val_if_fail (anal && op && anal->flb.get_at, false);
+	R_RETURN_VAL_IF_FAIL (anal && op && anal->flb.get_at, false);
 	RFlagItem *fi = anal->flb.get_at (anal->flb.f, op->addr + op->size, false);
 	return (fi && fi->name && is_symbol_flag (fi->name));
 }
@@ -294,7 +294,7 @@ static ut64 try_get_cmpval_from_parents(RAnal *anal, RAnalFunction *fcn, RAnalBl
 		R_LOG_DEBUG ("try_get_cmpval_from_parents: cmp_reg not defined");
 		return UT64_MAX;
 	}
-	r_return_val_if_fail (fcn && fcn->bbs, UT64_MAX);
+	R_RETURN_VAL_IF_FAIL (fcn && fcn->bbs, UT64_MAX);
 	RListIter *iter;
 	RAnalBlock *tmp_bb;
 	r_list_foreach (fcn->bbs, iter, tmp_bb) {
@@ -313,7 +313,7 @@ static ut64 try_get_cmpval_from_parents(RAnal *anal, RAnalFunction *fcn, RAnalBl
 }
 
 static inline bool regs_exist(RAnalValue *src, RAnalValue *dst) {
-	r_return_val_if_fail (src && dst, false);
+	R_RETURN_VAL_IF_FAIL (src && dst, false);
 	return src->reg && dst->reg;
 }
 
@@ -570,6 +570,18 @@ static inline bool has_vars(RAnal *anal, ut64 addr) {
 	return fcn && r_anal_var_count_all (fcn) > 0;
 }
 
+static void fcn_rename_readdr(RAnalFunction *fcn, ut64 to) {
+	r_strf_var (addrstr, 64, "%08"PFMT64x, fcn->addr);
+	char *s = strstr (fcn->name, addrstr);
+	if (s) {
+		char *pfx = r_str_ndup (fcn->name, s - fcn->name);
+		free (fcn->name);
+		fcn->name = r_str_newf ("%s%08"PFMT64x, pfx, to);
+		free (pfx);
+	}
+	fcn->addr = to;
+}
+
 static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int depth) {
 	const char *variadic_reg = NULL;
 	ReadAhead ra = {0};
@@ -612,12 +624,19 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 	};
 	const char *arch = anal->config? anal->config->arch: R_SYS_ARCH;
 	bool arch_destroys_dst = does_arch_destroys_dst (arch);
-	const bool is_arm = !strncmp (arch, "arm", 3);
+#if R2_USE_NEW_ABI
+	const bool flagends = anal->opt.flagends;
+#else
+	const bool flagends = anal->coreb.cfggeti (anal->coreb.core, "anal.flagends");
+#endif
+	const bool is_arm = r_str_startswith (arch, "arm");
+	const bool is_mips = !is_arm && r_str_startswith (arch, "mips");
 	const bool is_v850 = is_arm ? false: (arch && (!strncmp (arch, "v850", 4) || !strncmp (anal->coreb.cfgGet (anal->coreb.core, "asm.cpu"), "v850", 4)));
 	const bool is_x86 = is_arm ? false: arch && !strncmp (arch, "x86", 3);
 	const bool is_amd64 = is_x86 ? fcn->cc && !strcmp (fcn->cc, "amd64") : false;
 	const bool is_dalvik = is_x86 ? false : arch && !strncmp (arch, "dalvik", 6);
 	const bool propagate_noreturn = anal->opt.propagate_noreturn;
+	ut64 v1 = UT64_MAX;
 
 	if (r_cons_is_breaked ()) {
 		return R_ANAL_RET_END;
@@ -723,6 +742,7 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 	bool has_variadic_reg = !!variadic_reg;
 
 	op = r_anal_op_new ();
+	const ut32 opflags = R_ARCH_OP_MASK_BASIC | R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL | R_ARCH_OP_MASK_HINT;
 	while (addrbytes * idx < maxlen) {
 		if (!last_is_reg_mov_lea) {
 			last_reg_mov_lea_name = NULL;
@@ -750,7 +770,8 @@ repeat:
 			gotoBeach (R_ANAL_RET_ERROR)
 		}
 		r_anal_op_fini (op);
-		oplen = r_anal_op (anal, op, at, buf, bytes_read, R_ARCH_OP_MASK_BASIC | R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL | R_ARCH_OP_MASK_HINT);
+		oplen = r_anal_op (anal, op, at, buf, bytes_read, opflags);
+
 		if (oplen < 1) {
 			R_LOG_DEBUG ("Invalid instruction at 0x%"PFMT64x" with %d bits", at, anal->config->bits);
 			// gotoBeach (R_ANAL_RET_ERROR);
@@ -767,11 +788,17 @@ repeat:
 		src1 = r_vector_at (&op->srcs, 1);
 
 		if (anal->opt.nopskip && fcn->addr == at) {
+			const int codealign = r_anal_archinfo (anal, R_ARCH_INFO_CODE_ALIGN);
+			if (codealign > 1) {
+				if (at % codealign) {
+					goto noskip;
+				}
+			}
 			RFlagItem *fi = anal->flb.get_at (anal->flb.f, addr, false);
 			if (!fi || strstr (fi->name, "sym.")) {
 				if ((addr + delay.un_idx - oplen) == fcn->addr) {
 					if (r_anal_block_relocate (bb, bb->addr + oplen, bb->size - oplen)) {
-						fcn->addr += oplen;
+						fcn_rename_readdr (fcn, fcn->addr + oplen);
 						idx = delay.un_idx;
 						r_anal_op_fini (op);
 						goto repeat;
@@ -789,11 +816,13 @@ repeat:
 				if (r_anal_block_relocate (bb, at + op->size, bb->size)) {
 					r_anal_op_fini (op);
 					addr = at + op->size;
-					fcn->addr = addr;
+					fcn_rename_readdr (fcn, addr);
+					// force function rename if needed
 					goto repeat;
 				}
 			}
 		}
+noskip:
 		if (op->hint.new_bits) {
 			r_anal_hint_set_bits (anal, op->jump, op->hint.new_bits);
 		}
@@ -812,6 +841,14 @@ repeat:
 				R_LOG_DEBUG ("Overlapped at 0x%08"PFMT64x, at);
 			}
 		}
+		if (flagends && fcn->addr != at) {
+			RFlagItem *flag = anal->flag_get (anal->flb.f, at);
+			if (flag) {
+				if (r_str_startswith (flag->name, "sym")) {
+					gotoBeach (R_ANAL_RET_END);
+				}
+			}
+		}
 		if (!overlapped) {
 			const ut64 newbbsize = bb->size + oplen;
 			if (newbbsize > MAX_FCN_SIZE) {
@@ -1075,6 +1112,7 @@ repeat:
 			break;
 		case R_ANAL_OP_TYPE_LOAD:
 			// R2R db/anal/arm db/esil/apple
+			//v1 = UT64_MAX; // reset v1 jmptable pointer value for mips only
 			if (anal->iob.is_valid_offset (anal->iob.io, op->ptr, 0)) {
 				// TODO: what about the qword loads!??!?
 				ut8 dd[4] = {0};
@@ -1083,6 +1121,14 @@ repeat:
 				ut64 da = (ut64)r_read_ble32 (dd, R_ARCH_CONFIG_IS_BIG_ENDIAN (anal->config));
 				if (da != UT32_MAX && da != UT64_MAX && anal->iob.is_valid_offset (anal->iob.io, da, 0)) {
 					/// TODO: this must be CODE | READ , not CODE|DATA, but raises 10 fails
+					if (is_mips && anal->opt.jmptbl) {
+						const char *esil = r_strbuf_get (&op->esil);
+						if (strstr (esil, "v1,=")) {
+							// eprintf("iftarget is v1 (%s)\n", esil);
+							// eprintf ("LOAD FROM %llx -> %llx\n", op->ptr, da);
+							v1 = da;
+						}
+					}
 					// r_anal_xrefs_set (anal, op->addr, da, R_ANAL_REF_TYPE_CODE | R_ANAL_REF_TYPE_DATA);
 					r_anal_xrefs_set (anal, op->addr, da, R_ANAL_REF_TYPE_ICOD | R_ANAL_REF_TYPE_EXEC);
 				} else {
@@ -1094,14 +1140,24 @@ repeat:
 				// r_anal_xrefs_set (anal, op->addr, op->ptr, R_ANAL_REF_TYPE_DATA);
 				if (anal->opt.loads) {
 					// set this address as data if destination is not code
-						r_meta_set (anal, R_META_TYPE_DATA, op->ptr, 4, "");
+					r_meta_set (anal, R_META_TYPE_DATA, op->ptr, 4, "");
 				}
 			}
 			break;
 			break;
 			// Case of valid but unused "add [rax], al"
 		case R_ANAL_OP_TYPE_ADD:
-			if (is_arm && anal->config->bits == 32) {
+			if (is_mips) {
+				if (anal->opt.jmptbl && v1 != UT64_MAX) {
+					// TODO: ensure we add in v1 // const char *esil = r_strbuf_get (&op->esil);
+					v1 += (st32)op->val;
+					// align v1
+					while (v1 & 3) {
+						v1++;
+					}
+					R_LOG_DEBUG ("[0x%"PFMT64x"]============= 0x%"PFMT64x, op->addr, v1);
+				}
+			} else if (is_arm && anal->config->bits == 32) {
 				if (len >= 4 && !memcmp (buf, "\x00\xe0\x8f\xe2", 4)) {
 					// add lr, pc, 0 //
 					last_is_add_lr_pc = true; // TODO: support different values, not just 0
@@ -1235,7 +1291,7 @@ repeat:
 			if (bb->cond) {
 				bb->cond->type = op->cond;
 			}
-			if (anal->opt.jmptbl) {
+			if (anal->opt.jmptbl && !is_mips) {
 				if (op->ptr != UT64_MAX) {
 					ut64 table_size, default_case;
 					table_size = anal->cmpval + 1;
@@ -1310,6 +1366,32 @@ repeat:
 				op->type = R_ANAL_OP_TYPE_CALL;
 				op->fail = op->addr + 4;
 				break;
+			} else if (is_mips && anal->opt.jmptbl) {
+				// lw v1, -0x7fc4(gp) ; gp = 0x684c00 - 0x7fc4 // read 4 bytes at gp-0x7fc4
+				// sll v0, s0, 2   // select the case from the pointer table * 4
+				// addiu v1, v1, 0x74b4 ; [gp-0x7fc4] + 0x74b4 ;; is jmptbl_ptr_addr
+				// addu v0, v0, v1 // increment the pointer
+				ut64 jmptbl_ptr_addr = v1; // 0x005a74b4;
+				ut64 default_case = UT64_MAX; // op->addr + 8;
+				int tablesize = 0;
+				ut64 tblptr = v1;
+				while (1) {
+					ut8 dd[4];
+					// read le32 until the number is not negative
+					(void)anal->iob.read_at (anal->iob.io, tblptr, (ut8 *) dd, sizeof (dd));
+					// if page have exec perms
+					st32 n = (st32)r_read_ble32 (dd, R_ARCH_CONFIG_IS_BIG_ENDIAN (anal->config));
+					if (n >= -1) {
+						break;
+					}
+					tblptr += 4;
+					tablesize ++;
+				}
+				tablesize *= 4;
+				ut64 tblloc = jmptbl_ptr_addr;
+				int sz = 4;
+				ret = try_walkthrough_jmptbl (anal, fcn, bb, depth, op->addr, 0,
+						tblloc, jmptbl_ptr_addr, sz, tablesize, default_case, ret);
 			} else if (is_v850 && anal->opt.jmptbl) {
 				int ptsz = (anal->cmpval && anal->cmpval != UT64_MAX)? anal->cmpval + 1: 4;
 				if ((int)anal->cmpval > 0) {
@@ -1560,12 +1642,12 @@ beach:
 }
 
 R_API int r_anal_function_bb(RAnal *anal, RAnalFunction *fcn, ut64 addr, int depth) {
-	r_return_val_if_fail (anal && fcn, -1);
+	R_RETURN_VAL_IF_FAIL (anal && fcn, -1);
 	return fcn_recurse (anal, fcn, addr, anal->opt.bb_max_size, depth - 1);
 }
 
 R_API bool r_anal_check_fcn(RAnal *anal, ut8 *buf, ut16 bufsz, ut64 addr, ut64 low, ut64 high) {
-	r_return_val_if_fail (anal && buf, false);
+	R_RETURN_VAL_IF_FAIL (anal && buf, false);
 	RAnalOp op = {
 		0
 	};
@@ -1610,7 +1692,7 @@ R_API bool r_anal_check_fcn(RAnal *anal, ut8 *buf, ut16 bufsz, ut64 addr, ut64 l
 }
 
 R_API void r_anal_trim_jmprefs(RAnal *anal, RAnalFunction *fcn) {
-	r_return_if_fail (anal && fcn);
+	R_RETURN_IF_FAIL (anal && fcn);
 
 	const char *arch = R_UNWRAP4 (anal, arch, session, name);
 	const bool is_x86 = arch && !strcmp (arch, "x86"); // HACK
@@ -1634,7 +1716,7 @@ R_API void r_anal_trim_jmprefs(RAnal *anal, RAnalFunction *fcn) {
 }
 
 R_API void r_anal_del_jmprefs(RAnal *anal, RAnalFunction *fcn) {
-	r_return_if_fail (anal && fcn);
+	R_RETURN_IF_FAIL (anal && fcn);
 
 	RVecAnalRef *refs = r_anal_function_get_refs (fcn);
 	if (!refs) {
@@ -1654,7 +1736,7 @@ R_API void r_anal_del_jmprefs(RAnal *anal, RAnalFunction *fcn) {
 
 /* Does NOT invalidate read-ahead cache. */
 R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype) {
-	r_return_val_if_fail (anal && fcn, 0);
+	R_RETURN_VAL_IF_FAIL (anal && fcn, 0);
 	RPVector *metas = r_meta_get_all_in (anal, addr, R_META_TYPE_ANY);
 	if (metas) {
 		void **it;
@@ -1874,7 +1956,7 @@ R_API int r_anal_function_complexity(RAnalFunction *fcn) {
 	if (result < 1 && (!anal || anal->verbose)) {
 		R_LOG_WARN ("CC = E(%d) - N(%d) + (2 * P(%d)) < 1 at 0x%08"PFMT64x, E, N, P, fcn->addr);
 	}
-	// r_return_val_if_fail (result > 0, 0);
+	// R_RETURN_VAL_IF_FAIL (result > 0, 0);
 	return result;
 }
 
@@ -2042,7 +2124,7 @@ R_API char *r_anal_function_get_signature(RAnalFunction *function) {
 
 /* set function signature from string */
 R_API int r_anal_str_to_fcn(RAnal *a, RAnalFunction *f, const char *sig) {
-	r_return_val_if_fail (a || f || sig, false);
+	R_RETURN_VAL_IF_FAIL (a || f || sig, false);
 	char *error_msg = NULL;
 	const char *out = r_anal_cparse (a, sig, &error_msg);
 	if (out) {
@@ -2084,7 +2166,7 @@ R_API int r_anal_function_count(RAnal *anal, ut64 from, ut64 to) {
 /* return the basic block in fcn found at the given address.
  * NULL is returned if such basic block doesn't exist. */
 R_API RAnalBlock *r_anal_function_bbget_in(RAnal *anal, RAnalFunction *fcn, ut64 addr) {
-	r_return_val_if_fail (anal && fcn, NULL);
+	R_RETURN_VAL_IF_FAIL (anal && fcn, NULL);
 	if (addr == UT64_MAX) {
 		return NULL;
 	}
@@ -2104,7 +2186,7 @@ R_API RAnalBlock *r_anal_function_bbget_in(RAnal *anal, RAnalFunction *fcn, ut64
 }
 
 R_API RAnalBlock *r_anal_function_bbget_at(RAnal *anal, RAnalFunction *fcn, ut64 addr) {
-	r_return_val_if_fail (fcn && addr != UT64_MAX, NULL);
+	R_RETURN_VAL_IF_FAIL (fcn && addr != UT64_MAX, NULL);
 	RAnalBlock *b = r_anal_get_block_at (anal, addr);
 	if (b) {
 		return b;
@@ -2154,7 +2236,7 @@ R_API ut32 r_anal_function_cost(RAnalFunction *fcn) {
 }
 
 R_API int r_anal_function_count_edges(const RAnalFunction *fcn, R_NULLABLE int *ebbs) {
-	r_return_val_if_fail (fcn, 0);
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	RListIter *iter;
 	RAnalBlock *bb;
 	int edges = 0;
@@ -2207,7 +2289,7 @@ static bool can_affect_bp(RAnal *anal, RAnalOp* op) {
  * and "pop bp" at the end).
  */
 R_API void r_anal_function_check_bp_use(RAnalFunction *fcn) {
-	r_return_if_fail (fcn);
+	R_RETURN_IF_FAIL (fcn);
 	RAnal *anal = fcn->anal;
 	RListIter *iter;
 	RAnalBlock *bb;
@@ -2440,7 +2522,7 @@ static void calc_reachable_and_remove_block(RList *fcns, RAnalFunction *fcn, RAn
 }
 
 R_API void r_anal_update_analysis_range(RAnal *anal, ut64 addr, int size) {
-	r_return_if_fail (anal);
+	R_RETURN_IF_FAIL (anal);
 	RListIter *it, *it2, *tmp;
 	RAnalBlock *bb;
 	RAnalFunction *fcn;
@@ -2480,7 +2562,7 @@ R_API void r_anal_update_analysis_range(RAnal *anal, ut64 addr, int size) {
 }
 
 R_API void r_anal_function_update_analysis(RAnalFunction *fcn) {
-	r_return_if_fail (fcn);
+	R_RETURN_IF_FAIL (fcn);
 	RListIter *it, *it2, *tmp, *tmp2;
 	RAnalBlock *bb;
 	RAnalFunction *f;
