@@ -166,6 +166,7 @@ R_API bool try_walkthrough_casetbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *
 	return ret;
 }
 
+// TODO R2_600 - this api name must start with `r_anal_jmptbl`
 R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0) {
 	bool ret = ret0;
 	// jmptbl_size can not always be determined
@@ -198,6 +199,7 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 	}
 	bool is_arm = sarch ? r_str_startswith (sarch, "arm"): false;
 	bool is_x86 = !is_arm && r_str_startswith (sarch, "x86");
+	bool is_mips = !is_arm && !is_x86 && r_str_startswith (sarch, "mips");
 	const bool is_v850 = !is_arm && !is_x86 && ((sarch && r_str_startswith (sarch, "v850")) || r_str_startswith (anal->coreb.cfgGet (anal->coreb.core, "asm.cpu"), "v850"));
 	// eprintf ("JMPTBL AT 0x%"PFMT64x"\n", jmptbl_loc);
 	anal->iob.read_at (anal->iob.io, jmptbl_loc, jmptbl, jmptblsz);
@@ -245,6 +247,14 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 			// jump tables where sign extended movs are used
 			jmpptr = jmptbl_off + jmpdelta;
 		}
+		if (is_mips) {
+			jmpptr += (ip & 0xfffff);
+			// R2_600 - get the baddr from binbind
+			// ut64 baddr = r_anal_get_bbaddr (anal, ip);
+			// eprintf ("base address = %llx %llx\n", ip, baddr);
+			// const ut64 baddr = 0x400000; // hardcoded baddr
+			// jmpptr -= baddr;
+		}
 		if (anal->limit) {
 			if (jmpptr < anal->limit->from || jmpptr > anal->limit->to) {
 				break;
@@ -257,6 +267,11 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 		apply_case (anal, block, ip, sz, jmpptr, casenum, jmptbl_loc + offs, false);
 		analyze_new_case (anal, fcn, block, ip, jmpptr, depth);
 	}
+	if (is_mips) {
+		// default case for mips is right after the 'jr v0' instruction
+		apply_case (anal, block, ip, sz, ip + 8, -1, jmptbl_loc + offs, false);
+		analyze_new_case (anal, fcn, block, ip, ip + 8, depth);
+	}
 
 	if (offs > 0) {
 		if (default_case == 0) {
