@@ -265,7 +265,7 @@ static void dalvik_math_op(RAnalOp* op, const ut8* data, int len, RAnalOpMask ma
 }
 
 static int dalvik_disassemble(RArchSession *as, RAnalOp *op, ut64 addr, const ut8 *buf, int len, int size) {
-	r_return_val_if_fail  (as && op && buf && len > 0, -1);
+	R_RETURN_VAL_IF_FAIL  (as && op && buf && len > 0, -1);
 
 	int vA, vB, vC, vD, vE, vF, vG, vH, payload = 0;
 	char str[1024], *strasm = NULL;
@@ -761,9 +761,9 @@ static int dalvik_disassemble(RArchSession *as, RAnalOp *op, ut64 addr, const ut
 			R_FREE (strasm);
 			size = 2;
 		}
-		op->mnemonic = r_str_new (r_str_get_fail (strasm, "invalid"));
+		op->mnemonic = strdup (r_str_get_fail (strasm, "invalid"));
 	} else if (len > 0) {
-		op->mnemonic = r_str_new ("invalid");
+		op->mnemonic = strdup ("invalid");
 		size = len;
 	}
 
@@ -789,12 +789,12 @@ static bool decode(RArchSession *as, RAnalOp *op, RAnalOpMask mask) {
 	const ut64 addr = op->addr;
 	const ut8 *data = op->bytes;
 	const int len = op->size;
-	r_return_val_if_fail (as && op && data && len > 0, -1);
+	R_RETURN_VAL_IF_FAIL (as && op && data && len > 0, -1);
 
 	int sz = dalvik_opcodes[data[0]].len;
 	if (!op || sz > len) {
 		if (mask & R_ARCH_OP_MASK_DISASM) {
-			op->mnemonic = r_str_new ("invalid");
+			op->mnemonic = strdup ("invalid");
 		}
 		return false;
 	}
