@@ -1,74 +1,67 @@
-/* radare2 - LGPL - Copyright 2009-2021 - pancake, nibble, dso */
+/* radare2 - LGPL - Copyright 2009-2024 - pancake, nibble, dso */
 
 #include <r_bin.h>
 #include <r_hash.h>
 #include "i/private.h"
 
 // maybe too big sometimes? 2KB of stack eaten here..
-#define R_STRING_SCAN_BUFFER_SIZE 2048
+#define R_STRING_SCAN_BUFFER_SIZE 4096
 #define R_STRING_MAX_UNI_BLOCKS 4
 
 static RBinClass *__getClass(RBinFile *bf, const char *name) {
-	r_return_val_if_fail (bf && bf->o && bf->o->classes_ht && name, NULL);
-	return ht_pp_find (bf->o->classes_ht, name, NULL);
+	R_RETURN_VAL_IF_FAIL (bf && bf->bo && bf->bo->classes_ht && name, NULL);
+	return ht_pp_find (bf->bo->classes_ht, name, NULL);
 }
 
 static RBinSymbol *__getMethod(RBinFile *bf, const char *klass, const char *method) {
-	r_return_val_if_fail (bf && bf->o && bf->o->methods_ht && klass && method, NULL);
-	const char *name = sdb_fmt ("%s::%s", klass, method);
-	return ht_pp_find (bf->o->methods_ht, name, NULL);
+	R_RETURN_VAL_IF_FAIL (bf && bf->bo && bf->bo->methods_ht && klass && method, NULL);
+	r_strf_var (name, 128, "%s::%s", klass, method);
+	return ht_pp_find (bf->bo->methods_ht, name, NULL);
 }
 
 static RBinString *__stringAt(RBinFile *bf, RList *ret, ut64 addr) {
-	if (addr != 0 && addr != UT64_MAX) {
-		return ht_up_find (bf->o->strings_db, addr, NULL);
+	if (R_LIKELY (addr != 0 && addr != UT64_MAX)) {
+		return ht_up_find (bf->bo->strings_db, addr, NULL);
 	}
 	return NULL;
 }
 
-static ut64 binobj_a2b(RBinObject *o, ut64 addr) {
-	return o ? addr + o->baddr_shift : addr;
-}
-
 static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
-	r_return_if_fail (bf && string);
+	R_RETURN_IF_FAIL (bf && string);
 
 	int mode = bf->strmode;
-	ut64 addr, vaddr;
 	RBin *bin = bf->rbin;
 	if (!bin) {
 		return;
 	}
-	const char *section_name, *type_string;
 	RIO *io = bin->iob.io;
 	if (!io) {
 		return;
 	}
-	RBinSection *s = r_bin_get_section_at (bf->o, string->paddr, false);
+	RBinSection *s = r_bin_get_section_at (bf->bo, string->paddr, false);
 	if (s) {
 		string->vaddr = s->vaddr + (string->paddr - s->paddr);
 	}
-	section_name = s ? s->name : "";
-	type_string = r_bin_string_type (string->type);
-	vaddr = addr = r_bin_get_vaddr (bin, string->paddr, string->vaddr);
+	const char *section_name = s ? s->name : "";
+	const char *type_string = r_bin_string_type (string->type);
+	ut64 vaddr = r_bin_get_vaddr (bin, string->paddr, string->vaddr);
+	ut64 addr = vaddr;
 
 	// If raw string dump mode, use printf to dump directly to stdout.
 	//  PrintfCallback temp = io->cb_printf;
 	switch (mode) {
 	case R_MODE_JSON:
-		{
-			if (pj) {
-				pj_o (pj);
-				pj_kn (pj, "vaddr", vaddr);
-				pj_kn (pj, "paddr", string->paddr);
-				pj_kn (pj, "ordinal", string->ordinal);
-				pj_kn (pj, "size", string->size);
-				pj_kn (pj, "length", string->length);
-				pj_ks (pj, "section", section_name);
-				pj_ks (pj, "type", type_string);
-				pj_ks (pj, "string", string->string);
-				pj_end (pj);
-			}
+		if (pj) {
+			pj_o (pj);
+			pj_kn (pj, "vaddr", vaddr);
+			pj_kn (pj, "paddr", string->paddr);
+			pj_kn (pj, "ordinal", string->ordinal);
+			pj_kn (pj, "size", string->size);
+			pj_kn (pj, "length", string->length);
+			pj_ks (pj, "section", section_name);
+			pj_ks (pj, "type", type_string);
+			pj_ks (pj, "string", string->string);
+			pj_end (pj);
 		}
 		break;
 	case R_MODE_SIMPLEST:
@@ -108,28 +101,43 @@ static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
 	}
 }
 
-static int string_scan_range(RList *list, RBinFile *bf, int min,
-			      const ut64 from, const ut64 to, int type, int raw, RBinSection *section) {
+// TODO: this code must be implemented in RSearch as options for the strings mode
+static int string_scan_range(RList *list, RBinFile *bf, int min, const ut64 from, const ut64 to, int type, int raw, RBinSection *section) {
 	RBin *bin = bf->rbin;
-	ut8 tmp[R_STRING_SCAN_BUFFER_SIZE];
+	const bool strings_nofp = bin->strings_nofp;
+	ut8 tmp[64]; // temporal buffer to encode characters in utf8 form
+	RStrBuf *sb = r_strbuf_new ("");
 	ut64 str_start, needle = from;
-	int count = 0, i, rc, runes;
+	int i, rc, runes;
 	int str_type = R_STRING_TYPE_DETECT;
+	const int limit = bf->rbin->limit;
+	int minstr = bin->minstrlen;
+	if (minstr < 1) {
+		minstr = 1;
+	}
+	int maxstr = bin->maxstrlen;
+	if (maxstr < 1) {
+		maxstr = R_STRING_SCAN_BUFFER_SIZE;
+	}
 
 	// if list is null it means its gonna dump
-	r_return_val_if_fail (bf, -1);
+	R_RETURN_VAL_IF_FAIL (bf, -1);
 
 	if (type == -1) {
 		type = R_STRING_TYPE_DETECT;
 	}
-	if (from == to) {
+	if (from == UT64_MAX || from == to) {
 		return 0;
 	}
 	if (from > to) {
-		eprintf ("Invalid range to find strings 0x%"PFMT64x" .. 0x%"PFMT64x"\n", from, to);
+		R_LOG_ERROR ("Invalid range to find strings 0x%"PFMT64x" .. 0x%"PFMT64x, from, to);
+		return -1;
+	}
+	st64 len = (st64)(to - from);
+	if (len < 1 || len > ST32_MAX) {
+		R_LOG_ERROR ("String scan range is invalid (%"PFMT64d" bytes)", len);
 		return -1;
 	}
-	int len = to - from;
 	ut8 *buf = calloc (len, 1);
 	if (!buf || !min) {
 		free (buf);
@@ -147,7 +155,7 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 	}
 	r_buf_read_at (bf->buf, from, buf, len);
 	char *charset = r_sys_getenv ("RABIN2_CHARSET");
-	if (!R_STR_ISEMPTY (charset)) {
+	if (R_STR_ISNOTEMPTY (charset)) {
 		RCharset *ch = r_charset_new ();
 		if (r_charset_use (ch, charset)) {
 			int outlen = len * 4;
@@ -161,27 +169,27 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 						out[i] = 0;
 					}
 				}
-				len = res;
 				free (buf);
-#if 1
 				buf = out;
-#else
-				// buf = realloc (out, len + 1);
-#endif
-			} else {
-				eprintf ("Cannot allocate\n");
 			}
 		} else {
-			eprintf ("Invalid value for RABIN2_CHARSET.\n");
+			R_LOG_ERROR ("Invalid value for RABIN2_CHARSET");
 		}
 		r_charset_free (ch);
 	}
 	free (charset);
+	RConsIsBreaked is_breaked = (bin && bin->consb.is_breaked)? bin->consb.is_breaked: NULL;
 	// may oobread
-	while (needle < to) {
-		if (bin && bin->consb.is_breaked) {
-			if (bin->consb.is_breaked ()) {
-				break;
+	while (needle < to && needle < UT64_MAX - 4) {
+		if (is_breaked && is_breaked ()) {
+			break;
+		}
+		// smol optimization
+		if (to > 4 && needle < to - 4) {
+			ut32 n1 = r_read_le32 (buf + needle - from);
+			if (!n1) {
+				needle += 4;
+				continue;
 			}
 		}
 		rc = r_utf8_decode (buf + needle - from, to - needle, NULL);
@@ -189,18 +197,33 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 			needle++;
 			continue;
 		}
+		const bool addr_aligned = !(needle % 4);
+
 		if (type == R_STRING_TYPE_DETECT) {
 			char *w = (char *)buf + needle + rc - from;
-			if ((to - needle) > 5 + rc) {
-				bool is_wide32 = (needle + rc + 2 < to) && (!w[0] && !w[1] && !w[2] && w[3] && !w[4]);
-				if (is_wide32) {
-					str_type = R_STRING_TYPE_WIDE32;
+			if (((to - needle) > 8 + rc)) {
+				// TODO: support le and be
+				bool is_wide32le = (needle + rc + 2 < to) && (!w[0] && !w[1] && !w[2] && w[3] && !w[4]);
+				// reduce false positives
+				if (is_wide32le) {
+					if (!w[5] && !w[6] && w[7] && w[8]) {
+						is_wide32le = false;
+					}
+				}
+				if (!addr_aligned) {
+					is_wide32le = false;
+				}
+				if (is_wide32le && addr_aligned) {
+					str_type = R_STRING_TYPE_WIDE32; // asume big endian,is there little endian w32?
 				} else {
+					// bool is_wide = (n1 && n2 && n1 < 0xff && (!n2 || n2 < 0xff));
 					bool is_wide = needle + rc + 4 < to && !w[0] && w[1] && !w[2] && w[3] && !w[4];
 					str_type = is_wide? R_STRING_TYPE_WIDE: R_STRING_TYPE_ASCII;
 				}
 			} else {
-				str_type = R_STRING_TYPE_ASCII;
+				str_type = (rc > 1)
+					? R_STRING_TYPE_UTF8
+					: R_STRING_TYPE_ASCII;
 			}
 		} else if (type == R_STRING_TYPE_UTF8) {
 			str_type = R_STRING_TYPE_ASCII; // initial assumption
@@ -210,10 +233,10 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		runes = 0;
 		str_start = needle;
 
+		r_strbuf_set (sb, "");
 		/* Eat a whole C string */
-		for (i = 0; i < sizeof (tmp) - 4 && needle < to; i += rc) {
+		for (i = 0; i < maxstr && needle < to; i += rc) {
 			RRune r = {0};
-
 			if (str_type == R_STRING_TYPE_WIDE32) {
 				rc = r_utf32le_decode (buf + needle - from, to - needle, &r);
 				if (rc) {
@@ -245,43 +268,61 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 						r = 0;
 					}
 				}
-				rc = r_utf8_encode (tmp + i, r);
+				rc = r_utf8_encode (tmp, r);
+				tmp[rc] = 0;
+				r_strbuf_append (sb, (const char *)tmp);
 				runes++;
-				/* Print the escape code */
 			} else if (r && r < 0x100 && strchr ("\b\v\f\n\r\t\a\033\\", (char)r)) {
-				if ((i + 32) < sizeof (tmp) && r < 93) {
-					tmp[i + 0] = '\\';
-					tmp[i + 1] = "       abtnvfr             e  "
-					             "                              "
-					             "                              "
-					             "  \\"[r];
+				/* Print the escape code */
+				if (strings_nofp) {
+					rc = 2;
+					if (r && r < 0x100 && strchr ("\n\r\t\033\\", (char)r)) {
+						runes++; // accept it as it is
+						rc = 1;
+					} else {
+						rc = 1;
+						r = 0;
+						break;
+					}
 				} else {
-					// string too long
-					break;
+					if (r < 93) {
+						tmp[0] = '\\';
+						tmp[1] = "       abtnvfr             e  "
+							"                              "
+							"                              "
+							"  \\"[r];
+					} else {
+						// string too long
+						break;
+					}
+					rc = 2;
+					tmp[rc] = 0;
+					r_strbuf_append (sb, (const char *)tmp);
+					runes++;
 				}
-				rc = 2;
-				runes++;
 			} else {
 				/* \0 marks the end of C-strings */
 				break;
 			}
 		}
 
-		tmp[i++] = '\0';
+		i++;
 
 		if (runes < min && runes >= 2 && str_type == R_STRING_TYPE_ASCII && needle < to) {
 			// back up past the \0 to the last char just in case it starts a wide string
 			needle -= 2;
 		}
 		if (runes >= min) {
+			const char *tmpstr = r_strbuf_get (sb);
+			size_t tmplen = r_strbuf_length (sb);
 			// reduce false positives
-			int j, num_blocks, *block_list;
+			int j, num_blocks;
 			int *freq_list = NULL, expected_ascii, actual_ascii, num_chars;
 			if (str_type == R_STRING_TYPE_ASCII) {
-				for (j = 0; j < i; j++) {
-					char ch = tmp[j];
+				for (j = 0; j < tmplen; j++) {
+					char ch = tmpstr[j];
 					if (ch != '\n' && ch != '\r' && ch != '\t') {
-						if (!IS_PRINTABLE (tmp[j])) {
+						if (!IS_PRINTABLE (ch)) {
 							continue;
 						}
 					}
@@ -292,8 +333,8 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 			case R_STRING_TYPE_WIDE:
 			case R_STRING_TYPE_WIDE32:
 				num_blocks = 0;
-				block_list = r_utf_block_list ((const ut8*)tmp, i - 1,
-				                               str_type == R_STRING_TYPE_WIDE ? &freq_list : NULL);
+				int *block_list = r_utf_block_list ((const ut8*)tmpstr, tmplen - 1,
+						str_type == R_STRING_TYPE_WIDE? &freq_list: NULL);
 				if (block_list) {
 					for (j = 0; block_list[j] != -1; j++) {
 						num_blocks++;
@@ -313,12 +354,13 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 					if (actual_ascii > expected_ascii) {
 						ascii_only = true;
 						needle = str_start;
-						free (block_list);
+						R_FREE (block_list);
 						continue;
 					}
 				}
-				free (block_list);
+				R_FREE (block_list);
 				if (num_blocks > R_STRING_MAX_UNI_BLOCKS) {
+					needle++;
 					continue;
 				}
 			}
@@ -329,7 +371,12 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 			bs->type = str_type;
 			bs->length = runes;
 			bs->size = needle - str_start;
-			bs->ordinal = count++;
+			bs->ordinal = bf->string_count++;
+			if (limit > 0 && bf->string_count > limit) {
+				R_LOG_WARN ("el.limit for strings");
+				R_FREE (bs);
+				break;
+			}
 			// TODO: move into adjust_offset
 			switch (str_type) {
 			case R_STRING_TYPE_WIDE:
@@ -352,21 +399,28 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 			if (!s) {
 				if (section) {
 					s = section;
-				} else if (bf->o) {
-					s = r_bin_get_section_at (bf->o, str_start, false);
+				} else if (bf->bo) {
+					s = r_bin_get_section_at (bf->bo, str_start, false);
 				}
 				if (s) {
 					vdelta = s->vaddr;
 					pdelta = s->paddr;
 				}
 			}
-			bs->paddr = str_start;
-			bs->vaddr = str_start - pdelta + vdelta;
-			bs->string = r_str_ndup ((const char *)tmp, i);
+			ut64 baddr = bf->loadaddr && bf->bo? bf->bo->baddr: bf->loadaddr;
+			bs->paddr = str_start + baddr;
+			bs->vaddr = str_start - pdelta + vdelta + baddr;
+			bs->string = r_strbuf_drain (sb);
+			sb = r_strbuf_new ("");
+			if (strings_nofp) {
+				r_str_trim (bs->string); // trim spaces to ease readability
+			} else {
+				r_str_trim_tail (bs->string);
+			}
 			if (list) {
 				r_list_append (list, bs);
-				if (bf->o) {
-					ht_up_insert (bf->o->strings_db, bs->vaddr, bs);
+				if (bf->bo) {
+					ht_up_insert (bf->bo->strings_db, bs->vaddr, bs);
 				}
 			} else {
 				print_string (bf, bs, raw, pj);
@@ -390,22 +444,22 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		}
 		pj_free (pj);
 	}
-	return count;
+	r_strbuf_free (sb);
+	return bf->string_count;
 }
 
-static bool __isDataSection(RBinFile *a, RBinSection *s) {
+static bool is_data_section(RBinFile *a, RBinSection *s) {
 	if (s->has_strings || s->is_data) {
 		return true;
 	}
  	// Rust
-	return strstr (s->name, "_const") != NULL;
+	return strstr (s->name, "_const");
 }
 
-static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64 from, ut64 to, RBinSection * section) {
-	r_return_if_fail (bf && bf->buf);
+static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, bool nofp, ut64 from, ut64 to, RBinSection *section) {
+	R_RETURN_IF_FAIL (bf && bf->buf);
 
 	RBinPlugin *plugin = r_bin_file_cur_plugin (bf);
-
 	if (!raw && (!plugin || !plugin->info)) {
 		return;
 	}
@@ -421,7 +475,7 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 	}
 	{
 		RIO *io = bf->rbin->iob.io;
-		RCoreBind *cb = &io->corebind;
+		RCoreBind *cb = &io->coreb;
 		if (cb && cb->cfgGet) {
 			const bool cfg_debug = cb->cfgGet (cb->core, "cfg.debug");
 			if (!cfg_debug) {
@@ -439,15 +493,14 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 		// in case of dump ignore here
 		if (bf->rbin->maxstrbuf && size && size > bf->rbin->maxstrbuf) {
 			if (bf->rbin->verbose) {
-				eprintf ("Warning: bin_strings buffer is too big (0x%08" PFMT64x "). Use -zzz or set bin.maxstrbuf (RABIN2_MAXSTRBUF) in r2 (rabin2)\n",
-					size);
+				R_LOG_WARN ("bin_strings buffer is too big (0x%08" PFMT64x "). Use -zzz or set bin.str.maxbuf (RABIN2_MAXSTRBUF) in r2 (rabin2)", size);
 			}
 			return;
 		}
 	}
 	int type;
 	const char *enc = bf->rbin->strenc;
-	if (!enc) {
+	if (enc == NULL) {
 		type = R_STRING_TYPE_DETECT;
 	} else if (!strcmp (enc, "latin1")) {
 		type = R_STRING_TYPE_ASCII;
@@ -458,7 +511,7 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 	} else if (!strcmp (enc, "utf32le")) {
 		type = R_STRING_TYPE_WIDE32;
 	} else { // TODO utf16be, utf32be
-		eprintf ("ERROR: encoding %s not supported\n", enc);
+		R_LOG_ERROR ("encoding %s not supported", enc);
 		return;
 	}
 	string_scan_range (list, bf, min, from, to, type, raw, section);
@@ -478,11 +531,14 @@ R_IPI RBinFile *r_bin_file_new(RBin *bin, const char *file, ut64 file_sz, int ra
 		bf->fd = fd;
 		bf->curxtr = xtrname ? r_bin_get_xtrplugin_by_name (bin, xtrname) : NULL;
 		bf->sdb = sdb;
+		if ((st64)file_sz < 0) {
+			file_sz = 1024 * 64;
+		}
 		bf->size = file_sz;
 		bf->xtr_data = r_list_newf ((RListFree)r_bin_xtrdata_free);
 		bf->xtr_obj = NULL;
 		bf->sdb = sdb_new0 ();
-		bf->sdb_addrinfo = sdb_new0 (); //ns (bf->sdb, "addrinfo", 1);
+		bf->sdb_addrinfo = sdb_new0 (); // ns (bf->sdb, "addrinfo", 1);
 		// bf->sdb_addrinfo->refs++;
 	}
 	return bf;
@@ -505,13 +561,14 @@ static RBinPlugin *get_plugin_from_buffer(RBin *bin, RBinFile *bf, const char *p
 }
 
 R_API bool r_bin_file_object_new_from_xtr_data(RBin *bin, RBinFile *bf, ut64 baseaddr, ut64 loadaddr, RBinXtrData *data) {
-	r_return_val_if_fail (bin && bf && data, false);
+	R_RETURN_VAL_IF_FAIL (bin && bf && data, false);
 
 	ut64 offset = data->offset;
 	ut64 sz = data->size;
 
 	RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, NULL, data->buf);
 	bf->buf = r_buf_ref (data->buf);
+	bf->user_baddr = baseaddr;
 
 	RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, offset, sz);
 	if (!o) {
@@ -526,16 +583,24 @@ R_API bool r_bin_file_object_new_from_xtr_data(RBin *bin, RBinFile *bf, ut64 bas
 	if (!o->info) {
 		o->info = R_NEW0 (RBinInfo);
 	}
-	free (o->info->file);
-	free (o->info->arch);
-	free (o->info->machine);
-	free (o->info->type);
+	R_FREE (o->info->file);
+	R_FREE (o->info->arch);
+	R_FREE (o->info->machine);
+	R_FREE (o->info->type);
 	o->info->file = strdup (bf->file);
-	o->info->arch = strdup (data->metadata->arch);
-	o->info->machine = strdup (data->metadata->machine);
-	o->info->type = strdup (data->metadata->type);
-	o->info->bits = data->metadata->bits;
-	o->info->has_crypto = bf->o->info->has_crypto;
+	if (data->metadata) {
+		if (data->metadata->arch) {
+			o->info->arch = strdup (data->metadata->arch);
+		}
+		if (data->metadata->machine) {
+			o->info->machine = strdup (data->metadata->machine);
+		}
+		if (data->metadata->type) {
+			o->info->type = strdup (data->metadata->type);
+		}
+		o->info->bits = data->metadata->bits;
+	}
+	o->info->has_crypto = bf->bo->info->has_crypto;
 	data->loaded = true;
 	return true;
 }
@@ -550,12 +615,13 @@ static bool xtr_metadata_match(RBinXtrData *xtr_data, const char *arch, int bits
 }
 
 R_IPI RBinFile *r_bin_file_new_from_buffer(RBin *bin, const char *file, RBuffer *buf, int rawstr, ut64 baseaddr, ut64 loadaddr, int fd, const char *pluginname) {
-	r_return_val_if_fail (bin && file && buf, NULL);
+	R_RETURN_VAL_IF_FAIL (bin && file && buf, NULL);
 
 	RBinFile *bf = r_bin_file_new (bin, file, r_buf_size (buf), rawstr, fd, pluginname, NULL, false);
 	if (bf) {
 		RListIter *item = r_list_append (bin->binfiles, bf);
 		bf->buf = r_buf_ref (buf);
+		bf->user_baddr = baseaddr;
 		RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, pluginname, bf->buf);
 		RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, 0, r_buf_size (bf->buf));
 		if (!o) {
@@ -576,7 +642,7 @@ R_API RBinFile *r_bin_file_find_by_arch_bits(RBin *bin, const char *arch, int bi
 	RBinFile *binfile = NULL;
 	RBinXtrData *xtr_data;
 
-	r_return_val_if_fail (bin && arch, NULL);
+	R_RETURN_VAL_IF_FAIL (bin && arch, NULL);
 
 	r_list_foreach (bin->binfiles, iter, binfile) {
 		RListIter *iter_xtr;
@@ -619,7 +685,7 @@ R_API ut64 r_bin_file_delete_all(RBin *bin) {
 }
 
 R_API bool r_bin_file_delete(RBin *bin, ut32 bin_id) {
-	r_return_val_if_fail (bin, false);
+	R_RETURN_VAL_IF_FAIL (bin, false);
 
 	RListIter *iter;
 	RBinFile *bf, *cur = r_bin_cur (bin);
@@ -638,11 +704,11 @@ R_API bool r_bin_file_delete(RBin *bin, ut32 bin_id) {
 }
 
 R_API RBinFile *r_bin_file_find_by_fd(RBin *bin, ut32 bin_fd) {
+	R_RETURN_VAL_IF_FAIL (bin, NULL);
+
 	RListIter *iter;
 	RBinFile *bf;
 
-	r_return_val_if_fail (bin, NULL);
-
 	r_list_foreach (bin->binfiles, iter, bf) {
 		if (bf->fd == bin_fd) {
 			return bf;
@@ -652,11 +718,10 @@ R_API RBinFile *r_bin_file_find_by_fd(RBin *bin, ut32 bin_fd) {
 }
 
 R_API RBinFile *r_bin_file_find_by_name(RBin *bin, const char *name) {
+	R_RETURN_VAL_IF_FAIL (bin && name, NULL);
+
 	RListIter *iter;
 	RBinFile *bf;
-
-	r_return_val_if_fail (bin && name, NULL);
-
 	r_list_foreach (bin->binfiles, iter, bf) {
 		if (bf->file && !strcmp (bf->file, name)) {
 			return bf;
@@ -666,24 +731,26 @@ R_API RBinFile *r_bin_file_find_by_name(RBin *bin, const char *name) {
 }
 
 R_API bool r_bin_file_set_cur_by_id(RBin *bin, ut32 bin_id) {
+	R_RETURN_VAL_IF_FAIL (bin, false);
 	RBinFile *bf = r_bin_file_find_by_id (bin, bin_id);
 	return bf? r_bin_file_set_cur_binfile (bin, bf): false;
 }
 
 R_API bool r_bin_file_set_cur_by_fd(RBin *bin, ut32 bin_fd) {
+	R_RETURN_VAL_IF_FAIL (bin, false);
 	RBinFile *bf = r_bin_file_find_by_fd (bin, bin_fd);
 	return bf? r_bin_file_set_cur_binfile (bin, bf): false;
 }
 
-R_IPI bool r_bin_file_set_obj(RBin *bin, RBinFile *bf, RBinObject *obj) {
-	r_return_val_if_fail (bin && bf, false);
+R_IPI bool r_bin_file_set_obj(RBin *bin, RBinFile *bf, R_NULLABLE RBinObject *obj) {
+	R_RETURN_VAL_IF_FAIL (bin && bf, false);
 	bin->file = bf->file;
 	bin->cur = bf;
 	bin->narch = bf->narch;
 	if (obj) {
-		bf->o = obj;
+		bf->bo = obj;
 	} else {
-		obj = bf->o;
+		obj = bf->bo;
 	}
 	RBinPlugin *plugin = r_bin_file_cur_plugin (bf);
 	if (bin->minstrlen < 1) {
@@ -691,6 +758,7 @@ R_IPI bool r_bin_file_set_obj(RBin *bin, RBinFile *bf, RBinObject *obj) {
 	}
 	if (obj) {
 		if (!obj->info) {
+			R_LOG_DEBUG ("bin object have no information");
 			return false;
 		}
 		if (!obj->info->lang) {
@@ -700,19 +768,24 @@ R_IPI bool r_bin_file_set_obj(RBin *bin, RBinFile *bf, RBinObject *obj) {
 	return true;
 }
 
-R_API bool r_bin_file_set_cur_binfile(RBin *bin, RBinFile *bf) {
-	r_return_val_if_fail (bin && bf, false);
-	return r_bin_file_set_obj (bin, bf, bf->o);
+R_API bool r_bin_file_set_cur_binfile(RBin *bin, R_NULLABLE RBinFile *bf) {
+	// R2_600 - deprecate - because this is unsafe. always use id
+	R_RETURN_VAL_IF_FAIL (bin, false);
+	if (bf == NULL) {
+		bin->cur = NULL;
+		return true;
+	}
+	return r_bin_file_set_obj (bin, bf, bf->bo);
 }
 
 R_API bool r_bin_file_set_cur_by_name(RBin *bin, const char *name) {
-	r_return_val_if_fail (bin && name, false);
+	R_RETURN_VAL_IF_FAIL (bin && name, false);
 	RBinFile *bf = r_bin_file_find_by_name (bin, name);
 	return r_bin_file_set_cur_binfile (bin, bf);
 }
 
 R_API bool r_bin_file_deref(RBin *bin, RBinFile *a) {
-	r_return_val_if_fail (bin && a, false);
+	R_RETURN_VAL_IF_FAIL (bin && a, false);
 	if (!r_bin_cur_object (bin)) {
 		return false;
 	}
@@ -742,7 +815,7 @@ R_API void r_bin_file_free(void /*RBinFile*/ *_bf) {
 		bf->sdb_addrinfo = NULL;
 	}
 	free (bf->file);
-	r_bin_object_free (bf->o);
+	r_bin_object_free (bf->bo);
 	r_list_free (bf->xtr_data);
 	if (bf->id != -1) {
 		// TODO: use r_storage api
@@ -752,12 +825,12 @@ R_API void r_bin_file_free(void /*RBinFile*/ *_bf) {
 	free (bf);
 }
 
-R_IPI RBinFile *r_bin_file_xtr_load_buffer(RBin *bin, RBinXtrPlugin *xtr, const char *filename, RBuffer *buf, ut64 baseaddr, ut64 loadaddr, int idx, int fd, int rawstr) {
-	r_return_val_if_fail (bin && xtr && buf, NULL);
+R_IPI RBinFile *r_bin_file_xtr_load(RBin *bin, RBinXtrPlugin *xtr, const char *filename, RBuffer *buf, ut64 baseaddr, ut64 loadaddr, int idx, int fd, int rawstr) {
+	R_RETURN_VAL_IF_FAIL (bin && xtr && buf, NULL);
 
 	RBinFile *bf = r_bin_file_find_by_name (bin, filename);
 	if (!bf) {
-		bf = r_bin_file_new (bin, filename, r_buf_size (buf), rawstr, fd, xtr->name, bin->sdb, false);
+		bf = r_bin_file_new (bin, filename, r_buf_size (buf), rawstr, fd, xtr->meta.name, bin->sdb, false);
 		if (!bf) {
 			return NULL;
 		}
@@ -773,7 +846,7 @@ R_IPI RBinFile *r_bin_file_xtr_load_buffer(RBin *bin, RBinXtrPlugin *xtr, const
 	} else if (xtr->extractall_from_bytes) {
 		ut64 sz = 0;
 		const ut8 *bytes = r_buf_data (buf, &sz);
-		eprintf ("TODO: Implement extractall_from_buffer in '%s' xtr.bin plugin\n", xtr->name);
+		R_LOG_WARN ("TODO: Implement extractall_from_buffer in '%s' xtr.bin plugin", xtr->meta.name);
 		bf->xtr_data = xtr->extractall_from_bytes (bin, bytes, sz);
 	}
 	if (bf->xtr_data) {
@@ -782,8 +855,12 @@ R_IPI RBinFile *r_bin_file_xtr_load_buffer(RBin *bin, RBinXtrPlugin *xtr, const
 		//populate xtr_data with baddr and laddr that will be used later on
 		//r_bin_file_object_new_from_xtr_data
 		r_list_foreach (bf->xtr_data, iter, x) {
-			x->baddr = baseaddr? baseaddr : UT64_MAX;
-			x->laddr = loadaddr? loadaddr : UT64_MAX;
+			if (x == NULL) {
+				R_LOG_WARN ("Null entry found in xtrdata list");
+			} else {
+				x->baddr = baseaddr? baseaddr : UT64_MAX;
+				x->laddr = loadaddr? loadaddr : UT64_MAX;
+			}
 		}
 	}
 	bf->loadaddr = loadaddr;
@@ -792,38 +869,40 @@ R_IPI RBinFile *r_bin_file_xtr_load_buffer(RBin *bin, RBinXtrPlugin *xtr, const
 
 // XXX deprecate this function imho.. wee can just access bf->buf directly
 R_IPI bool r_bin_file_set_bytes(RBinFile *bf, const ut8 *bytes, ut64 sz, bool steal_ptr) {
-	r_return_val_if_fail (bf && bytes, false);
+	R_RETURN_VAL_IF_FAIL (bf && bytes, false);
 	r_buf_free (bf->buf);
 	if (steal_ptr) {
 		bf->buf = r_buf_new_with_pointers (bytes, sz, true);
 	} else {
 		bf->buf = r_buf_new_with_bytes (bytes, sz);
 	}
-	return bf->buf != NULL;
+	return bf->buf;
 }
 
 R_API RBinPlugin *r_bin_file_cur_plugin(RBinFile *bf) {
-	return (bf && bf->o)? bf->o->plugin: NULL;
+	return (bf && bf->bo)? bf->bo->plugin: NULL;
 }
 
 // TODO: searchStrings() instead
 R_IPI RList *r_bin_file_get_strings(RBinFile *bf, int min, int dump, int raw) {
-	r_return_val_if_fail (bf, NULL);
+	R_RETURN_VAL_IF_FAIL (bf, NULL);
+	RBinObject *o = bf->bo;
+	const bool nofp = bf->rbin->strings_nofp;
 	RListIter *iter;
 	RBinSection *section;
 	RList *ret = dump? NULL: r_list_newf (r_bin_string_free);
 
-	if (!raw && bf && bf->o && bf->o->sections && !r_list_empty (bf->o->sections)) {
-		RBinObject *o = bf->o;
+	bf->string_count = 0;
+	if (!raw && o && o->sections && !r_list_empty (o->sections)) {
 		r_list_foreach (o->sections, iter, section) {
-			if (__isDataSection (bf, section)) {
-				get_strings_range (bf, ret, min, raw, section->paddr,
+			if (is_data_section (bf, section)) {
+				get_strings_range (bf, ret, min, raw, nofp, section->paddr,
 						section->paddr + section->size, section);
 			}
 		}
 		r_list_foreach (o->sections, iter, section) {
 			/* load objc/swift strings */
-			const int bits = (bf->o && bf->o->info) ? bf->o->info->bits : 32;
+			const int bits = (bf->bo && bf->bo->info) ? bf->bo->info->bits : 32;
 			const int cfstr_size = (bits == 64) ? 32 : 16;
 			const int cfstr_offs = (bits == 64) ? 16 :  8;
 			if (strstr (section->name, "__cfstring")) {
@@ -833,6 +912,9 @@ R_IPI RList *r_bin_file_get_strings(RBinFile *bf, int min, int dump, int raw) {
 				if (section->size > bf->size) {
 					continue;
 				}
+				if (section->size < 1) {
+					continue;
+				}
 				ut8 *sbuf = malloc (section->size);
 				if (!sbuf) {
 					continue;
@@ -866,20 +948,20 @@ R_IPI RList *r_bin_file_get_strings(RBinFile *bf, int min, int dump, int raw) {
 			}
 		}
 	} else {
-		get_strings_range (bf, ret, min, raw, 0, bf->size, NULL);
+		get_strings_range (bf, ret, min, raw, nofp, 0, bf->size, NULL);
 	}
 	return ret;
 }
 
 R_API ut64 r_bin_file_get_baddr(RBinFile *bf) {
-	if (bf && bf->o) {
-		return bf->o->baddr;
+	if (bf && bf->bo) {
+		return bf->bo->baddr;
 	}
 	return UT64_MAX;
 }
 
 R_API bool r_bin_file_close(RBin *bin, int bd) {
-	r_return_val_if_fail (bin, false);
+	R_RETURN_VAL_IF_FAIL (bin, false);
 	RBinFile *bf = r_id_storage_take (bin->ids, bd);
 	if (bf) {
 		// file_free removes the fd already.. maybe its unnecessary
@@ -890,11 +972,12 @@ R_API bool r_bin_file_close(RBin *bin, int bd) {
 	return false;
 }
 
+// TODO: do not compute md5 or sha1, those are weak and vulnerable hashes
 R_API RList *r_bin_file_compute_hashes(RBin *bin, ut64 limit) {
-	r_return_val_if_fail (bin && bin->cur && bin->cur->o, NULL);
+	R_RETURN_VAL_IF_FAIL (bin && bin->cur && bin->cur->bo, NULL);
 	ut64 buf_len = 0, r = 0;
 	RBinFile *bf = bin->cur;
-	RBinObject *o = bf->o;
+	RBinObject *o = bf->bo;
 
 	RIODesc *iod = r_io_desc_get (bin->iob.io, bf->fd);
 	if (!iod) {
@@ -905,14 +988,13 @@ R_API RList *r_bin_file_compute_hashes(RBin *bin, ut64 limit) {
 	// By SLURP_LIMIT normally cannot compute ...
 	if (buf_len > limit) {
 		if (bin->verbose) {
-			eprintf ("Warning: r_bin_file_hash: file exceeds bin.hashlimit\n");
+			R_LOG_WARN ("file size exceeds bin.hashlimit");
 		}
 		return NULL;
 	}
 	const size_t blocksize = 64000;
 	ut8 *buf = malloc (blocksize);
 	if (!buf) {
-		eprintf ("Cannot allocate computation buffer\n");
 		return NULL;
 	}
 
@@ -931,7 +1013,7 @@ R_API RList *r_bin_file_compute_hashes(RBin *bin, ut64 limit) {
 		const size_t rem_len = buf_len-r;
 		int b = r_io_desc_read (iod, buf, rem_len);
 		if (b < 1) {
-			eprintf ("r_io_desc_read: error\n");
+			R_LOG_ERROR ("cannot read from descriptor");
 		} else {
 			(void)r_hash_do_md5 (ctx, buf, b);
 			(void)r_hash_do_sha1 (ctx, buf, b);
@@ -981,9 +1063,9 @@ R_API RList *r_bin_file_compute_hashes(RBin *bin, ut64 limit) {
 
 // Set new hashes to current RBinInfo, caller should free the returned RList
 R_API RList *r_bin_file_set_hashes(RBin *bin, RList/*<RBinFileHash*/ *new_hashes) {
-	r_return_val_if_fail (bin && bin->cur && bin->cur->o && bin->cur->o->info, NULL);
+	R_RETURN_VAL_IF_FAIL (bin && bin->cur && bin->cur->bo && bin->cur->bo->info, NULL);
 	RBinFile *bf = bin->cur;
-	RBinInfo *info = bf->o->info;
+	RBinInfo *info = bf->bo->info;
 
 	RList *prev_hashes = info->file_hashes;
 	info->file_hashes = new_hashes;
@@ -991,23 +1073,27 @@ R_API RList *r_bin_file_set_hashes(RBin *bin, RList/*<RBinFileHash*/ *new_hashes
 	return prev_hashes;
 }
 
-R_IPI RBinClass *r_bin_class_new(const char *name, const char *super, int view) {
-	r_return_val_if_fail (name, NULL);
+R_API RBinClass *r_bin_class_new(const char *name, const char *super, ut64 attr) {
+	R_RETURN_VAL_IF_FAIL (name, NULL);
 	RBinClass *c = R_NEW0 (RBinClass);
 	if (c) {
-		c->name = strdup (name);
-		c->super = super? strdup (super): NULL;
-		c->methods = r_list_new ();
-		c->fields = r_list_new ();
-		c->visibility = view;
+		c->name = r_bin_name_new (name);
+		if (R_STR_ISNOTEMPTY (super)) {
+			c->super = r_list_newf (free);
+			r_list_append (c->super, r_bin_name_new (super));
+		}
+		// TODO: use vectors!
+		c->methods = r_list_newf (r_bin_symbol_free);
+		c->fields = r_list_newf (r_bin_field_free);
+		c->attr = attr;
 	}
 	return c;
 }
 
-R_IPI void r_bin_class_free(RBinClass *k) {
+R_API void r_bin_class_free(RBinClass *k) {
 	if (k) {
 		free (k->name);
-		free (k->super);
+		r_list_free (k->super);
 		free (k->visibility_str);
 		r_list_free (k->methods);
 		r_list_free (k->fields);
@@ -1015,50 +1101,55 @@ R_IPI void r_bin_class_free(RBinClass *k) {
 	}
 }
 
-R_API RBinClass *r_bin_file_add_class(RBinFile *bf, const char *name, const char *super, int view) {
-	r_return_val_if_fail (name && bf && bf->o, NULL);
+R_API RBinClass *r_bin_file_add_class(RBinFile *bf, const char *name, const char *super, ut64 attr) {
+	R_RETURN_VAL_IF_FAIL (name && bf && bf->bo, NULL);
 	RBinClass *c = __getClass (bf, name);
 	if (c) {
-		if (super) {
-			free (c->super);
-			c->super = strdup (super);
+		if (R_STR_ISNOTEMPTY (super)) {
+			r_list_free (c->super);
+			c->super = r_list_newf (free);
+			r_list_append (c->super, strdup (super));
 		}
 		return c;
 	}
-	c = r_bin_class_new (name, super, view);
+	c = r_bin_class_new (name, super, attr);
 	if (c) {
 		// XXX. no need for a list, the ht is iterable too
-		c->index = r_list_length (bf->o->classes);
-		r_list_append (bf->o->classes, c);
-		ht_pp_insert (bf->o->classes_ht, name, c);
+		c->index = r_list_length (bf->bo->classes);
+		r_list_append (bf->bo->classes, c);
+		ht_pp_insert (bf->bo->classes_ht, name, c);
 	}
 	return c;
 }
 
 R_API RBinSymbol *r_bin_file_add_method(RBinFile *bf, const char *klass, const char *method, int nargs) {
-	r_return_val_if_fail (bf, NULL);
+	R_RETURN_VAL_IF_FAIL (bf, NULL);
 
 	RBinClass *c = r_bin_file_add_class (bf, klass, NULL, 0);
 	if (!c) {
-		eprintf ("Cannot allocate class %s\n", klass);
+		R_LOG_ERROR ("Cannot allocate class %s", klass);
 		return NULL;
 	}
+	int lang = (strstr (method, "JNI") || strstr (klass, "JNI"))? R_BIN_LANG_JNI: R_BIN_LANG_CXX;
+	c->lang = lang;
 	RBinSymbol *sym = __getMethod (bf, klass, method);
 	if (!sym) {
 		sym = R_NEW0 (RBinSymbol);
 		if (sym) {
-			sym->name = strdup (method);
+			sym->name = r_bin_name_new (method);
+			sym->lang = lang;
+			char *name = r_str_newf ("%s::%s", klass, method);
+			ht_pp_insert (bf->bo->methods_ht, name, sym);
+			// RBinSymbol *dsym = r_bin_symbol_clone (sym);
 			r_list_append (c->methods, sym);
-			const char *name = sdb_fmt ("%s::%s", klass, method);
-			ht_pp_insert (bf->o->methods_ht, name, sym);
+			free (name);
 		}
 	}
 	return sym;
 }
 
 R_API RBinField *r_bin_file_add_field(RBinFile *binfile, const char *classname, const char *name) {
-	//TODO: add_field into class
-	//eprintf ("TODO add field: %s \n", name);
+	R_LOG_TODO ("RBinFile.addField() is not implemented");
 	return NULL;
 }
 
@@ -1066,23 +1157,70 @@ R_API RBinField *r_bin_file_add_field(RBinFile *binfile, const char *classname,
 /* returns vaddr, rebased with the baseaddr of binfile, if va is enabled for
  * bin, paddr otherwise */
 R_API ut64 r_bin_file_get_vaddr(RBinFile *bf, ut64 paddr, ut64 vaddr) {
-	r_return_val_if_fail (bf && bf->o, paddr);
-	if (bf->o->info && bf->o->info->has_va) {
-		return binobj_a2b (bf->o, vaddr);
+	R_RETURN_VAL_IF_FAIL (bf && bf->bo, paddr);
+	if (bf->bo->info && bf->bo->info->has_va) {
+		return bf->bo->baddr_shift + vaddr;
 	}
 	return paddr;
 }
 
 R_API RList *r_bin_file_get_trycatch(RBinFile *bf) {
-	r_return_val_if_fail (bf && bf->o && bf->o->plugin, NULL);
-	if (bf->o->plugin->trycatch) {
-		return bf->o->plugin->trycatch (bf);
+	R_RETURN_VAL_IF_FAIL (bf && bf->bo && bf->bo->plugin, NULL);
+	if (bf->bo->plugin->trycatch) {
+		return bf->bo->plugin->trycatch (bf);
 	}
 	return NULL;
 }
 
+// TODO: Deprecate, we dont want to clone the vec into a list
 R_API RList *r_bin_file_get_symbols(RBinFile *bf) {
-	r_return_val_if_fail (bf, NULL);
-	RBinObject *o = bf->o;
-	return o? o->symbols: NULL;
+	R_RETURN_VAL_IF_FAIL (bf, NULL);
+	RBinObject *bo = bf->bo;
+	if (!bo->symbols) {
+		if (!RVecRBinSymbol_empty (&bo->symbols_vec)) {
+			R_LOG_DEBUG ("cloning symbols vector into a list"); // R2_600
+			RList *list = r_list_newf (NULL);
+			RBinSymbol *s;
+			R_VEC_FOREACH (&bo->symbols_vec, s) {
+				r_list_append (list, s);
+			}
+			bo->symbols = list;
+		}
+	}
+	return bo? bo->symbols: NULL;
+}
+
+R_API RVecRBinSymbol *r_bin_file_get_symbols_vec(RBinFile *bf) {
+	R_RETURN_VAL_IF_FAIL (bf, NULL);
+	RBinObject *bo = bf->bo;
+	if (bo) {
+		if (bo->symbols && RVecRBinSymbol_empty (&bo->symbols_vec)) {
+			R_LOG_DEBUG ("SLOW: cloning symbols list into a vec"); // R2_600
+			RBinSymbol *symbol;
+			// Create a vector for those plugins not loading the rvec
+			RList *list = bo->symbols;
+			RListIter *iter;
+			r_list_foreach (list, iter, symbol) {
+				RVecRBinSymbol_push_back (&bo->symbols_vec, symbol);
+			}
+		}
+		return &bo->symbols_vec;
+	}
+	return NULL;
+}
+
+R_API RBinFile *r_bin_file_open(RBin *bin, const char *file, RBinFileOptions *opt) {
+	if (r_bin_open (bin, file, opt)) {
+		return r_bin_cur (bin);
+	}
+	return NULL;
+}
+
+// TODO Improve this API
+R_API void r_bin_file_merge(RBinFile *dst, RBinFile *src) {
+	// merge imports
+	// merge dbginfo
+	sdb_merge (dst->bo->kv, src->bo->kv);
+	sdb_merge (dst->sdb_addrinfo, src->sdb_addrinfo);
+	sdb_merge (dst->sdb_info, src->sdb_info);
 }
