@@ -7,7 +7,7 @@ Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
-   http://www.apache.org/licenses/LICENSE-2.0
+http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
@@ -30,8 +30,8 @@ limitations under the License.
 #define BYTE uint8_t
 
 typedef struct _IMAGE_DATA_DIRECTORY {
-    DWORD   VirtualAddress;
-    DWORD   Size;
+	DWORD   VirtualAddress;
+	DWORD   Size;
 } IMAGE_DATA_DIRECTORY, *PIMAGE_DATA_DIRECTORY;
 
 #include "r_types.h"
@@ -71,7 +71,7 @@ PIMAGE_DATA_DIRECTORY pe_get_directory_entry( PE* pe, int entry) {
 		? &pe->header64->OptionalHeader.DataDirectory[entry]
 		: &pe->header->OptionalHeader.DataDirectory[entry];
 #else
-	PIMAGE_DATA_DIRECTORY result = { 0 };
+	PIMAGE_DATA_DIRECTORY result = {0};
 #endif
 	return result;
 }
@@ -257,7 +257,7 @@ STREAMS dotnet_parse_stream_headers(
   char stream_name[DOTNET_STREAM_NAME_SIZE + 1];
   unsigned int i;
 
-  memset(&headers, '\0', sizeof(STREAMS));
+  memset(&headers, '\0', sizeof (STREAMS));
 
   stream_header = (PSTREAM_HEADER) (pe->data + offset);
 
@@ -290,24 +290,24 @@ STREAMS dotnet_parse_stream_headers(
     // Store necessary bits to parse these later. Not all tables will be
     // parsed, but are referenced from others. For example, the #Strings
     // stream is referenced from various tables in the #~ heap.
-    if (strncmp(stream_name, "#GUID", 5) == 0)
+    if (strncmp (stream_name, "#GUID", 5))
       headers.guid = stream_header;
     // Believe it or not, I have seen at least one binary which has a #- stream
     // instead of a #~ (215e1b54ae1aac153e55596e6f1a4350). This isn't in the
     // documentation anywhere but the structure is the same. I'm chosing not
     // to parse it for now.
-    else if (strncmp(stream_name, "#~", 2) == 0 && headers.tilde == NULL)
+    else if (strncmp (stream_name, "#~", 2) == 0 && headers.tilde == NULL)
       headers.tilde = stream_header;
-    else if (strncmp(stream_name, "#Strings", 8) == 0 && headers.string == NULL)
+    else if (strncmp (stream_name, "#Strings", 8) == 0 && headers.string == NULL)
       headers.string = stream_header;
-    else if (strncmp(stream_name, "#Blob", 5) == 0)
+    else if (strncmp (stream_name, "#Blob", 5) == 0)
       headers.blob = stream_header;
-    else if (strncmp(stream_name, "#US", 3) == 0 && headers.us == NULL)
+    else if (strncmp (stream_name, "#US", 3) == 0 && headers.us == NULL)
       headers.us = stream_header;
 
     // Stream name is padded to a multiple of 4.
     stream_header = (PSTREAM_HEADER) ((uint8_t*) stream_header +
-        sizeof(STREAM_HEADER) +
+        sizeof (STREAM_HEADER) +
         strlen(stream_name) +
         4 - (strlen(stream_name) % 4));
   }
@@ -410,7 +410,7 @@ void dotnet_parse_tilde_2(
 
   row_offset = (uint32_t*) (tilde_header + 1);
   table_offset = (uint8_t*) row_offset;
-  table_offset += sizeof(uint32_t) * valid_rows;
+  table_offset += sizeof (uint32_t) * valid_rows;
 
 #define DOTNET_STRING_INDEX(Name) \
   index_sizes.string == 2 ? Name.Name_Short : Name.Name_Long
@@ -451,7 +451,7 @@ void dotnet_parse_tilde_2(
             string_offset,
             DOTNET_STRING_INDEX(module_table->Name));
 
-        if (name != NULL)
+        if (name)
           set_string(name, pe->object, "module_name");
 
         table_offset += (
@@ -670,7 +670,7 @@ void dotnet_parse_tilde_2(
 
         row_size = (index_size + index_size2 + index_sizes.blob);
 
-        if (typeref_ptr != NULL && memberref_ptr != NULL)
+        if (typeref_ptr && memberref_ptr)
         {
           row_ptr = table_offset;
 
@@ -801,7 +801,7 @@ void dotnet_parse_tilde_2(
                   pe, string_offset, *(WORD*) typeref_row);
             }
 
-            if (name != NULL && strncmp(name, "GuidAttribute", 13) != 0)
+            if (name && strncmp (name, "GuidAttribute", 13) != 0)
             {
               row_ptr += row_size;
               continue;
@@ -1002,7 +1002,7 @@ void dotnet_parse_tilde_2(
               string_offset,
               DOTNET_STRING_INDEX(moduleref_table->Name));
 
-          if (name != NULL)
+          if (name)
           {
             set_string(name, pe->object, "modulerefs[%i]", counter);
             counter++;
@@ -1085,7 +1085,7 @@ void dotnet_parse_tilde_2(
                   row_ptr + 4 + 2 + 2 + 2 + 2 + 4 +
                   index_sizes.blob));
 
-        if (name != NULL)
+        if (name)
           set_string(name, pe->object, "assembly.name");
 
         // Culture comes after Name.
@@ -1112,7 +1112,7 @@ void dotnet_parse_tilde_2(
 
         // Sometimes it will be a zero length string. This is technically
         // against the specification but happens from time to time.
-        if (name != NULL && strlen(name) > 0)
+        if (name && strlen(name) > 0)
           set_string(name, pe->object, "assembly.culture");
 
         table_offset += row_size * num_rows;
@@ -1185,7 +1185,7 @@ void dotnet_parse_tilde_2(
                 string_offset,
                 *(WORD*) (row_ptr + 2 + 2 + 2 + 2 + 4 + index_sizes.blob));
 
-          if (name != NULL)
+          if (name)
             set_string(name, pe->object, "assembly_refs[%i].name", i);
 
           row_ptr += row_size;
@@ -1259,7 +1259,7 @@ void dotnet_parse_tilde_2(
           if (!fits_in_pe(
                 pe,
                 pe->data + resource_base + resource_offset,
-                sizeof(DWORD)))
+                sizeof (DWORD)))
           {
             row_ptr += row_size;
             continue;
@@ -1287,7 +1287,7 @@ void dotnet_parse_tilde_2(
               string_offset,
               DOTNET_STRING_INDEX(manifestresource_table->Name));
 
-          if (name != NULL)
+          if (name)
             set_string(name, pe->object, "resources[%i].name", counter);
 
           row_ptr += row_size;
@@ -1377,10 +1377,10 @@ void dotnet_parse_tilde(
 
   // Default all rows to 0. They will be set to actual values later on, if
   // they exist in the file.
-  memset(&rows, '\0', sizeof(ROWS));
+  memset(&rows, '\0', sizeof (ROWS));
 
   // Default index sizes are 2. Will be bumped to 4 if necessary.
-  memset(&index_sizes, 2, sizeof(index_sizes));
+  memset(&index_sizes, 2, sizeof (index_sizes));
 
   tilde_header = (PTILDE_HEADER) (
       pe->data +
@@ -1414,7 +1414,7 @@ void dotnet_parse_tilde(
       continue;
 
 #define ROW_CHECK(name) \
-    if (fits_in_pe(pe, row_offset, (matched_bits + 1) * sizeof(uint32_t))) \
+    if (fits_in_pe(pe, row_offset, (matched_bits + 1) * sizeof (uint32_t))) \
       rows.name = *(row_offset + matched_bits);
 
 #define ROW_CHECK_WITH_INDEX(name) \
@@ -1564,7 +1564,7 @@ void dotnet_parse_com(PE* pe, ut64 baddr) {
 	// 4. We need to exclude the terminator and the padding, so search for the
 	// first NULL byte.
 	end = (char*) memmem((void*) metadata->Version, metadata->Length, "\0", 1);
-	if (end != NULL)
+	if (end)
 		set_sized_string(metadata->Version,
 				(end - metadata->Version),
 				pe->object,
@@ -1574,7 +1574,7 @@ void dotnet_parse_com(PE* pe, ut64 baddr) {
 	// We must manually parse things from here on out.
 	//
 	// Flags are 2 bytes (always 0).
-	offset += sizeof(NET_METADATA) + metadata->Length + 2;
+	offset += sizeof (NET_METADATA) + metadata->Length + 2;
 
 	// 2 bytes for Streams.
 	if (!fits_in_pe(pe, pe->data + offset, 2))
