@@ -1,16 +1,13 @@
-/* radare - LGPL - 2015-2019 - a0rtega */
+/* radare - LGPL - 2015-2023 - a0rtega */
 
-#include <r_types.h>
-#include <r_util.h>
 #include <r_lib.h>
 #include <r_bin.h>
-#include <string.h>
 
 #include "../format/nin/nds.h"
 
-static struct nds_hdr loaded_header;
+static R_TH_LOCAL struct nds_hdr loaded_header = {0};
 
-static bool check_buffer(RBinFile *bf, RBuffer *b) {
+static bool check(RBinFile *bf, RBuffer *b) {
 	ut8 ninlogohead[6];
 	if (r_buf_read_at (b, 0xc0, ninlogohead, sizeof (ninlogohead)) == 6) {
 		/* begin of nintendo logo =    \x24\xff\xae\x51\x69\x9a */
@@ -18,27 +15,23 @@ static bool check_buffer(RBinFile *bf, RBuffer *b) {
 			return true;
 		}
 		/* begin of Homebrew magic */
-		if (!memcmp (ninlogohead, "\xC8\x60\x4F\xE2\x01\x70", 6)){
+		if (!memcmp (ninlogohead, "\xC8\x60\x4F\xE2\x01\x70", 6)) {
 			return true;
 		}
 	}
 	return false;
 }
 
-static bool load_buffer (RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
+static bool load(RBinFile *bf, RBuffer *b, ut64 loadaddr) {
 	r_buf_read_at (b, 0, (ut8*)&loaded_header, sizeof (loaded_header));
-	*bin_obj = &loaded_header;
-	return (*bin_obj != NULL);
+	bf->bo->bin_obj = &loaded_header;
+	return bf->bo->bin_obj != NULL;
 }
 
 static ut64 baddr(RBinFile *bf) {
 	return (ut64) loaded_header.arm9_ram_address;
 }
 
-static ut64 boffset(RBinFile *bf) {
-	return 0LL;
-}
-
 static RList *sections(RBinFile *bf) {
 	RList *ret = NULL;
 	RBinSection *ptr9 = NULL, *ptr7 = NULL;
@@ -110,7 +103,7 @@ static RList *entries(RBinFile *bf) {
 }
 
 static RBinInfo *info(RBinFile *bf) {
-	r_return_val_if_fail (bf && bf->buf, NULL);
+	R_RETURN_VAL_IF_FAIL (bf && bf->buf, NULL);
 	RBinInfo *ret = R_NEW0 (RBinInfo);
 	if (ret) {
 		char *filepath = r_str_newf ("%.12s - %.4s",
@@ -127,13 +120,14 @@ static RBinInfo *info(RBinFile *bf) {
 }
 
 RBinPlugin r_bin_plugin_ninds = {
-	.name = "ninds",
-	.desc = "Nintendo DS format r_bin plugin",
-	.license = "LGPL3",
-	.load_buffer = &load_buffer,
-	.check_buffer = &check_buffer,
+	.meta = {
+		.name = "ninds",
+		.desc = "Nintendo DS format r_bin plugin",
+		.license = "LGPL3",
+	},
+	.load = &load,
+	.check = &check,
 	.baddr = &baddr,
-	.boffset = &boffset,
 	.entries = &entries,
 	.sections = &sections,
 	.info = &info,
