@@ -30,15 +30,15 @@ static ut64 baddr(RBinFile *bf) {
 	return bf? r_buf_read_le32_at (bf->buf, NRO_OFFSET_MODMEMOFF): 0;
 }
 
-static bool check_buffer(RBinFile *bf, RBuffer *b) {
+static bool check(RBinFile *bf, RBuffer *b) {
 	ut8 magic[4];
 	if (r_buf_read_at (b, NRO_OFF (magic), magic, sizeof (magic)) == 4) {
-		return fileType (magic) != NULL;
+		return fileType (magic);
 	}
 	return false;
 }
 
-static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
+static bool load(RBinFile *bf, RBuffer *b, ut64 loadaddr) {
 	// XX bf->buf vs b :D this load_b
 	RBinNXOObj *bin = R_NEW0 (RBinNXOObj);
 	if (bin) {
@@ -48,7 +48,7 @@ static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr,
 		bin->classes_list = r_list_newf ((RListFree)free);
 		ut32 mod0 = r_buf_read_le32_at (b, NRO_OFFSET_MODMEMOFF);
 		parseMod (b, bin, mod0, ba);
-		*bin_obj = bin;
+		bf->bo->bin_obj = bin;
 	}
 	return true;
 }
@@ -88,7 +88,7 @@ static RList *sections(RBinFile *bf) {
 	RList *ret = NULL;
 	RBinSection *ptr = NULL;
 	RBuffer *b = bf->buf;
-	if (!bf->o->info) {
+	if (!bf->bo->info) {
 		return NULL;
 	}
 	if (!(ret = r_list_new ())) {
@@ -127,7 +127,7 @@ static RList *sections(RBinFile *bf) {
 		ptr->add = false;
 		r_list_append (ret, ptr);
 	} else {
-		eprintf ("Invalid MOD0 address\n");
+		R_LOG_ERROR ("Invalid MOD0 address");
 	}
 
 	ut32 sig0 = r_buf_read_le32_at (bf->buf, 0x18);
@@ -145,7 +145,7 @@ static RList *sections(RBinFile *bf) {
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	} else {
-		eprintf ("Invalid SIG0 address\n");
+		R_LOG_ERROR ("Invalid SIG0 address");
 	}
 
 	// add text segment
@@ -194,19 +194,19 @@ static RList *sections(RBinFile *bf) {
 
 static RList *symbols(RBinFile *bf) {
 	RBinNXOObj *bin;
-	if (!bf || !bf->o || !bf->o->bin_obj) {
+	if (!bf || !bf->bo || !bf->bo->bin_obj) {
 		return NULL;
 	}
-	bin = (RBinNXOObj*) bf->o->bin_obj;
+	bin = (RBinNXOObj*) bf->bo->bin_obj;
 	return bin->methods_list;
 }
 
 static RList *imports(RBinFile *bf) {
 	RBinNXOObj *bin;
-	if (!bf || !bf->o || !bf->o->bin_obj) {
+	if (!bf || !bf->bo || !bf->bo->bin_obj) {
 		return NULL;
 	}
-	bin = (RBinNXOObj*) bf->o->bin_obj;
+	bin = (RBinNXOObj*) bf->bo->bin_obj;
 	return bin->imports_list;
 }
 
@@ -253,11 +253,13 @@ static RBinInfo *info(RBinFile *bf) {
 #if !R_BIN_NRO
 
 RBinPlugin r_bin_plugin_nro = {
-	.name = "nro",
-	.desc = "Nintendo Switch NRO0 binaries",
-	.license = "MIT",
-	.load_buffer = &load_buffer,
-	.check_buffer = &check_buffer,
+	.meta = {
+		.name = "nro",
+		.desc = "Nintendo Switch NRO0 binaries",
+		.license = "MIT",
+	},
+	.load = &load,
+	.check = &check,
 	.baddr = &baddr,
 	.binsym = &binsym,
 	.entries = &entries,
