@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014-2019 - pancake */
+/* radare - LGPL - Copyright 2014-2022 - pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -35,7 +35,7 @@ typedef struct  {
 	ut8 uuid[16];
 }) PebbleAppInfo;
 
-static bool check_buffer(RBinFile *bf, RBuffer *b) {
+static bool check(RBinFile *bf, RBuffer *b) {
 	ut8 magic[8];
 	if (r_buf_read_at (b, 0, magic, sizeof (magic)) != sizeof (magic)) {
 		return false;
@@ -43,8 +43,8 @@ static bool check_buffer(RBinFile *bf, RBuffer *b) {
 	return !memcmp (magic, "PBLAPP\x00\x00", 8);
 }
 
-static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb){
-	return check_buffer (bf, b);
+static bool load(RBinFile *bf, RBuffer *b, ut64 loadaddr) {
+	return check (bf, b);
 }
 
 static ut64 baddr(RBinFile *bf) {
@@ -62,7 +62,7 @@ static RBinInfo* info(RBinFile *bf) {
 	memset (&pai, 0, sizeof (pai));
 	int reat = r_buf_read_at (bf->buf, 0, (ut8*)&pai, sizeof (pai));
 	if (reat != sizeof (pai)) {
-		eprintf ("Truncated Header\n");
+		R_LOG_ERROR ("Truncated Header");
 		return NULL;
 	}
 	if (!(ret = R_NEW0 (RBinInfo))) {
@@ -89,8 +89,8 @@ static RList* sections(RBinFile *bf) {
 	RList *ret = NULL;
 	RBinSection *ptr = NULL;
 	PebbleAppInfo pai = {{0}};
-	if (!r_buf_read_at (bf->buf, 0, (ut8*)&pai, sizeof(pai))) {
-		eprintf ("Truncated Header\n");
+	if (!r_buf_read_at (bf->buf, 0, (ut8*)&pai, sizeof (pai))) {
+		R_LOG_ERROR ("Truncated Header");
 		return NULL;
 	}
 	if (!(ret = r_list_new ())) {
@@ -102,13 +102,21 @@ static RList* sections(RBinFile *bf) {
 		return ret;
 	}
 	ptr->name = strdup ("relocs");
-	ptr->vsize = ptr->size = pai.num_reloc_entries * sizeof (ut32);
-	ptr->vaddr = ptr->paddr = pai.reloc_list_start;
-	ptr->perm = R_PERM_RW;
-	ptr->add = true;
-	r_list_append (ret, ptr);
-	if (ptr->vaddr < textsize) {
-		textsize = ptr->vaddr;
+	ut64 sz = pai.num_reloc_entries * sizeof (ut32);
+	ut64 ss = pai.reloc_list_start;
+	if (ss < r_buf_size (bf->buf)) {
+		if (ss + sz >= r_buf_size (bf->buf)) {
+			ut64 left = r_buf_size (bf->buf) - ss;
+			sz = left;
+		}
+		ptr->vaddr = ptr->paddr = ss;
+		ptr->vsize = ptr->size = sz;
+		ptr->perm = R_PERM_RWX;
+		ptr->add = true;
+		r_list_append (ret, ptr);
+		if (ptr->vaddr < textsize) {
+			textsize = ptr->vaddr;
+		}
 	}
 
 	// imho this must be a symbol
@@ -166,8 +174,8 @@ static RList* entries(RBinFile *bf) {
 	RBinAddr *ptr = NULL;
 	RList *ret;
 	PebbleAppInfo pai;
-	if (!r_buf_read_at (bf->buf, 0, (ut8*)&pai, sizeof(pai))) {
-		eprintf ("Truncated Header\n");
+	if (!r_buf_read_at (bf->buf, 0, (ut8*)&pai, sizeof (pai))) {
+		R_LOG_ERROR ("Truncated Header");
 		return NULL;
 	}
 	if (!(ret = r_list_new ())) {
@@ -184,11 +192,13 @@ static RList* entries(RBinFile *bf) {
 }
 
 RBinPlugin r_bin_plugin_pebble = {
-	.name = "pebble",
-	.desc = "Pebble Watch App",
-	.license = "LGPL",
-	.load_buffer = &load_buffer,
-	.check_buffer = &check_buffer,
+	.meta = {
+		.name = "pebble",
+		.desc = "Pebble Watch App",
+		.license = "LGPL",
+	},
+	.load = &load,
+	.check = &check,
 	.baddr = &baddr,
 	.entries = entries,
 	.sections = sections,
