@@ -33,7 +33,7 @@ static Sdb *get_sdb(RBinFile *bf) {
 }
 
 static bool check(RBinFile *bf, RBuffer *b) {
-	r_return_val_if_fail (b, false);
+	R_RETURN_VAL_IF_FAIL (b, false);
 	ut8 sig[4];
 	if (r_buf_read_at (b, 0, sig, sizeof (sig)) != 4) {
 		return false;
@@ -46,7 +46,7 @@ static bool check(RBinFile *bf, RBuffer *b) {
 
 static bool load(RBinFile *bf, RBuffer *buf, ut64 loadaddr) {
 	WadObj *wo = R_NEW0 (WadObj);
-	r_return_val_if_fail (wo, false);
+	R_RETURN_VAL_IF_FAIL (wo, false);
 #if 0
 	// we can just use bf->sdb in here
 	wad_header_load (wo, bf->sdb);
@@ -63,7 +63,7 @@ static bool load(RBinFile *bf, RBuffer *buf, ut64 loadaddr) {
 }
 
 static RBinInfo *info(RBinFile *bf) {
-	r_return_val_if_fail (bf, NULL);
+	R_RETURN_VAL_IF_FAIL (bf, NULL);
 	RBinInfo *ret = R_NEW0 (RBinInfo);
 	if (!ret) {
 		return NULL;
