@@ -1,11 +1,11 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2024 - pancake, nibble */
 
 #include <r_cons.h>
 
-static bool gethtmlrgb(const char *str, char *buf) {
+static bool gethtmlrgb(const char *str, char *buf, size_t buf_size) {
 	ut8 r = 0, g = 0, b = 0;
 	if (r_cons_rgb_parse (str, &r, &g, &b, 0)) {
-		sprintf (buf, "#%02x%02x%02x", r, g, b);
+		snprintf (buf, buf_size, "#%02x%02x%02x", r, g, b);
 		return true;
 	}
 	buf[0] = '\0';
@@ -32,10 +32,10 @@ static const char *gethtmlcolor(const char ptrch) {
 R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 	const char *str = ptr;
 	int esc = 0;
-	int len = 0;
 	bool inv = false;
 	char text_color[16] = {0};
 	char background_color[16] = {0};
+	bool has_bold = false;
 	bool has_set = false;
 	bool need_to_set = false;
 	bool need_to_clear = false;
@@ -48,7 +48,7 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 	if (!res) {
 		return NULL;
 	}
-	for (; ptr[0]; ptr = ptr + 1) {
+	for (; ptr[0]; ptr++) {
 		if (esc == 0 && ptr[0] != 0x1b && need_to_set) {
 			if (has_set) {
 				r_strbuf_append (res, "</font>");
@@ -70,6 +70,11 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 					r_strbuf_append (res, "text-decoration:underline overline");
 					first_style = false;
 				}
+				if (has_bold) {
+					r_strbuf_append (res, first_style? " style='": ";");
+					r_strbuf_append (res, "font-weight:bold");
+					first_style = false;
+				}
 				r_strbuf_append (res, first_style? ">": "'>");
 				has_set = true;
 			}
@@ -77,15 +82,19 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 			need_to_set = false;
 		}
 		if (ptr[0] == '\n') {
-			tmp = (int) (size_t) (ptr - str);
-			r_strbuf_append_n (res, str, tmp);
-			if (!ptr[1]) {
-				// write new line if it's the end of the output
-				r_strbuf_append (res, "\n");
+			if (ptr > str) {
+				tmp = (int) (size_t) (ptr - str);
+				r_strbuf_append_n (res, str, tmp);
+				if (!ptr[1]) {
+					// write new line if it's the end of the output
+					r_strbuf_append (res, "\n");
+				} else {
+					r_strbuf_append (res, "<br />");
+				}
+				str = ptr + 1;
 			} else {
-				r_strbuf_append (res, "<br />");
+				r_strbuf_append (res, "<br />\n");
 			}
-			str = ptr + 1;
 			continue;
 		} else if (ptr[0] == '<') {
 			tmp = (int)(size_t) (ptr - str);
@@ -101,9 +110,13 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 			continue;
 		} else if (ptr[0] == ' ') {
 			tmp = (int) (size_t) (ptr - str);
-			r_strbuf_append_n (res, str, tmp);
+			if (tmp > 0) {
+				r_strbuf_append_n (res, str, tmp);
+				str = ptr + 1;
+			} else {
+				str++;
+			}
 			r_strbuf_append (res, "&nbsp;");
-			str = ptr + 1;
 			continue;
 		}
 		if (ptr[0] == 0x1b) {
@@ -116,19 +129,23 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 		if (esc == 1) {
 			// \x1b[2J
 			if (ptr[0] != '[') {
-				eprintf ("Oops invalid escape char\n");
+				R_LOG_ERROR ("Oops invalid escape char");
 				esc = 0;
 				str = ptr + 1;
 				continue;
 			}
 			esc = 2;
 			continue;
-		} else if (esc == 2) {
+		}
+		if (esc == 2) {
 			// TODO: use dword comparison here
 			if (ptr[0] == '0' && ptr[1] == 'J') { // R_CONS_CLEAR_FROM_CURSOR_TO_END
 				ptr += 2;
 				esc = 0;
 				str = ptr;
+			} else if (ptr[0] == '1') {
+				// ignore bold
+				has_bold = true;
 			} else if (!memcmp (ptr, "2K", 2)) {
 				ptr += 2;
 				esc = 0;
@@ -140,29 +157,50 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 				esc = 0;
 				str = ptr;
 				continue;
-			} else if (!strncmp (ptr, "48;5;", 5) || !strncmp (ptr, "48;2;", 5)) {
+			} else if (IS_DIGIT (ptr[0]) && ptr[1] == ';' && IS_DIGIT (ptr[2])) {
+				char *m = strchr (ptr, 'm');
+				if (m) {
+					gethtmlrgb (ptr, background_color, sizeof (background_color));
+					need_to_set = true;
+					ptr = m;
+					str = ptr + 1;
+					esc = 0;
+				}
+			} else if (IS_DIGIT (ptr[0]) && IS_DIGIT (ptr[1]) && ptr[2] == ';') {
+				char *m = strchr (ptr, 'm');
+				if (m) {
+					gethtmlrgb (ptr, text_color, sizeof (text_color));
+					need_to_set = true;
+					ptr = m;
+					str = ptr + 1;
+					esc = 0;
+				}
+			} else if (r_str_startswith (ptr, "48;5;") || r_str_startswith (ptr, "48;2;")) {
 				char *end = strchr (ptr, 'm');
-				gethtmlrgb (ptr, background_color);
+				gethtmlrgb (ptr, background_color, sizeof (background_color));
 				need_to_set = true;
 				ptr = end;
 				str = ptr + 1;
 				esc = 0;
-			} else if (!strncmp (ptr, "38;5;", 5) || !strncmp (ptr, "38;2;", 5)) {
+			} else if (r_str_startswith (ptr, "38;5;") || r_str_startswith (ptr, "38;2;")) {
 				char *end = strchr (ptr, 'm');
-				gethtmlrgb (ptr, text_color);
+				gethtmlrgb (ptr, text_color, sizeof (text_color));
 				need_to_set = true;
-				ptr = end;
-				str = ptr + 1;
+				if (end) {
+					ptr = end;
+					str = ptr + 1;
+				}
 				esc = 0;
-			} else if (ptr[0] == '0' && ptr[1] == ';' && ptr[2] == '0') {
-				// wtf ?
+			} else if ((ptr[0] == '0' || ptr[0] == '1') && ptr[1] == ';' && IS_DIGIT (ptr[2])) {
+				// bg color is kind of ignored, but no glitch so far
 				r_cons_gotoxy (0, 0);
 				ptr += 4;
 				esc = 0;
 				str = ptr;
 				continue;
 			} else if (ptr[0] == '0' && ptr[1] == 'm') {
-				str = (++ptr) + 1;
+				ptr++;
+				str = ptr + 1;
 				esc = 0;
 				inv = false;
 				text_color[0] = '\0';
@@ -170,7 +208,7 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 				need_to_set = need_to_clear = true;
 				continue;
 				// reset color
-			} else if (!strncmp (ptr, "27m", 3)) {
+			} else if (r_str_startswith (ptr, "27m")) {
 				inv = false;
 				need_to_set = true;
 				ptr = ptr + 2;
@@ -179,7 +217,8 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 				continue;
 				// reset invert color
 			} else if (ptr[0] == '7' && ptr[1] == 'm') {
-				str = (++ptr) + 1;
+				ptr++;
+				str = ptr + 1;
 				inv = true;
 				need_to_set = true;
 				esc = 0;
@@ -195,7 +234,7 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 				str = ptr + 1;
 				esc = 0;
 				continue;
-			} else if (ptr[0] == '4' && ptr[2] == 'm') {
+			} else if ((ptr[0] == '4' || ptr[0] == '9') && ptr[2] == 'm') {
 				const char *htmlColor = gethtmlcolor (ptr[1]);
 				if (htmlColor) {
 					r_str_ncpy (background_color, htmlColor, sizeof (background_color));
@@ -207,9 +246,10 @@ R_API char *r_cons_html_filter(const char *ptr, int *newlen) {
 				continue;
 			}
 		}
-		len++;
 	}
-	r_strbuf_append_n (res, str, ptr - str);
+	if (ptr > str) {
+		r_strbuf_append_n (res, str, ptr - str);
+	}
 	if (has_set) {
 		r_strbuf_append (res, "</font>");
 	}
