@@ -3,7 +3,7 @@
 #include <r_util.h>
 #include <r_cons.h>
 
-static RLine r_line_instance;
+static R_TH_LOCAL RLine r_line_instance;
 #define I r_line_instance
 
 R_API RLine *r_line_singleton(void) {
@@ -17,16 +17,13 @@ R_API RLine *r_line_new(void) {
 	I.contents = NULL;
 	I.enable_vi_mode = false;
 	I.clipboard = NULL;
-	I.kill_ring = r_list_newf (NULL);
+	I.kill_ring = r_list_newf (free);
 	I.kill_ring_ptr = -1;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	I.vtmode = r_cons_is_vtcompat ();
 #else
 	I.vtmode = 2;
 #endif
-	if (!r_line_dietline_init ()) {
-		eprintf ("error: r_line_dietline_init\n");
-	}
 	r_line_completion_init (&I.completion, 4096);
 	return &I;
 }
@@ -40,7 +37,7 @@ R_API void r_line_free(void) {
 	r_line_completion_fini (&I.completion);
 }
 
-R_API void r_line_clipboard_push (const char *str) {
+R_API void r_line_clipboard_push(const char *str) {
 	I.kill_ring_ptr += 1;
 	r_list_insert (I.kill_ring, I.kill_ring_ptr, strdup (str));
 }
@@ -70,37 +67,40 @@ R_API void r_line_completion_fini(RLineCompletion *completion) {
 }
 
 R_API void r_line_completion_push(RLineCompletion *completion, const char *str) {
-	r_return_if_fail (completion && str);
+	R_RETURN_IF_FAIL (completion && str);
 	if (completion->quit) {
-	        return;
+		return;
 	}
-	if (r_pvector_len (&completion->args) < completion->args_limit) {
+	if (r_pvector_length (&completion->args) < completion->args_limit) {
 		char *s = strdup (str);
 		if (s) {
 			r_pvector_push (&completion->args, (void *)s);
 		}
 	} else {
-	        completion->quit = true;
-	        eprintf ("Warning: Maximum completion capacity reached, increase scr.maxtab");
+		completion->quit = true;
+		R_LOG_WARN ("Maximum completion capacity reached, increase scr.maxtab");
 	}
 }
 
 R_API void r_line_completion_set(RLineCompletion *completion, int argc, const char **argv) {
-	r_return_if_fail (completion && (argc >= 0));
+	R_RETURN_IF_FAIL (completion && (argc >= 0));
 	r_line_completion_clear (completion);
 	if (argc > completion->args_limit) {
-                eprintf ("Warning: Maximum completion capacity reached, increase scr.maxtab");
+		argc = completion->args_limit;
+		R_LOG_DEBUG ("Maximum completion capacity reached, increase scr.maxtab (%d %d)",
+				argc, completion->args_limit);
 	}
 	size_t count = R_MIN (argc, completion->args_limit);
-	r_pvector_reserve (&completion->args, count);
-	int i;
-	for (i = 0; i < count; i++) {
-		r_line_completion_push (completion, argv[i]);
+	if (r_pvector_reserve (&completion->args, count)) {
+		int i;
+		for (i = 0; i < count; i++) {
+			r_line_completion_push (completion, argv[i]);
+		}
 	}
 }
 
 R_API void r_line_completion_clear(RLineCompletion *completion) {
-	r_return_if_fail (completion);
+	R_RETURN_IF_FAIL (completion);
 	completion->quit = false;
 	r_pvector_clear (&completion->args);
 }
