@@ -27,7 +27,7 @@ static RCoreHelpMessage help_msg_w = {
 	"wp", "[?] -|file", "apply radare patch file. See wp? fmi",
 	"wr", " 10", "write 10 random bytes",
 	"ws", "[?] pstring", "write pascal string: 1 byte for length + N for the string",
-	"wt", "[afs][?] [filename] [size]", "write to file (from current seek, blocksize or sz bytes)",
+	"wt", "[?][afs] [filename] [size]", "write to file (from current seek, blocksize or sz bytes)",
 	"ww", " foobar", "write wide string 'f\\x00o\\x00o\\x00b\\x00a\\x00r\\x00'",
 	"wx", "[?][fs] 9090", "write two intel nops (from wxfile or wxseek)",
 	"wX", " 1b2c3d", "fill current block with cyclic hexpairs",
@@ -209,7 +209,7 @@ static void cmd_write_fail(RCore *core) {
 }
 
 R_API int cmd_write_hexpair(RCore* core, const char* pairs) {
-	r_return_val_if_fail (core && pairs, 0);
+	R_RETURN_VAL_IF_FAIL (core && pairs, 0);
 	ut8 *buf = malloc (strlen (pairs) + 1);
 	if (!buf) {
 		return 0;
@@ -1020,7 +1020,7 @@ static int cmd_w6(void *data, const char *input) {
 }
 
 static int cmd_wh(void *data, const char *input) {
-	r_return_val_if_fail (data && input, -1);
+	R_RETURN_VAL_IF_FAIL (data && input, -1);
 	char *space = strchr (input, ' ');
 	const char *arg = space? r_str_trim_head_ro (space): NULL;
 	if (arg) {
@@ -1653,157 +1653,157 @@ static int cmd_wt(RCore *core, const char *input) {
 
 	input++;
 	switch (*input) {
-		case 's': { // "wts"
-			ut64 addr = 0;
-			char *host_port;
-			R_BORROW char *host;
-			R_BORROW char *port;
-			ut8 *buf;
-			RSocket *sock;
+	case 's': { // "wts"
+		ut64 addr = 0;
+		char *host_port;
+		R_BORROW char *host;
+		R_BORROW char *port;
+		ut8 *buf;
+		RSocket *sock;
 
-			if (argc < 2) {
-				r_core_cmd_help_match (core, help_msg_wt, "wts");
-				ret = 1;
-				goto leave;
-			}
+		if (argc < 2) {
+			r_core_cmd_help_match (core, help_msg_wt, "wts");
+			ret = 1;
+			goto leave;
+		}
 
-			sz = r_io_size (core->io);
+		sz = r_io_size (core->io);
+		if (sz < 0) {
+			R_LOG_ERROR ("Unknown file size");
+			ret = 1;
+			goto leave;
+		}
+
+		host_port = strdup (argv[1]);
+
+		host = host_port;
+		port = strchr (host_port, ':');
+		if (!port) {
+			r_core_cmd_help_match (core, help_msg_wt, "wts");
+			free (host_port);
+			ret = 1;
+			goto leave;
+		}
+
+		*port++ = 0;
+
+		if (argc > 2) {
+			sz = r_num_math (core->num, argv[2]);
 			if (sz < 0) {
-				R_LOG_ERROR ("Unknown file size");
-				ret = 1;
-				goto leave;
-			}
-
-			host_port = strdup (argv[1]);
-
-			host = host_port;
-			port = strchr (host_port, ':');
-			if (!port) {
-				r_core_cmd_help_match (core, help_msg_wt, "wts");
+				R_LOG_ERROR ("%s is not a valid size", argv[2]);
 				free (host_port);
 				ret = 1;
 				goto leave;
 			}
-
-			*port++ = 0;
-
-			if (argc > 2) {
-				sz = r_num_math (core->num, argv[2]);
-				if (sz < 0) {
-					R_LOG_ERROR ("%s is not a valid size", argv[2]);
-					free (host_port);
-					ret = 1;
-					goto leave;
-				}
-				addr = core->offset;
-			}
-
-			buf = malloc (sz);
-			r_io_read_at (core->io, addr, buf, sz);
-
-			sock = r_socket_new (false);
-			if (r_socket_connect (sock, host, port, R_SOCKET_PROTO_TCP, 0)) {
-				ut64 sent = 0;
-				R_LOG_INFO ("Connection created. Sending data to TCP socket");
-				while (sent < sz) {
-					bool sockret = r_socket_write (sock, buf + sent, sz - sent);
-					if (!sockret) {
-						R_LOG_ERROR ("Socket write error");
-						ret = 1;
-						break;
-					}
-				}
-			} else {
-				R_LOG_ERROR ("Connection to %s failed", host_port);
-				ret = 1;
-			}
-
-			free (host_port);
-			free (buf);
-			r_socket_free (sock);
-			goto leave;
+			addr = core->offset;
 		}
-		case 'f': // "wtf"
-			switch (input[1]) {
-			case '\0':
-			case '?': // "wtf?"
+
+		buf = malloc (sz);
+		r_io_read_at (core->io, addr, buf, sz);
+
+		sock = r_socket_new (false);
+		if (r_socket_connect (sock, host, port, R_SOCKET_PROTO_TCP, 0)) {
+			ut64 sent = 0;
+			R_LOG_INFO ("Connection created. Sending data to TCP socket");
+			while (sent < sz) {
+				bool sockret = r_socket_write (sock, buf + sent, sz - sent);
+				if (!sockret) {
+					R_LOG_ERROR ("Socket write error");
+					ret = 1;
+					break;
+				}
+			}
+		} else {
+			R_LOG_ERROR ("Connection to %s failed", host_port);
+			ret = 1;
+		}
+
+		free (host_port);
+		free (buf);
+		r_socket_free (sock);
+		goto leave;
+	}
+	case 'f': // "wtf"
+		switch (input[1]) {
+		case '\0':
+		case '?': // "wtf?"
+			r_core_cmd_help_match (core, help_msg_wt, "wtf");
+			ret = 1;
+			goto leave;
+		case '!': { // "wtf!"
+			RIOMap *map;
+			if (input[2] == '?') {
+				r_core_cmd_help_match (core, help_msg_wt, "wtf!");
+				ret = 1;
+				goto leave;
+			}
+
+			map = r_io_map_get_at (core->io, poff);
+			if (map) {
+				// convert vaddr to paddr
+				poff = poff - r_io_map_begin (map) + map->delta;
+			}
+
+			sz = r_io_fd_size (core->io, core->io->desc->fd) - core->offset;
+
+			// ignore given size
+			if (argc > 2) {
+				argc = 2;
+			}
+			break;
+		}
+		case 'f': // "wtff"
+			if (input[2] == '?') {
+				r_core_cmd_help_match (core, help_msg_wt, "wtff");
+				ret = 1;
+				goto leave;
+			}
+
+			if (argc > 1) {
+				prefix = argv[1];
+			}
+
+			default_filename_sep = '-';
+			break;
+		default: // "wtf"
+			if (input[2] == '?') {
 				r_core_cmd_help_match (core, help_msg_wt, "wtf");
 				ret = 1;
 				goto leave;
-			case '!': { // "wtf!"
-				RIOMap *map;
-				if (input[2] == '?') {
-					r_core_cmd_help_match (core, help_msg_wt, "wtf!");
-					ret = 1;
-					goto leave;
-				}
-
-				map = r_io_map_get_at (core->io, poff);
-				if (map) {
-					// convert vaddr to paddr
-					poff = poff - r_io_map_begin (map) + map->delta;
-				}
-
-				sz = r_io_fd_size (core->io, core->io->desc->fd) - core->offset;
-
-				// ignore given size
-				if (argc > 2) {
-					argc = 2;
-				}
-				break;
 			}
-			case 'f': // "wtff"
-				if (input[2] == '?') {
-					r_core_cmd_help_match (core, help_msg_wt, "wtff");
+
+			if (r_str_startswith (filename, "base64:")) {
+				const char *encoded = filename + 7;
+				int len;
+				if (strlen (encoded) > 31) {
+					R_LOG_ERROR ("Base64 blob must be fewer than 32 characters");
 					ret = 1;
 					goto leave;
 				}
 
-				if (argc > 1) {
-					prefix = argv[1];
-				}
+				len = r_base64_decode ((ut8 *)fn_local, encoded, -1);
 
-				default_filename_sep = '-';
-				break;
-			default: // "wtf"
-				if (input[2] == '?') {
-					r_core_cmd_help_match (core, help_msg_wt, "wtf");
+				filename = fn_local;
+
+				if (len < 0) {
+					R_LOG_ERROR ("Couldn't decode b64 filename");
 					ret = 1;
 					goto leave;
 				}
-
-				if (r_str_startswith (filename, "base64:")) {
-					const char *encoded = filename + 7;
-					int len;
-					if (strlen (encoded) > 31) {
-						R_LOG_ERROR ("Base64 blob must be fewer than 32 characters");
-						ret = 1;
-						goto leave;
-					}
-
-					len = r_base64_decode ((ut8 *)fn_local, encoded, -1);
-
-					filename = fn_local;
-
-					if (len < 0) {
-						R_LOG_ERROR ("Couldn't decode b64 filename");
-						ret = 1;
-						goto leave;
-					}
-				}
-				break;
 			}
 			break;
-		case 'a':
-			append = true;
-			break;
-		case '\0': // "wt"
-		case ' ': // "wt "
-			break;
-		case '?': // "wt?"
-		default:
-			r_core_cmd_help (core, help_msg_wt);
-			goto leave;
+		}
+		break;
+	case 'a':
+		append = true;
+		break;
+	case '\0': // "wt"
+	case ' ': // "wt "
+		break;
+	case '?': // "wt?"
+	default:
+		r_core_cmd_help (core, help_msg_wt);
+		goto leave;
 	}
 
 	// default filename is prefix.addr
