@@ -113,6 +113,29 @@ static bool addGraphZign(RCore *core, const char *name, RList *args) {
 	return r_sign_add_graph (core->anal, name, graph);
 }
 
+// R2_600 - move to libr/anal/sign.c as public api
+static RSignItem *item_new_named(RAnal *a, const char *n) {
+	RSignItem *it = r_sign_item_new ();
+	if (it && (it->name = strdup (n))) {
+		it->space = r_spaces_current (&a->zign_spaces);
+		return it;
+	}
+	r_sign_item_free (it);
+	return NULL;
+}
+static bool r_sign_add_next(RAnal *a, const char *name, const char *nextname) {
+	R_RETURN_VAL_IF_FAIL (a && name && nextname, false);
+
+	bool retval = false;
+	RSignItem *it = item_new_named (a, name);
+	if (it) {
+		it->next = strdup (nextname);
+		retval = r_sign_add_item (a, it);
+		r_sign_item_free (it);
+	}
+	return retval;
+}
+
 static inline bool za_add(RCore *core, const char *input) {
 	char *args = r_str_trim_dup (input + 1);
 	if (!args) {
@@ -173,8 +196,12 @@ static inline bool za_add(RCore *core, const char *input) {
 	case R_SIGN_BBHASH:
 		ret = r_sign_add_hash (core->anal, name, t, sig, strlen (sig));
 		break;
+	case R_SIGN_NEXT:
+		// TODO
+		ret = r_sign_add_next(core->anal, name, sig);
+		break;
 	default:
-		R_LOG_ERROR ("unknown zignature type");
+		R_LOG_ERROR ("unknown zignature type: %s", stype);
 	}
 	r_list_free (lst);
 	free (args);
@@ -279,7 +306,7 @@ static int cmdAdd(void *data, const char *input) {
 			R_LOG_INFO ("generated zignatures: %d", count);
 		}
 		break;
-	case '?':
+	case '?': // “za?”
 		r_core_cmd_help (core, help_msg_za);
 		break;
 	default:
@@ -294,25 +321,25 @@ static int cmdOpen(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 
 	switch (*input) {
-	case ' ':
+	case ' ': // "zo"
 		if (input[1]) {
 			return r_sign_load (core->anal, r_str_trim_head_ro (input + 1), false);
 		}
 		r_core_cmd_help_contains (core, help_msg_zo, "zo");
 		return false;
-	case 's':
+	case 's': // "zos"
 		if (input[1] == ' ' && input[2]) {
 			return r_sign_save (core->anal, r_str_trim_head_ro (input + 2));
 		}
 		r_core_cmd_help_contains (core, help_msg_zo, "zos");
 		return false;
-	case 'z':
+	case 'z': // "zoz"
 		if (input[1] == ' ' && input[2]) {
 			return r_sign_load_gz (core->anal, input + 2, false);
 		}
 		r_core_cmd_help_contains (core, help_msg_zo, "zoz");
 		return false;
-	case '?':
+	case '?': // "zo?"
 		r_core_cmd_help (core, help_msg_zo);
 		break;
 	default:
@@ -327,7 +354,7 @@ static int cmdSpace(void *data, const char *input) {
 	RSpaces *zs = &core->anal->zign_spaces;
 
 	switch (*input) {
-	case '+':
+	case '+': // "zs+"
 		if (!input[1]) {
 			r_core_cmd_help_contains (core, help_msg_zs, "zs+");
 			return false;
@@ -338,14 +365,14 @@ static int cmdSpace(void *data, const char *input) {
 			free (sp);
 		}
 		break;
-	case 'r':
+	case 'r': // "zsr"
 		if (input[1] != ' ' || !input[2]) {
 			r_core_cmd_help_contains (core, help_msg_zs, "zsr");
 			return false;
 		}
 		r_spaces_rename (zs, NULL, input + 2);
 		break;
-	case '-':
+	case '-': // "zs-"
 		if (input[1] == '\x00') {
 			r_spaces_pop (zs);
 		} else if (input[1] == '*') {
@@ -354,19 +381,19 @@ static int cmdSpace(void *data, const char *input) {
 			r_spaces_unset (zs, input + 1);
 		}
 		break;
-	case 'j':
-	case '*':
+	case 'j': // "zsj"
+	case '*': // "zs*"
 	case '\0':
-		spaces_list (zs, input[0]);
+		spaces_list (core, zs, input[0]);
 		break;
-	case ' ':
+	case ' ': // "zs"
 		if (!input[1]) {
 			r_core_cmd_help (core, help_msg_zs);
 			return false;
 		}
 		r_spaces_set (zs, input + 1);
 		break;
-	case '?':
+	case '?': // "zs?"
 		r_core_cmd_help (core, help_msg_zs);
 		break;
 	default:
@@ -381,7 +408,7 @@ static int cmdFlirt(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 
 	switch (*input) {
-	case 'd':
+	case 'd': // "zfd"
 		// TODO
 		if (input[1] != ' ') {
 			r_core_cmd_help_contains (core, help_msg_zf, "zfd");
@@ -389,7 +416,7 @@ static int cmdFlirt(void *data, const char *input) {
 		}
 		r_sign_flirt_dump (core->anal, input + 2);
 		break;
-	case 's':
+	case 's': // "zfs"
 		// TODO
 		if (input[1] != ' ') {
 			r_core_cmd_help_contains (core, help_msg_zf, "zfs");
@@ -404,10 +431,10 @@ static int cmdFlirt(void *data, const char *input) {
 		}
 		r_list_free (files);
 		break;
-	case 'z':
+	case 'z': // "zfz"
 		// TODO
 		break;
-	case '?':
+	case '?': // "zf?"
 		r_core_cmd_help (core, help_msg_zf);
 		break;
 	default:
@@ -434,7 +461,7 @@ struct ctxSearchCB {
 };
 
 static void apply_name(RCore *core, RAnalFunction *fcn, RSignItem *it, bool rad) {
-	r_return_if_fail (core && fcn && it && it->name);
+	R_RETURN_IF_FAIL (core && fcn && it && it->name);
 	const char *name = it->realname? it->realname: it->name;
 	if (rad) {
 		char *tmp = r_name_filter_dup (name);
@@ -499,7 +526,7 @@ static int searchBytesHitCB(RSignItem *it, RSearchKeyword *kw, ut64 addr, void *
 }
 
 static int fcnMatchCB(RSignItem *it, RAnalFunction *fcn, RSignType *types, void *user, RList *col) {
-	r_return_val_if_fail (types && *types != R_SIGN_END, 1);
+	R_RETURN_VAL_IF_FAIL (types && *types != R_SIGN_END, 1);
 	struct ctxSearchCB *ctx = (struct ctxSearchCB *)user;
 	ut64 sz = r_anal_function_realsize (fcn);
 	RSignType t;
@@ -612,7 +639,7 @@ static bool searchRange(RCore *core, ut64 from, ut64 to, bool rad, struct ctxSea
 
 static void search_add_to_types(RCore *c, RSignSearchMetrics *sm, RSignType t, const char *str, unsigned int *i) {
 	unsigned int count = *i;
-	r_return_if_fail (count < sizeof (sm->stypes) / sizeof (RSignType) - 1);
+	R_RETURN_IF_FAIL (count < sizeof (sm->stypes) / sizeof (RSignType) - 1);
 	if (r_config_get_i (c->config, str)) {
 		sm->stypes[count++] = t;
 		sm->stypes[count] = 0;
@@ -813,9 +840,9 @@ static double get_zb_threshold(RCore *core) {
 }
 
 static bool bestmatch_fcn(RCore *core, const char *input, bool json) {
-	r_return_val_if_fail (input && core, false);
+	R_RETURN_VAL_IF_FAIL (input && core, false);
 
-	char *argv = r_str_new (input);
+	char *argv = strdup (input);
 	if (!argv) {
 		return false;
 	}
@@ -871,7 +898,7 @@ static bool bestmatch_fcn(RCore *core, const char *input, bool json) {
 }
 
 static bool bestmatch_sig(RCore *core, const char *input, bool json) {
-	r_return_val_if_fail (input && core, false);
+	R_RETURN_VAL_IF_FAIL (input && core, false);
 	int count = 5;
 	if (!R_STR_ISEMPTY (input)) {
 		count = atoi (input);
@@ -924,26 +951,26 @@ static bool bestmatch_sig(RCore *core, const char *input, bool json) {
 }
 
 static bool bestmatch(void *data, const char *input) {
-	r_return_val_if_fail (data && input, false);
+	R_RETURN_VAL_IF_FAIL (data && input, false);
 	bool json = false;
 	RCore *core = (RCore *)data;
 	switch (input[0]) {
-	case 'r':
+	case 'r': // "zbr"
 		input++;
 		if (*input == 'j') {
 			input++;
 			json = true;
 		}
 		return bestmatch_fcn (core, input, json);
-	case 'j':
+	case 'j': // "zbj"
 		json = true;
 		/* fallthrough */
-	case ' ':
+	case ' ': // "zb"
 		input++;
 		/* fallthrough */
 	case '\x00':
 		return bestmatch_sig (core, input, json);
-	case '?':
+	case '?': // "zb?"
 	default:
 		r_core_cmd_help (core, help_msg_zb);
 		return false;
@@ -1063,7 +1090,7 @@ static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops
 #undef freelines
 
 static bool diff_zig(void *data, const char *input) {
-	r_return_val_if_fail (data && input, false);
+	R_RETURN_VAL_IF_FAIL (data && input, false);
 	RCore *core = (RCore *)data;
 
 	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
@@ -1072,7 +1099,7 @@ static bool diff_zig(void *data, const char *input) {
 		return false;
 	}
 
-	char *argv = r_str_new (input);
+	char *argv = strdup (input);
 	if (!argv) {
 		return false;
 	}
@@ -1141,7 +1168,7 @@ static int cmdCompare(void *data, const char *input) {
 	RSignOptions *options = r_sign_options_new (raw_bytes_thresh, raw_graph_thresh);
 
 	switch (*input) {
-	case ' ':
+	case ' ': // "zc"
 		if (!input[1]) {
 			r_core_cmd_help (core, help_msg_zc);
 			result = false;
@@ -1151,7 +1178,7 @@ static int cmdCompare(void *data, const char *input) {
 		break;
 	case 'n':
 		switch (input[1]) {
-		case ' ':
+		case ' ': // "zcn"
 			if (!input[2]) {
 				r_core_cmd_help_contains (core, help_msg_zc, "zcn");
 				result = false;
@@ -1159,7 +1186,7 @@ static int cmdCompare(void *data, const char *input) {
 			}
 			result = r_sign_diff_by_name (core->anal, options, input + 2, false);
 			break;
-		case '!':
+		case '!': // "zcn!"
 			if (input[2] != ' ' || !input[3]) {
 				r_core_cmd_help_contains (core, help_msg_zc, "zcn");
 				result = false;
@@ -1172,7 +1199,7 @@ static int cmdCompare(void *data, const char *input) {
 			result = false;
 		}
 		break;
-	case '?':
+	case '?': // "zc?"
 		r_core_cmd_help (core, help_msg_zc);
 		break;
 	default:
@@ -1247,7 +1274,7 @@ static int cmdSearch(void *data, const char *input) {
 			r_core_cmd_help_contains (core, help_msg_z_slash, "z/f");
 			return false;
 		}
-	case '?':
+	case '?': // "z/?"
 		r_core_cmd_help (core, help_msg_z_slash);
 		break;
 	default:
@@ -1319,7 +1346,7 @@ static int cmd_zign(void *data, const char *input) {
 
 	switch (*input) {
 	case '\0':
-	case ' ':
+	case ' ': // "z"
 	case '*': // "z*"
 	case 'q': // "zq"
 	case 'j': // "zj"
@@ -1337,7 +1364,7 @@ static int cmd_zign(void *data, const char *input) {
 			core->offset = oaddr;
 		}
 		break;
-	case ',':
+	case ',': // "z,"
 		return csvZignatures (core, arg);
 	case 'k': // "zk"
 		r_core_cmd0 (core, "k anal/zigns/*");
