@@ -1,16 +1,17 @@
-/* radare - Copyright 2014-2021 pancake, defragger */
+/* radare - Copyright 2014-2023 pancake, defragger */
+
+#define R_LOG_ORIGIN "a2f"
 
-#include <r_types.h>
 #include <r_core.h>
-#include <r_io.h>
 
 #define MAXFCNSIZE 4096
 
-#define Fbb(x) sdb_fmt("bb.%"PFMT64x,x)
-#define Fhandled(x) sdb_fmt("handled.%"PFMT64x,x)
-#define FbbTo(x) sdb_fmt("bb.%"PFMT64x".to",x)
+#define Fbb(x) r_strf("bb.%"PFMT64x,x)
+#define Fhandled(x) r_strf("handled.%"PFMT64x,x)
+#define FbbTo(x) r_strf("bb.%"PFMT64x".to",x)
 
 static ut64 getCrossingBlock(Sdb *db, const char *key, ut64 start, ut64 end) {
+	r_strf_buffer (64);
 	ut64 block_start, block_end;
 	ut64 nearest_start = UT64_MAX;
 	const char *s = sdb_const_get (db, key, NULL);
@@ -53,6 +54,7 @@ static ut64 getCrossingBlock(Sdb *db, const char *key, ut64 start, ut64 end) {
 */
 
 static int bbAdd(Sdb *db, ut64 from, ut64 to, ut64 jump, ut64 fail) {
+	r_strf_buffer (64);
 	ut64 block_start = getCrossingBlock (db, "bbs", from, to);
 	bool add = true;
 	if (block_start == UT64_MAX) {
@@ -73,10 +75,12 @@ static int bbAdd(Sdb *db, ut64 from, ut64 to, ut64 jump, ut64 fail) {
 		if (from > block_start) {
 			// from inside
 			// RESIZE this
-			sdb_num_set (db, Fbb (block_start), from, 0);
-			sdb_num_set (db, FbbTo (block_start), from, 0);
-			sdb_array_set_num (db, FbbTo (block_start), 0, from, 0);
-			sdb_array_set_num (db, FbbTo (block_start), 1, UT64_MAX, 0);
+			r_strf_var (bbst, 64, "bb.%"PFMT64x, block_start);
+			r_strf_var (bben, 64, "bb.%"PFMT64x".to", block_start);
+			sdb_num_set (db, bbst, from, 0);
+			sdb_num_set (db, bben, from, 0);
+			sdb_array_set_num (db, bben, 0, from, 0);
+			sdb_array_set_num (db, bben, 1, UT64_MAX, 0);
 		} else {
 			// < the current runs into a known block
 			to = block_start;
@@ -96,18 +100,19 @@ static int bbAdd(Sdb *db, ut64 from, ut64 to, ut64 jump, ut64 fail) {
 }
 
 void addTarget(RCore *core, RStack *stack, Sdb *db, ut64 addr) {
+	r_strf_buffer (64);
 	if (sdb_num_get (db, Fhandled (addr), NULL)) {
 		// already set
 		return;
 	}
 	ut64* value = (ut64*) calloc (1, sizeof (ut64));
 	if (!value) {
-		eprintf ("Failed to allocate memory for address stack\n");
+		R_LOG_DEBUG ("Failed to allocate memory for address stack");
 		return;
 	}
 	*value = addr;
 	if (!r_stack_push (stack, (void*)value)) {
-		eprintf ("Failed to push address on stack\n");
+		R_LOG_DEBUG ("Failed to push address on stack");
 		free (value);
 		return;
 	}
@@ -120,7 +125,8 @@ static ut64 analyzeStackBased(RCore *core, Sdb *db, ut64 addr, RList *delayed_co
 #define addUjmp(x) sdb_array_add_num (db, "ujmps", x, 0);
 #define addCjmp(x) sdb_array_add_num (db, "cjmps", x, 0);
 #define addRet(x) sdb_array_add_num (db, "rets", x, 0);
-#define bbAddOpcode(x) sdb_array_insert_num (db, sdb_fmt ("bb.%"PFMT64x, addr+cur), -1, x, 0);
+#define bbAddOpcode(x) sdb_array_insert_num (db, r_strf ("bb.%"PFMT64x, addr+cur), -1, x, 0);
+	r_strf_buffer (32);
 	ut64 oaddr = addr;
 	ut64 *value = NULL;
 	RAnalOp *op;
@@ -134,7 +140,7 @@ static ut64 analyzeStackBased(RCore *core, Sdb *db, ut64 addr, RList *delayed_co
 		block_end = false;
 		value = (ut64*) r_stack_pop (stack);
 		if (!value) {
-			eprintf ("Failed to pop next address from stack\n");
+			R_LOG_ERROR ("Failed to pop next address from stack");
 			break;
 		}
 
@@ -142,14 +148,14 @@ static ut64 analyzeStackBased(RCore *core, Sdb *db, ut64 addr, RList *delayed_co
 		free (value);
 		cur = 0;
 		while (!block_end && cur < maxfcnsize) {
-			op = r_core_anal_op (core, addr + cur, R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_DISASM);
+			op = r_core_anal_op (core, addr + cur, R_ARCH_OP_MASK_BASIC | R_ARCH_OP_MASK_DISASM);
 			if (!op || !op->mnemonic) {
-				eprintf ("a2f: Cannot analyze opcode at 0x%"PFMT64x"\n", addr+cur);
+				R_LOG_DEBUG ("a2f: Cannot analyze opcode at 0x%"PFMT64x, addr+cur);
 				oaddr = UT64_MAX;
 				break;
 			}
 			if (op->mnemonic[0] == '?') {
-				eprintf ("a2f: Cannot analyze opcode at 0x%"PFMT64x"\n", addr+cur);
+				R_LOG_DEBUG ("a2f: Cannot analyze opcode at 0x%"PFMT64x, addr+cur);
 				oaddr = UT64_MAX;
 				break;
 			}
@@ -233,7 +239,7 @@ static ut64 analyzeStackBased(RCore *core, Sdb *db, ut64 addr, RList *delayed_co
 				break;
 			case R_ANAL_OP_TYPE_UNK:
 			case R_ANAL_OP_TYPE_ILL:
-				eprintf ("a2f: Invalid instruction\n");
+				R_LOG_DEBUG ("a2f: Invalid instruction");
 				block_end = true;
 				break;
 			default:
@@ -255,6 +261,7 @@ static ut64 analyzeStackBased(RCore *core, Sdb *db, ut64 addr, RList *delayed_co
 }
 
 static ut64 getFunctionSize(Sdb *db) {
+	r_strf_buffer (64);
 	ut64 min = UT64_MAX, max = 0;
 	char *c, *bbs = sdb_get (db, "bbs", NULL);
 	bool first = true;
@@ -280,29 +287,28 @@ static ut64 getFunctionSize(Sdb *db) {
 }
 
 static bool analyzeFunction(RCore *core, ut64 addr) {
+	r_strf_buffer (64);
 	Sdb *db = sdb_new0 ();
 	RFlagItem *fi;
-	RList *delayed_commands = NULL;
 	RListIter *iter;
 	ut64 loc_addr = 0;
 	char *command = NULL;
 	char *function_label;
 	bool vars = r_config_get_b (core->config, "anal.vars");
 	if (!db) {
-		eprintf ("Cannot create db\n");
+		R_LOG_ERROR ("Cannot create db");
 		return false;
 	}
 
-	delayed_commands = r_list_newf (free);
+	RList *delayed_commands = r_list_newf (free);
 	if (!delayed_commands) {
-		eprintf ("Failed to initialize the delayed command list\n");
 		sdb_free (db);
 		return false;
 	}
 
 	ut64 a = analyzeStackBased (core, db, addr, delayed_commands);
 	if (addr == UT64_MAX) {
-		eprintf ("Initial analysis failed\n");
+		R_LOG_ERROR ("Initial analysis failed");
 		return false;
 	}
 	if (a != UT64_MAX) {
@@ -354,7 +360,6 @@ static bool analyzeFunction(RCore *core, ut64 addr) {
 			  	loc_addr, addr, (int)(addr_end - addr), jump, fail);
 			sdb_aforeach_next (c);
 		}
-
 		if (vars) {
 			// handling arguments
 			RAnalFunction *fcn = r_anal_get_function_at (core->anal, addr);
@@ -390,16 +395,25 @@ static bool analyzeFunction(RCore *core, ut64 addr) {
 
 static int r_cmd_anal_call(void *user, const char *input) {
 	RCore *core = (RCore *) user;
+	static RCoreHelpMessage help_msg_a2f = {
+		"Usage:", "a2f", "Experimental function analysis",
+		"a2f", "", "like af, but with an experimental engine. see anal.a2f",
+		NULL
+	};
 	if (!strncmp (input, "a2", 2)) {
 		switch (input[2]) {
 		case 'f':
+			if (input[3] == '?') {
+				r_core_cmd_help (core, help_msg_a2f);
+				return true;
+			}
+
 			if (!analyzeFunction (core, core->offset)) {
-				eprintf ("a2f: Failed to analyze function at 0x%08"PFMT64x".\n", core->offset);
+				R_LOG_DEBUG ("a2f: Failed to analyze function at 0x%08"PFMT64x, core->offset);
 			}
 			break;
 		default:
-			eprintf ("Usage: a2f @ address_of_function          # See anal.a2f\n");
-			eprintf ("a2f is an experimental analysis engine that replaces af.\n");
+			r_core_cmd_help (core, help_msg_a2f);
 			break;
 		}
 		return true;
@@ -409,9 +423,12 @@ static int r_cmd_anal_call(void *user, const char *input) {
 
 // PLUGIN Definition Info
 RCorePlugin r_core_plugin_a2f = {
-	.name = "a2f",
-	.desc = "The reworked analysis from scratch thing",
-	.license = "LGPL3",
+	.meta = {
+		.name = "a2f",
+		.desc = "The reworked analysis from scratch thing",
+		.author = "pancake",
+		.license = "LGPL3",
+	},
 	.call = r_cmd_anal_call,
 };
 
