@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2019 - pancake */
+/* radare - LGPL - Copyright 2011-2023 - pancake */
 
 #include <r_core.h>
 
@@ -8,9 +8,7 @@ R_API bool r_core_patch_line(RCore *core, char *str) {
 		return false;
 	}
 	*p = 0;
-	for (++p; *p == ' '; p++) {
-		; // XXX: skipsspaces here
-	}
+	p = (char *)r_str_trim_head_ro (p + 1);
 
 	switch (*p) {
 	case '"':
@@ -18,29 +16,27 @@ R_API bool r_core_patch_line(RCore *core, char *str) {
 		if (q) {
 			*q = 0;
 		}
-		r_core_cmdf (core, "s %s", str);
-		r_core_cmdf (core, "\"w %s\"", p+1);
+		r_core_cmdf (core, "\"\"s %s", str);
+		r_core_cmdf (core, "\"\"w %s", p+1);
 		break;
 	case ':':
-		r_core_cmdf (core, "s %s", str);
-		r_core_cmdf (core, "\"wa %s\"", p);
+		r_core_cmdf (core, "\"\"s %s", str);
+		r_core_cmdf (core, "\"\"wa %s", p);
 		break;
 	case 'v':
-		q = strchr (p + 1,' ');
+		q = strchr (p + 1, ' ');
 		if (q) {
 			*q = 0;
-			for (++q; *q == ' '; q++) {
-				; // XXX: skipsspaces here
-			}
+			q = (char *)r_str_trim_head_ro (q + 1);
 		} else {
 			return 0;
 		}
-		r_core_cmdf (core, "s %s", str);
-		r_core_cmdf (core, "wv%s %s", p + 1, q);
+		r_core_cmdf (core, "\"\"s %s", str);
+		r_core_cmdf (core, "\"\"wv%s %s", p + 1, q);
 		break;
 	default:
-		r_core_cmdf (core, "s %s", str);
-		r_core_cmdf (core, "wx %s", p);
+		r_core_cmdf (core, "\"\"s %s", str);
+		r_core_cmdf (core, "\"\"wx %s", p);
 		break;
 	}
 	return true;
@@ -69,14 +65,14 @@ static bool __core_patch_bracket(RCore *core, const char *str, ut64 *noff) {
 			break;
 		}
 		if ((q = strstr (str, "${"))) {
-			char *end = strchr (q+2,'}');
+			char *end = strchr (q + 2,'}');
 			if (end) {
 				*q = *end = 0;
-				*noff = r_num_math (core->num, q+2);
+				*noff = r_num_math (core->num, q + 2);
 				r_buf_append_bytes (b, (const ut8*)str, strlen (str));
 				snprintf (tmp, sizeof (tmp), "0x%08"PFMT64x, *noff);
 				r_buf_append_bytes (b, (const ut8*)tmp, strlen (tmp));
-				r_buf_append_bytes (b, (const ut8*)end+1, strlen (end+1));
+				r_buf_append_bytes (b, (const ut8*)end + 1, strlen (end + 1));
 			}
 		} else {
 			r_buf_append_bytes (b, (const ut8*)str, strlen (str));
@@ -84,7 +80,7 @@ static bool __core_patch_bracket(RCore *core, const char *str, ut64 *noff) {
 		str = p;
 	}
 
-	s = r_buf_to_string (b);
+	s = r_buf_tostring (b);
 	r_egg_load (core->egg, s, 0);
 	free (s);
 
@@ -116,7 +112,7 @@ R_API int r_core_patch(RCore *core, const char *patch) {
 	for (; *p; p++) {
 		/* read until newline */
 		if (!*p || *p == '\n') {
-			*p++ = 0; 
+			*p++ = 0;
 		} else {
 			continue;
 		}
@@ -128,7 +124,7 @@ R_API int r_core_patch(RCore *core, const char *patch) {
 		case '\0':
 			break;
 		case ':':
-			r_core_cmd0 (core, str+1);
+			r_core_cmd0 (core, str + 1);
 			break;
 		case '.':
 		case '!':
@@ -146,8 +142,6 @@ R_API int r_core_patch(RCore *core, const char *patch) {
 		}
 		str = p;
 	}
-//	eprintf ("%d\n", *p);
-//	eprintf ("Missing newline\n");
 	free (p0);
 	return 0;
 }
