@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2015-2021 - pancake */
+/* radare - LGPL - Copyright 2015-2024 - pancake */
 
 #include <r_core.h>
 #define TYPE_NONE 0
@@ -8,6 +8,8 @@
 #define IS_STRING(x,y) ((x)+3<end && *(x) == 's' && *((x)+1) == 't' && *((x)+2) == 'r' && *((x)+3) == '.')
 #define IS_SYMBOL(x,y) ((x)+3<end && *(x) == 's' && *((x)+1) == 'y' && *((x)+2) == 'm' && *((x)+3) == '.')
 
+// R2R db/cmd/cmd_pdc
+
 typedef struct _find_ctx {
 	char *comment;
 	char *left;
@@ -28,9 +30,8 @@ static void find_and_change(char* in, int len) {
 	if (!in || len < 1) {
 		return;
 	}
-	char *end;
 	RFindCTX ctx = {0};
-	end = in + len;
+	char *end = in + len;
 //	type = TYPE_NONE;
 	for (ctx.linebegin = in; in < end; in++) {
 		if (*in == '\n' || !*in) {
@@ -162,7 +163,16 @@ static void find_and_change(char* in, int len) {
 	}
 }
 
-static const char *help_msg_pdc[] = {
+#if 0
+static int cmpnbbs(const void *_a, const void *_b) {
+	const RAnalBlock *a = _a, *b = _b;
+	ut64 as = a->addr;
+	ut64 bs = b->addr;
+	return (as> bs)? -1: (as< bs)? 1: 0;
+}
+#endif
+
+static RCoreHelpMessage help_msg_pdc = {
 	"Usage: pdc[oj]", "", "experimental, unreliable and hacky pseudo-decompiler",
 	"pdc", "", "pseudo decompile function in current offset",
 	"pdcc", "", "pseudo-decompile with C helpers around",
@@ -171,37 +181,112 @@ static const char *help_msg_pdc[] = {
 	NULL
 };
 
+static void unvisit(RList *visited, RAnalBlock *bb) {
+	RListIter *iter;
+	RAnalBlock *b;
+	r_list_foreach (visited, iter, b) {
+		if (b->addr == bb->addr) {
+			r_list_delete (visited, iter);
+			break;
+		}
+	}
+}
+
+static char *cleancomments(char *s) {
+	// trim newline+spaces before //
+	char *p = s;
+	char *nl = NULL;
+	int spaces = 0;
+	bool ispfx = false;
+	while (*p) {
+		if (*p == '\n') {
+			nl = p;
+			spaces = 0;
+			ispfx = true;
+		} else if (r_str_startswith (p, "//")) {
+			char *dsnl = strchr (p, '\n');
+			if (r_str_startswith (p, "// goto")) {
+				memmove (p, dsnl + 1, strlen (dsnl + 1) + 1);
+				p = dsnl;
+				continue;
+			}
+			if (nl && spaces > 4) {
+				*nl = ' ';
+				if (0) {
+					char *nnl = strchr (p, '\n');
+					char *port = r_str_ndup (p, nnl - p);
+					eprintf ("MEMMO (%s)\n", port);
+					free (port);
+				}
+				memmove (nl + 1, p, strlen (p) + 1);
+				p = strchr (nl + 1, '\n');
+				if (!p) {
+					break;
+				}
+				spaces = 0;
+				nl = p;
+			}
+		} else if (*p == ' ') {
+			if (ispfx) {
+				spaces++;
+			}
+		} else {
+			ispfx = false;
+			spaces = 0;
+		}
+		p++;
+	}
+	// remove empty lines
+	s = r_str_replace (s, "\n\n", "\n", true);
+	p = s;
+	while (*p) {
+		char *nlnl = strstr (p, "  \n");
+		if (!nlnl) {
+			break;
+		}
+		char *prev = (char *)r_str_rchr (p, nlnl, '\n');
+		if (!prev) {
+			break;
+		}
+		memmove (prev + 1, nlnl + 3, strlen (nlnl + 3) + 1);
+		p = prev + 2;
+	}
+	return s;
+}
+
 #define I_TAB 2
-#define K_MARK(x) sdb_fmt("mark.%"PFMT64x,x)
-#define K_ELSE(x) sdb_fmt("else.%"PFMT64x,x)
-#define K_INDENT(x) sdb_fmt("loc.%"PFMT64x,x)
-#define SET_INDENT(x) { (x) = (x)>0?(x):0; memset (indentstr, ' ', sizeof(indentstr)); indentstr [((x) * I_TAB)] = 0; }
+#define K_MARK(x) r_strf ("mark.%"PFMT64x,x)
+#define K_ELSE(x) r_strf ("else.%"PFMT64x,x)
+#define K_INDENT(x) r_strf ("loc.%"PFMT64x,x)
+#define SET_INDENT(x) { (x) = (x)>0?(x):0; memset (indentstr, ' ', sizeof (indentstr)); indentstr [((x) * I_TAB)] = 0; }
 R_API int r_core_pseudo_code(RCore *core, const char *input) {
 	bool show_c_headers = *input == 'c';
 	if (*input == '?') {
 		r_core_cmd_help (core, help_msg_pdc);
 		return false;
 	}
+	RStrBuf *out = r_strbuf_new ("");
 #define PRINTF(a, ...) {\
 	if (pj) {\
 		r_strbuf_appendf (codestr, a, ##__VA_ARGS__);\
 	} else {\
-		r_cons_printf (a, ##__VA_ARGS__);\
+		r_strbuf_appendf (out, a, ##__VA_ARGS__);\
 	}}
 #define NEWLINE(a,i) {\
 	size_t eos = R_MIN ((i) * 2, sizeof (indentstr) - 2);\
-	if (eos < 1) eos = 0;\
+	if (eos < 1) { eos = 0; }\
 	memset (indentstr, ' ', sizeof (indentstr)); indentstr [(eos * 2)] = 0;\
 	if (pj) {\
 		if (show_addr) r_strbuf_appendf (codestr, "\n0x%08"PFMT64x" | %s", a, indentstr);\
 		else r_strbuf_appendf (codestr, "\n%s", indentstr);\
 	} else {\
-		r_cons_newline();\
-		if (show_addr) r_cons_printf (" 0x%08"PFMT64x" | %s", a, indentstr);\
-		else r_cons_printf ("%s", indentstr); }\
+		r_strbuf_append (out, "\n");\
+		if (show_addr) r_strbuf_appendf (out, " 0x%08"PFMT64x" | %s", a, indentstr);\
+		else r_strbuf_append (out, indentstr); }\
 	}
+#define PRINTGOTO(y, x) if (x != UT64_MAX && y != x) { NEWLINE (x, indent); PRINTF (" goto loc_0x%08"PFMT64x, x); }
 	const char *cmdPdc = r_config_get (core->config, "cmd.pdc");
-	if (cmdPdc && *cmdPdc && !strstr (cmdPdc, "pdc")) {
+	if (R_STR_ISNOTEMPTY (cmdPdc) && !strstr (cmdPdc, "pdc")) {
 		if (strstr (cmdPdc, "!*") || strstr (cmdPdc, "#!")) {
 			if (!strcmp (input, "*")) {
 				input = " -r2";
@@ -230,7 +315,7 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 	r_config_hold (hc, "scr.color", "emu.str", "asm.emu", "emu.write", NULL);
 	r_config_hold (hc, "io.cache", NULL);
 	if (!fcn) {
-		eprintf ("Cannot find function in 0x%08"PFMT64x"\n", core->offset);
+		R_LOG_ERROR ("Cannot find function in 0x%08"PFMT64x, core->offset);
 		r_config_hold_free (hc);
 		return false;
 	}
@@ -254,11 +339,8 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 	r_config_set_i (core->config, "asm.cmt.col", 30);
 	r_config_set_b (core->config, "io.cache", true);
 	r_core_cmd0 (core, "aeim");
-	PJ *pj = NULL;
-	if (show_json) {
-		pj = r_core_pj_new (core);
-	}
 
+	r_strf_buffer (64);
 	RStrBuf *codestr = r_strbuf_new ("");
 	db = sdb_new0 ();
 	// walk all basic blocks
@@ -270,12 +352,21 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 	char indentstr[1024] = {0};
 	int indent = 0;
 	int nindent = 1;
+	// XXX sorting basic blocks is nice for the reader, but introduces conceptual problems
+	// when the entrypoint is not starting at the lowest address. // r_list_sort (fcn->bbs, cmpnbbs);
 	int n_bb = r_list_length (fcn->bbs);
+	PJ *pj = NULL;
 	if (show_json) {
 		pj = r_core_pj_new (core);
 		pj_o (pj);
 		pj_ka (pj, "annotations");
 	}
+	const char *cc = fcn->cc ? fcn->cc: "default";
+	const char *cc_a0 = r_anal_cc_arg (core->anal, cc, 0, -1);
+	const char *cc_a1 = r_anal_cc_arg (core->anal, cc, 1, -1);
+	const char *a0 = cc_a0? cc_a0: r_reg_get_name_by_type (core->anal->reg, "A0");
+	const char *a1 = cc_a1? cc_a1: r_reg_get_name_by_type (core->anal->reg, "A1");
+	const char *r0 = r_reg_get_name_by_type (core->anal->reg, "R0");
 	if (show_c_headers) {
 		// NEWLINE (fcn->addr, indent);
 		PRINTF ("// global registers\n");
@@ -285,51 +376,51 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 		PRINTF ("unsigned int *dword = &stack;\n");
 		PRINTF ("unsigned short *word = &stack;\n");
 		PRINTF ("unsigned char *byte = &stack;\n");
-		PRINTF ("int eax, ebx, ecx, edx;\n");
+		PRINTF ("int %s, %s;\n", a0, a1);
 		PRINTF ("// This function contains %d basic blocks and its %d long.",
 			n_bb, (int)r_anal_function_realsize (fcn));
 		NEWLINE (fcn->addr, indent);
 		const char *S0 = "esp";
-		PRINTF ("static inline void push (int reg) {%s-=%d;stack[%s]=reg;}\n", S0, (int)sizeof (int), S0);
-		PRINTF ("static inline int pop() {int r = stack[%s]; %s+=%d; return r;}\n", S0, S0, (int)sizeof (int));
+		PRINTF ("static inline void push (int reg) {%s -= %d; stack[%s] = reg; }\n", S0, (int)sizeof (int), S0);
+		PRINTF ("static inline int pop() {int r = stack[%s]; %s += %d; return r; }\n", S0, S0, (int)sizeof (int));
 		PRINTF ("\n");
 	}
-	PRINTF ("int %s (int esi, int edx) {", fcn->name);
+
+	PRINTF ("int %s (int %s, int %s) {", fcn->name, a0, a1);
 	indent++;
 	RList *visited = r_list_newf (NULL);
 	ut64 addr = fcn->addr;
-	do {
-		if (!bb) {
-			break;
-		}
+	while (bb) {
 		r_list_append (visited, bb);
 		r_cons_push ();
 		bool html = r_config_get_b (core->config, "scr.html");
 		r_config_set_b (core->config, "scr.html", false);
-		char *code = r_core_cmd_str (core, sdb_fmt ("pD %"PFMT64d" @ 0x%08"PFMT64x"\n", bb->size, bb->addr));
+		char *code = r_core_cmd_str (core, r_strf ("pD %"PFMT64d" @ 0x%08"PFMT64x, bb->size, bb->addr));
 		r_cons_pop ();
 		r_config_set_b (core->config, "scr.html", html);
 		indent = 2;
-		SET_INDENT (indent);
 		if (!code) {
-			eprintf ("No code here\n");
+			R_LOG_ERROR ("No code here");
 			break;
 		}
+		// SET_INDENT (indent);
+		// PRINTF ("\n---\n");
+		code = r_str_replace (code, "\n\n", "\n", true);
 		code = r_str_replace (code, ";", "//", true);
-		const char *R0 = "eax";
+		code = cleancomments (code);
 		size_t len = strlen (code);
 		code[len - 1] = 0; // chop last newline
 		find_and_change (code, len);
 		if (!sdb_const_get (db, K_MARK (bb->addr), 0)) {
 			bool mustprint = !queuegoto || queuegoto != bb->addr;
 			if (mustprint) {
-				if (queuegoto) {
-					NEWLINE (bb->addr, indent);
-					PRINTF ("goto loc_0x%"PFMT64x, queuegoto);
+				if (queuegoto && queuegoto != UT64_MAX) {
+					// NEWLINE (bb->addr, indent);
+					// PRINTF ("3goto loc_0x%"PFMT64x, queuegoto);
 					queuegoto = 0LL;
 				}
 				NEWLINE (bb->addr, indent - 1);
-				PRINTF ("loc_0x%"PFMT64x":", bb->addr);
+				PRINTF ("loc_0x%08"PFMT64x":", bb->addr);
 				// foreach lines
 				RList *lines = r_str_split_list (code, "\n", 0);
 				RListIter *iter;
@@ -344,6 +435,8 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 						char *s = strchr (line, ' ');
 						if (s) {
 							line = r_str_trim_head_ro (s + 1);
+						} else {
+							line = "";
 						}
 					}
 					if (pj) {
@@ -354,28 +447,64 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 						pj_ks (pj, "type", "offset");
 						pj_end (pj);
 					}
-					NEWLINE (addr, indent);
-					PRINTF ("%s", line);
+					if (R_STR_ISNOTEMPTY (line)) {
+						NEWLINE (addr, indent);
+						PRINTF ("%s", line);
+					}
 				}
 				r_list_free (lines);
 				free (code);
 				sdb_num_set (db, K_MARK (bb->addr), 1, 0);
 			}
 		}
+		bool closed = false;
+		ut64 gotoaddr = UT64_MAX;
+		if (bb->fail == UT64_MAX) {
+			if (bb->jump != UT64_MAX) {
+#if 1
+				gotoaddr = bb->jump;
+				// PRINTGOTO (UT64_MAX, bb->jump);
+#endif
+			} else {
+				closed = true;
+#if 0
+				NEWLINE (bb->addr, indent);
+				PRINTF ("return; ");
+#endif
+			}
+		} else {
+			NEWLINE (bb->addr, indent);
+			PRINTF ("goto loc_0x%08"PFMT64x";", bb->fail);
+		}
 		if (sdb_const_get (db, K_INDENT (bb->addr), 0)) {
 			// already analyzed, go pop and continue
 			// XXX check if can't pop
-			//eprintf ("%s// 0x%08llx already analyzed\n", indentstr, bb->addr);
+			unvisit (visited, bb);
+			R_LOG_DEBUG ("%s// 0x%08"PFMT64x" already analyzed", indentstr, bb->addr);
 			ut64 addr = sdb_array_pop_num (db, "indent", NULL);
 			if (addr == UT64_MAX) {
-				int i;
 				nindent = 1;
+#if 0
+				int i;
 				for (i = indent; i != nindent && i > 0; i--) {
 					NEWLINE (bb->addr, i);
-					PRINTF ("}");
+				//	PRINTF ("}");
+					closed = true;
+				}
+#else
+				closed = true;
+#endif
+				if (closed) {
+					NEWLINE (bb->addr, indent);
+					if (r0) {
+						PRINTF ("return %s;", r0);
+					} else {
+						PRINTF ("return;");
+					}
+				} else if (bb->fail != UT64_MAX) {
+					NEWLINE (bb->addr, indent);
+					PRINTF ("goto loc_0x%08"PFMT64x";", bb->fail);
 				}
-				NEWLINE (bb->addr, indent);
-				PRINTF ("return %s;", R0);
 				RAnalBlock *nbb = r_anal_bb_from_offset (core->anal, bb->fail);
 				if (r_list_contains (visited, nbb)) {
 					nbb = r_anal_bb_from_offset (core->anal, bb->jump);
@@ -386,28 +515,28 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 				if (!nbb) {
 					break;
 				}
+				PRINTGOTO (nbb->addr, gotoaddr);
 				bb = nbb;
 				indent--;
 				continue;
 			}
+#if 0
 			if (sdb_num_get (db, K_ELSE (bb->addr), 0)) {
 				NEWLINE (addr, indent);
 				if (!strcmp (blocktype, "else")) {
-					PRINTF (" // } %s {", blocktype);
+					PRINTF (" // %s", blocktype);
 				} else {
-					PRINTF (" // } %s (?);", blocktype);
+					PRINTF (" // %s ();", blocktype);
 				}
-			} else {
-				NEWLINE (addr, indent);
-				PRINTF (" // }");
 			}
+#endif
 			if (addr != bb->addr) {
 				queuegoto = addr;
-				// r_cons_printf ("\n%s  goto loc_0x%llx", indentstr, addr);
+				// r_cons_printf ("\n%s  goto loc_0x%"PFMT64x, indentstr, addr);
 			}
 			bb = r_anal_bb_from_offset (core->anal, addr);
 			if (!bb) {
-				eprintf ("failed block\n");
+				R_LOG_ERROR ("failed block");
 				break;
 			}
 			nindent = sdb_num_get (db, K_INDENT (addr), NULL);
@@ -439,12 +568,12 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 					if (!sdb_get (db, K_INDENT (fail), 0)) {
 						bb = r_anal_bb_from_offset (core->anal, fail);
 					} else {
-						eprintf (" // FAIL%c", 10);
+						R_LOG_ERROR ("sdb.get fail");
 					}
 				} else {
 					bb = r_anal_bb_from_offset (core->anal, jump);
 					if (!bb) {
-						eprintf ("failed to retrieve block at 0x%"PFMT64x"\n", jump);
+						R_LOG_ERROR ("Failed to retrieve block at 0x%"PFMT64x, jump);
 						break;
 					}
 					if (fail != UT64_MAX) {
@@ -452,13 +581,13 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 						indent++;
 						if (sdb_get (db, K_INDENT (bb->fail), 0)) {
 							/* do nothing here */
-							eprintf ("BlockAlready 0x%"PFMT64x"\n", bb->addr);
+							R_LOG_DEBUG ("There's already a block at 0x%"PFMT64x, bb->addr);
 						} else {
 							// r_cons_printf (" { RADICAL %llx\n", bb->addr);
 							sdb_array_push_num (db, "indent", fail, 0);
 							sdb_num_set (db, K_INDENT (fail), indent, 0);
 							sdb_num_set (db, K_ELSE (fail), 1, 0);
-							NEWLINE (bb->addr, indent);
+				//			NEWLINE (bb->addr, indent);
 						}
 					} else {
 						sdb_array_push_num (db, "indent", jump, 0);
@@ -470,15 +599,16 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 							blocktype = "else";
 						}
 						NEWLINE (bb->addr, indent);
-						PRINTF (" // do {");
+						// PRINTF ("do {");
+						indent++;
 						indent++;
 					}
 				}
-			} else {
+			} else if (!closed) {
 				ut64 addr = sdb_array_pop_num (db, "indent", NULL);
 				if (addr == UT64_MAX) {
 					NEWLINE (bb->addr, indent);
-					PRINTF (" // (break)");
+					PRINTF ("break;");
 					break;
 				}
 				bb = r_anal_bb_from_offset (core->anal, addr);
@@ -490,43 +620,124 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 						PRINTF ("}");
 					}
 				}
+				PRINTF ("goto loc_0x%08"PFMT64x";", bb->fail);
+#if 0
 				if (nindent != indent) {
 					NEWLINE (bb->addr, indent);
-					PRINTF (" // } else {");
+					PRINTF ("} else {");
 				}
+#endif
 				indent = nindent;
 			}
+			if (bb) {
+				PRINTGOTO (bb->addr, gotoaddr);
+			} else {
+				PRINTGOTO (UT64_MAX, gotoaddr);
+			}
 		}
-		//n_bb --;
-	} while (n_bb > 0);
+	}
 	RListIter *iter;
-	size_t orphan = 0;
+	bool use_html = r_config_get_b (core->config, "scr.html");
 	r_list_foreach (fcn->bbs, iter, bb) {
-		if (!r_list_contains (visited, bb)) {
-			orphan ++;
-			char *s = r_core_cmd_strf (core, "pdb@0x%08"PFMT64x"@e:asm.offset=0", bb->addr);
-			s = r_str_replace (s, ";", "//", true);
+		if (r_list_contains (visited, bb)) {
+			continue;
+		}
+		ut64 nextbbaddr = UT64_MAX;
+		if (iter->n) {
+			RListIter *nit = (RListIter*)(iter->n);
+			RAnalBlock *nbb = (RAnalBlock*)(nit->data);
+			nextbbaddr = nbb->addr;
+		}
+		if (use_html) {
+			r_config_set_b (core->config, "scr.html", false);
+		}
+		char *s = r_core_cmd_strf (core, "pdb@0x%08"PFMT64x"@e:asm.offset=%d", bb->addr, show_addr);
+		if (use_html) {
+			r_config_set_b (core->config, "scr.html", true);
+		}
+		s = r_str_replace (s, ";", "//", true);
+		char *lastgoto = strstr (s, "goto ");
+		if (lastgoto) {
+			if (!strchr (lastgoto, '\n')) {
+				*s = 0;
+			}
+		}
+		s = r_str_replace (s, "goto ", "// goto loc_", true);
+		s = cleancomments (s);
+		if (show_addr) {
+			// indent with | or stgh
+			char *os = r_str_prefix_all (s, " ");
+			free (s);
+			s = os;
+		} else {
+			int eos = indent;
+			memset (indentstr, ' ', sizeof (indentstr)); indentstr [(eos * 2)] = 0;
 			char *os = r_str_prefix_all (s, indentstr);
 			free (s);
 			s = os;
-			if (pj) {
-				pj_o (pj);
-				pj_kn (pj, "start", r_strbuf_length (codestr));
-				r_strbuf_append (codestr, s);
-				pj_kn (pj, "end", r_strbuf_length (codestr));
-				pj_kn (pj, "offset", addr);
-				pj_ks (pj, "type", "offset");
-				pj_end (pj);
-			}
-			NEWLINE (bb->addr, 1);
-			PRINTF ("loc_0x%08"PFMT64x": // orphan\n%s", bb->addr, s);
-			free (s);
 		}
+		size_t codelen = r_strbuf_length (codestr);
+		if (pj) {
+			pj_o (pj);
+			pj_kn (pj, "start", codelen);
+			r_strbuf_append (codestr, s);
+			pj_kn (pj, "end", codelen);
+			pj_kn (pj, "offset", addr);
+			pj_ks (pj, "type", "offset");
+			pj_end (pj);
+		} else {
+			r_strbuf_append (codestr, s);
+			// PRINTF ("goto loc_0x%"PFMT64x";", bb->fail);
+		}
+		if (codelen > 0) {
+			if (show_addr) {
+				r_strbuf_appendf (out, "\n 0x%08"PFMT64x" | ", bb->addr);
+			} else {
+				NEWLINE (bb->addr, 1);
+			}
+			RFlagItem *fi = r_flag_get_i (core->flags, bb->addr);
+			if (fi && r_str_startswith (fi->name, "case.")) {
+				const char *val = r_str_lchr (fi->name, '.') + 1;
+				char *hex = r_str_newf ("0x%s", val);
+				int nval = r_num_get (NULL, hex);
+				free (hex);
+				if (IS_PRINTABLE (nval)) {
+					PRINTF ("loc_0x%08"PFMT64x": // case '%c'\n%s", bb->addr, nval, s);
+				} else {
+					PRINTF ("loc_0x%08"PFMT64x": // case %s\n%s", bb->addr, val, s);
+				}
+			} else {
+				PRINTF ("loc_0x%08"PFMT64x": // orphan\n%s", bb->addr, s);
+			}
+			ut64 nbbaddr = nextbbaddr; // UT64_MAX;
+#if 0
+			eprintf ("iter %p %p\n", iter, iter->n);
+			if (nextbbaddr) {
+			}
+			if (iter->n) {
+				RAnalBlock *nbb = (RAnalBlock*)(iter->n);
+				nbbaddr = nbb->addr;
+			}
+#endif
+			if (bb->jump == UT64_MAX) {
+				NEWLINE (bb->addr, indent);
+				if (r0) {
+					PRINTF ("return %s;", r0);
+				} else {
+					PRINTF ("return;");
+				}
+			} else {
+				PRINTGOTO (nbbaddr, bb->jump);
+			}
+		}
+		free (s);
 	}
 	r_list_free (visited);
 	indent = 0;
 	NEWLINE (addr, indent);
 	PRINTF ("}\n");
+	r_config_hold_restore (hc);
+	r_config_hold_free (hc);
 	if (pj) {
 		pj_end (pj);
 		char *kode = r_strbuf_drain (codestr);
@@ -536,9 +747,18 @@ R_API int r_core_pseudo_code(RCore *core, const char *input) {
 		r_cons_printf ("%s\n", j);
 		free (kode);
 		free (j);
+		r_strbuf_free (out);
+	} else {
+		char *s = r_strbuf_drain (out);
+		if (r_config_get_i (core->config, "scr.color") > 0) {
+			char *ss = r_print_code_tocolor (s);
+			free (s);
+			s = ss;
+		}
+		r_cons_printf ("%s\n", s);
+		free (s);
+		r_strbuf_free (codestr);
 	}
-	r_config_hold_restore (hc);
-	r_config_hold_free (hc);
 	sdb_free (db);
 	return true;
 }
