@@ -1,4 +1,4 @@
-/* radare - Copyright 2009-2021 - pancake, nibble */
+/* radare - Copyright 2009-2024 - pancake, nibble */
 
 #include "r_core.h"
 #include "r_socket.h"
@@ -23,10 +23,10 @@ SECURITY IMPLICATIONS
 #define rtr_n core->rtr_n
 #define rtr_host core->rtr_host
 
-static RSocket *s = NULL;
-static RThread *httpthread = NULL;
-static RThread *rapthread = NULL;
-static const char *listenport = NULL;
+static R_TH_LOCAL RSocket *s = NULL;
+static R_TH_LOCAL RThread *httpthread = NULL;
+static R_TH_LOCAL RThread *rapthread = NULL;
+static R_TH_LOCAL const char *listenport = NULL;
 
 typedef struct {
 	const char *host;
@@ -55,7 +55,7 @@ R_API void r_core_wait(RCore *core) {
 }
 
 static void http_logf(RCore *core, const char *fmt, ...) {
-	bool http_log_enabled = r_config_get_i (core->config, "http.log");
+	bool http_log_enabled = r_config_get_b (core->config, "http.log");
 	va_list ap;
 	va_start (ap, fmt);
 	if (http_log_enabled) {
@@ -74,7 +74,7 @@ static void http_logf(RCore *core, const char *fmt, ...) {
 	va_end (ap);
 }
 
-static char *rtrcmd (TextLog T, const char *str) {
+static char *rtrcmd(TextLog T, const char *str) {
 	char *res, *ptr2;
 	char *ptr = r_str_uri_encode (str);
 	char *uri = r_str_newf ("http://%s:%s/%s%s", T.host, T.port, T.file, ptr? ptr: str);
@@ -96,7 +96,7 @@ static char *rtrcmd (TextLog T, const char *str) {
 static void showcursor(RCore *core, int x) {
 	if (core && core->vmode) {
 		r_cons_show_cursor (x);
-		r_cons_enable_mouse (x? r_config_get_i (core->config, "scr.wheel"): false);
+		r_cons_enable_mouse (x? r_config_get_b (core->config, "scr.wheel"): false);
 	} else {
 		r_cons_enable_mouse (false);
 	}
@@ -105,14 +105,14 @@ static void showcursor(RCore *core, int x) {
 
 // TODO: rename /name to /nick or /so?
 // clone of textlog_chat () using rtrcmd()
-static void rtr_textlog_chat (RCore *core, TextLog T) {
+static void rtr_textlog_chat(RCore *core, TextLog T) {
 	char prompt[64];
 	char buf[1024];
 	int lastmsg = 0;
 	const char *me = r_config_get (core->config, "cfg.user");
-	char *ret, msg[1024];
+	char *ret, msg[1024] = {0};
 
-	eprintf ("Type '/help' for commands and ^D to quit:\n");
+	R_LOG_INFO ("Type '/help' for commands and ^D to quit:");
 	char *oldprompt = strdup (r_line_singleton ()->prompt);
 	snprintf (prompt, sizeof (prompt) - 1, "[%s]> ", me);
 	r_line_set_prompt (prompt);
@@ -160,8 +160,8 @@ static void rtr_textlog_chat (RCore *core, TextLog T) {
 			free (rtrcmd (T, "T-"));
 		} else if (!strcmp (buf, "/quit")) {
 			goto beach;
-		} else if (*buf=='/') {
-			eprintf ("Unknown command: %s\n", buf);
+		} else if (*buf == '/') {
+			R_LOG_ERROR ("Unknown command: %s", buf);
 		} else {
 			char *cmd = r_str_newf ("T [%s] %s", me, buf);
 			free (rtrcmd (T, cmd));
@@ -179,7 +179,7 @@ R_API int r_core_rtr_http_stop(RCore *u) {
 	const char *port;
 	RSocket* sock;
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_socket_http_server_set_breaked (&r_cons_context ()->breaked);
 #endif
 	if (((size_t)u) > 0xff) {
@@ -201,7 +201,7 @@ static char *rtr_dir_files(const char *path) {
 	RListIter *iter;
 	// list files
 	RList *files = r_sys_dir (path);
-	eprintf ("Listing directory %s\n", path);
+	R_LOG_INFO ("Listing directory %s", path);
 	r_list_foreach (files, iter, file) {
 		if (file[0] == '.') {
 			continue;
@@ -213,9 +213,9 @@ static char *rtr_dir_files(const char *path) {
 	return r_str_append (ptr, "</body></html>\n");
 }
 
-#if __UNIX__
+#if R2__UNIX__ && !__wasi__
 static void dietime(int sig) {
-	eprintf ("It's Die Time!\n");
+	eprintf ("It's Time To Die!\n");
 	exit (0);
 }
 #endif
@@ -223,17 +223,17 @@ static void dietime(int sig) {
 static void activateDieTime(RCore *core) {
 	int dt = r_config_get_i (core->config, "http.dietime");
 	if (dt > 0) {
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 		r_sys_signal (SIGALRM, dietime);
 		alarm (dt);
 #else
-		eprintf ("http.dietime only works on *nix systems\n");
+		R_LOG_ERROR ("http.dietime only works on *nix systems");
 #endif
 	}
 }
 
-#include "rtr_http.c"
-#include "rtr_shell.c"
+#include "rtr_http.inc.c"
+#include "rtr_shell.inc.c"
 
 static int write_reg_val(char *buf, ut64 sz, ut64 reg, int regsize, bool bigendian) {
 	if (!bigendian) {
@@ -248,8 +248,7 @@ static int write_reg_val(char *buf, ut64 sz, ut64 reg, int regsize, bool bigendi
 			reg = r_swap_ut64 (reg);
 			break;
 		default:
-			eprintf ("%s: Unsupported reg size: %d\n",
-				 __func__, regsize);
+			R_LOG_ERROR ("Unsupported reg size: %d", regsize);
 			return -1;
 		}
 	}
@@ -288,8 +287,7 @@ static int write_big_reg(char *buf, ut64 sz, const utX *val, int regsize, bool b
 				 r_swap_ut64 (val->v128.Low),
 				 r_swap_ut64 (val->v128.High));
 	default:
-		eprintf ("%s: big registers (%d byte(s)) not yet supported\n",
-			 __func__, regsize);
+		R_LOG_ERROR ("big registers (%d byte(s)) not yet supported", regsize);
 		return -1;
 	}
 }
@@ -335,8 +333,7 @@ static int swap_big_regs(char *dest, ut64 sz, const char *src, int regsz) {
 		return snprintf (dest, sz, "0x%016"PFMT64x"%016"PFMT64x,
 				 val.v128.High, val.v128.Low);
 	default:
-		eprintf ("%s: big registers (%d byte(s)) not yet supported\n",
-			 __func__, regsz);
+		R_LOG_ERROR ("big registers (%d byte(s)) not yet supported", regsz);
 		return -1;
 	}
 }
@@ -392,26 +389,29 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 			case 't':
 				switch (cmd[3]) {
 				case '\0': // dpt
-					if (!core->dbg->h->threads) {
-						return -1;
-					}
-					if (!(list = core->dbg->h->threads(core->dbg, core->dbg->pid))) {
-						return -1;
-					}
-					memset (out_buf, 0, max_len);
-					out_buf[0] = 'm';
-					ret = 1;
-					r_list_foreach (list, iter, dbgpid) {
-						// Max length of a hex pid = 8?
-						if (ret >= max_len - 9) {
-							break;
+					{
+						RDebugPlugin *plugin = R_UNWRAP3 (core->dbg, current, plugin);
+						if (!plugin || !plugin->threads) {
+							return -1;
+						}
+						if (!(list = plugin->threads (core->dbg, core->dbg->pid))) {
+							return -1;
+						}
+						memset (out_buf, 0, max_len);
+						out_buf[0] = 'm';
+						ret = 1;
+						r_list_foreach (list, iter, dbgpid) {
+							// Max length of a hex pid = 8?
+							if (ret >= max_len - 9) {
+								break;
+							}
+							snprintf (out_buf + ret, max_len - ret - 1, "%x,", dbgpid->pid);
+							ret = strlen (out_buf);
+						}
+						if (ret > 1) {
+							ret--;
+							out_buf[ret] = '\0';
 						}
-						snprintf (out_buf + ret, max_len - ret - 1, "%x,", dbgpid->pid);
-						ret = strlen (out_buf);
-					}
-					if (ret > 1) {
-						ret--;
-						out_buf[ret] = '\0';
 					}
 					return 0;
 				case 'r': // dptr -> return current tid as int
@@ -423,10 +423,10 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 			break;
 		case 'r': // dr
 			r_debug_reg_sync (core->dbg, R_REG_TYPE_ALL, false);
-			be = r_config_get_i (core->config, "cfg.bigendian");
+			be = r_config_get_b (core->config, "cfg.bigendian");
 			if (isspace ((ut8)cmd[2])) { // dr reg
 				const char *name, *val_ptr;
-				char new_cmd[128] = { 0 };
+				char new_cmd[128] = {0};
 				int off = 0;
 				name = cmd + 3;
 				// Temporarily using new_cmd to store reg name
@@ -547,7 +547,6 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 	RSocket *sock;
 	int p, ret;
-	bool debug_msg = false;
 	char port[10];
 	char *file = NULL, *args = NULL;
 	libgdbr_t *g;
@@ -555,25 +554,21 @@ static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 	if (!core || !path) {
 		return -1;
 	}
-	if (*path == '!') {
-		debug_msg = true;
-		path++;
-	}
 	if (!(path = r_str_trim_head_ro (path)) || !*path) {
-		eprintf ("gdbserver: Port not specified\n");
+		R_LOG_ERROR ("gdbserver: Port not specified");
 		return -1;
 	}
 	if (!(p = atoi (path)) || p < 0 || p > 65535) {
-		eprintf ("gdbserver: Invalid port: %d\n", p);
+		R_LOG_ERROR ("gdbserver: Invalid port: %d", p);
 		return -1;
 	}
 	snprintf (port, sizeof (port) - 1, "%d", p);
 	if (!(file = strchr (path, ' '))) {
-		eprintf ("gdbserver: File not specified\n");
+		R_LOG_ERROR ("gdbserver: File not specified");
 		return -1;
 	}
 	if (!(file = (char *)r_str_trim_head_ro (file)) || !*file) {
-		eprintf ("gdbserver: File not specified\n");
+		R_LOG_ERROR ("gdbserver: File not specified");
 		return -1;
 	}
 	args = strchr (file, ' ');
@@ -587,32 +582,31 @@ static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 	}
 
 	if (!r_core_file_open (core, file, R_PERM_RX, 0)) {
-		eprintf ("Cannot open file (%s)\n", file);
+		R_LOG_ERROR ("Cannot open file (%s)", file);
 		return -1;
 	}
 	r_core_file_reopen_debug (core, args);
 
 	if (!(sock = r_socket_new (false))) {
-		eprintf ("gdbserver: Could not open socket for listening\n");
+		R_LOG_ERROR ("gdbserver: Could not open socket for listening");
 		return -1;
 	}
 	if (!r_socket_listen (sock, port, NULL)) {
 		r_socket_free (sock);
-		eprintf ("gdbserver: Cannot listen on port: %s\n", port);
+		R_LOG_ERROR ("gdbserver: Cannot listen on port: %s", port);
 		return -1;
 	}
 	if (!(g = R_NEW0 (libgdbr_t))) {
 		r_socket_free (sock);
-		eprintf ("gdbserver: Cannot alloc libgdbr instance\n");
+		R_LOG_ERROR ("gdbserver: Cannot alloc libgdbr instance");
 		return -1;
 	}
 	gdbr_init (g, true);
-	g->server_debug = debug_msg;
 	int arch = r_sys_arch_id (r_config_get (core->config, "asm.arch"));
 	int bits = r_config_get_i (core->config, "asm.bits");
 	gdbr_set_architecture (g, arch, bits);
 	core->gdbserver_up = 1;
-	eprintf ("gdbserver started on port: %s, file: %s\n", port, file);
+	R_LOG_INFO ("gdbserver started on port: %s, file: %s", port, file);
 
 	for (;;) {
 		if (!(g->sock = r_socket_accept (sock))) {
@@ -636,12 +630,12 @@ static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 R_API int r_core_rtr_gdb(RCore *core, int launch, const char *path) {
 	int ret;
 	if (r_sandbox_enable (0)) {
-		eprintf ("sandbox: connect disabled\n");
+		R_LOG_ERROR ("connect disable the sandbox");
 		return -1;
 	}
 	// TODO: do stuff with launch
 	if (core->gdbserver_up) {
-		eprintf ("gdbserver is already running\n");
+		R_LOG_INFO ("the gdbserver is already running");
 		return -1;
 	}
 	ret = r_core_rtr_gdb_run (core, launch, path);
@@ -659,7 +653,7 @@ R_API void r_core_rtr_pushout(RCore *core, const char *input) {
 			}
 		}
 		if (!(cmd = strchr (input, ' '))) {
-			eprintf ("Error\n");
+			R_LOG_ERROR ("Missing space");
 			return;
 		}
 	} else {
@@ -667,31 +661,31 @@ R_API void r_core_rtr_pushout(RCore *core, const char *input) {
 	}
 
 	if (!rtr_host[rtr_n].fd || !rtr_host[rtr_n].fd->fd) {
-		eprintf ("Error: Unknown host\n");
+		R_LOG_ERROR ("Unknown host");
 		return;
 	}
 
 	if (!(str = r_core_cmd_str (core, cmd))) {
-		eprintf ("Error: radare_cmd_str returned NULL\n");
+		R_LOG_ERROR ("radare_cmd_str returned NULL");
 		return;
 	}
 
 	switch (rtr_host[rtr_n].proto) {
 	case RTR_PROTOCOL_RAP:
-		eprintf ("Error: Cannot use '=<' to a rap connection.\n");
+		R_LOG_ERROR ("Cannot use '=<' to a rap connection");
 		break;
 	case RTR_PROTOCOL_UNIX:
 		r_socket_write (rtr_host[rtr_n].fd, str, strlen (str));
 		break;
 	case RTR_PROTOCOL_HTTP:
-		eprintf ("TODO\n");
+		R_LOG_TODO ("RTR_PROTOCOL_HTTP");
 		break;
 	case RTR_PROTOCOL_TCP:
 	case RTR_PROTOCOL_UDP:
 		r_socket_write (rtr_host[rtr_n].fd, str, strlen (str));
 		break;
 	default:
-		eprintf ("Unknown protocol\n");
+		R_LOG_ERROR ("Unknown protocol");
 		break;
 	}
 	free (str);
@@ -734,12 +728,12 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			const char *name;
 			int protocol;
 		} uris[7] = {
-			{"tcp", RTR_PROTOCOL_TCP},
-			{"udp", RTR_PROTOCOL_UDP},
-			{"rap", RTR_PROTOCOL_RAP},
-			{"r2p", RTR_PROTOCOL_RAP},
-			{"http", RTR_PROTOCOL_HTTP},
-			{"unix", RTR_PROTOCOL_UNIX},
+			{ "tcp", RTR_PROTOCOL_TCP},
+			{ "udp", RTR_PROTOCOL_UDP},
+			{ "rap", RTR_PROTOCOL_RAP},
+			{ "r2p", RTR_PROTOCOL_RAP},
+			{ "http", RTR_PROTOCOL_HTTP},
+			{ "unix", RTR_PROTOCOL_UNIX},
 			{NULL, 0}
 		};
 		char *s = r_str_ndup (input, pikaboo - input);
@@ -774,19 +768,19 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			// it's fine to listen without serving a file
 		} else {
 			file = "cmd/";
-			eprintf ("Error: Missing '/'\n");
+			R_LOG_ERROR ("Missing '/'");
 			//c:wreturn;
 		}
 	}
 
 	if (r_sandbox_enable (0)) {
-		eprintf ("sandbox: connect disabled\n");
+		R_LOG_ERROR ("sandbox: connect disabled");
 		return;
 	}
 
 	fd = r_socket_new (false);
 	if (!fd) {
-		eprintf ("Error: Cannot create new socket\n");
+		R_LOG_ERROR ("Cannot create new socket");
 		return;
 	}
 	switch (proto) {
@@ -796,54 +790,54 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			char *uri = r_str_newf ("http://%s:%s/%s", host, port, file);
 			char *str = r_socket_http_get (uri, NULL, &len);
 			if (!str) {
-				eprintf ("Cannot find peer\n");
+				R_LOG_ERROR ("Cannot find peer");
 				r_socket_free (fd);
 				return;
 			}
-			core->num->value = 0;
 			// eprintf ("Connected to: 'http://%s:%s'\n", host, port);
+			r_core_return_value (core, R_CMD_RC_SUCCESS);
 			free (str);
 		}
 		break;
 	case RTR_PROTOCOL_RAP:
 		if (!r_socket_connect_tcp (fd, host, port, timeout)) { //TODO: Use rap.ssl
-			eprintf ("Error: Cannot connect to '%s' (%s)\n", host, port);
+			R_LOG_ERROR ("Cannot connect to '%s' (%s)", host, port);
 			r_socket_free (fd);
 			return;
 		} else {
 			int n = r_socket_rap_client_open (fd, file, 0);
-			eprintf ("opened as fd = %d\n", n);
+			R_LOG_INFO ("opened as fd = %d", n);
 		}
 		break;
 	case RTR_PROTOCOL_UNIX:
 		if (!r_socket_connect_unix (fd, host)) {
-			core->num->value = 1;
-			eprintf ("Error: Cannot connect to 'unix://%s'\n", host);
+			r_core_return_value (core, R_CMD_RC_FAILURE);
+			R_LOG_ERROR ("Cannot connect to 'unix://%s'", host);
 			r_socket_free (fd);
 			return;
 		}
-		core->num->value = 0;
-		eprintf ("Connected to: 'unix://%s'\n", host);
+		r_core_return_value (core, R_CMD_RC_SUCCESS);
+		R_LOG_INFO ("Connected to: 'unix://%s'", host);
 		break;
 	case RTR_PROTOCOL_TCP:
 		if (!r_socket_connect_tcp (fd, host, port, timeout)) { //TODO: Use rap.ssl
-			core->num->value = 1;
-			eprintf ("Error: Cannot connect to '%s' (%s)\n", host, port);
+			r_core_return_value (core, R_CMD_RC_FAILURE);
+			R_LOG_ERROR ("Cannot connect to '%s' (%s)", host, port);
 			r_socket_free (fd);
 			return;
 		}
-		core->num->value = 0;
-		eprintf ("Connected to: %s at port %s\n", host, port);
+		r_core_return_value (core, R_CMD_RC_SUCCESS);
+		R_LOG_INFO ("Connected to: %s at port %s", host, port);
 		break;
 	case RTR_PROTOCOL_UDP:
 		if (!r_socket_connect_udp (fd, host, port, timeout)) { //TODO: Use rap.ssl
-			core->num->value = 1;
-			eprintf ("Error: Cannot connect to '%s' (%s)\n", host, port);
+			r_core_return_value (core, R_CMD_RC_FAILURE);
+			R_LOG_ERROR ("Cannot connect to '%s' (%s)", host, port);
 			r_socket_free (fd);
 			return;
 		}
-		core->num->value = 0;
-		eprintf ("Connected to: %s at port %s\n", host, port);
+		r_core_return_value (core, R_CMD_RC_SUCCESS);
+		R_LOG_INFO ("Connected to: %s at port %s", host, port);
 		break;
 	}
 	ret = core->num->value;
@@ -862,8 +856,8 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 		rtr_n = i;
 		break;
 	}
-	core->num->value = ret;
-	// double free wtf is freed this here? r_socket_free(fd);
+	r_core_return_value (core, ret);
+	// double free wtf is freed this here? r_socket_free (fd);
 	//r_core_rtr_list (core);
 }
 
@@ -907,29 +901,29 @@ R_API void r_core_rtr_event(RCore *core, const char *input) {
 	}
 	if (!strcmp (input, "errmsg")) {
 		// TODO: support udp, tcp, rap, ...
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 		char *f = r_file_temp ("errmsg");
 		r_cons_printf ("%s\n", f);
 		r_file_rm (f);
 		errmsg_tmpfile = strdup (f);
 		int e = mkfifo (f, 0644);
 		if (e == -1) {
-			perror ("mkfifo");
+			r_sys_perror ("mkfifo");
 		} else {
 			int ff = open (f, O_RDWR);
 			if (ff != -1) {
 				dup2 (ff, 2);
 				errmsg_fd = ff;
 			} else {
-				eprintf ("Cannot open fifo: %s\n", f);
+				R_LOG_ERROR ("Cannot open fifo: %s", f);
 			}
 		}
 		// r_core_event (core, );
 		free (s);
 		free (f);
-		// TODO: those files are leaked when closing r_core_free() should be deleted
+		// TODO: those files are leaked when closing r_core_free () should be deleted
 #else
-		eprintf ("Not supported for your platform.\n");
+		R_LOG_ERROR ("Not supported for your platform");
 #endif
 	} else {
 		eprintf ("(%s)\n", input);
@@ -993,8 +987,8 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 
 	if (*input == '&') { // "=h&" "=&:9090"
 		if (rapthread) {
-			eprintf ("RAP Thread is already running\n");
-			eprintf ("This is experimental and probably buggy. Use at your own risk\n");
+			R_LOG_INFO ("RAP Thread is already running");
+			R_LOG_INFO ("This is experimental and probably buggy. Use at your own risk");
 		} else {
 			// TODO: use tasks
 			RapThread *RT = R_NEW0 (RapThread);
@@ -1003,11 +997,13 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 				RT->input = strdup (input + 1);
 				//RapThread rt = { core, strdup (input + 1) };
 				rapthread = r_th_new (r_core_rtr_rap_thread, RT, false);
+#if 0
 				int cpuaff = (int)r_config_get_i (core->config, "cfg.cpuaffinity");
 				r_th_setaffinity (rapthread, cpuaff);
+#endif
 				r_th_setname (rapthread, "rapthread");
-				r_th_start (rapthread, true);
-				eprintf ("Background rap server started.\n");
+				r_th_start (rapthread);
+				R_LOG_INFO ("Background rap server started");
 			}
 		}
 		return;
@@ -1025,8 +1021,8 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 	}
 
 	if (!rtr_host[rtr_n].fd) {
-		eprintf ("Error: Unknown host\n");
-		core->num->value = 1; // fail
+		R_LOG_ERROR ("Unknown host");
+		r_core_return_value (core, R_CMD_RC_FAILURE);
 		return;
 	}
 
@@ -1037,8 +1033,9 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 			return;
 		}
 		r_socket_close (s);
-		if (!r_socket_connect (s, rh->host, sdb_fmt ("%d", rh->port), R_SOCKET_PROTO_TCP, 0)) {
-			eprintf ("Error: Cannot connect to '%s' (%d)\n", rh->host, rh->port);
+		r_strf_var (portstr, 32, "%d", rh->port);
+		if (!r_socket_connect (s, rh->host, portstr, R_SOCKET_PROTO_TCP, 0)) {
+			R_LOG_ERROR ("Cannot connect to '%s' (%d)", rh->host, rh->port);
 			r_socket_free (s);
 			return;
 		}
@@ -1047,7 +1044,7 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 		int maxlen = 4096; // r_read_le32 (blen);
 		char *cmd_output = calloc (1, maxlen + 1);
 		if (!cmd_output) {
-			eprintf ("Error: Allocating cmd output\n");
+			R_LOG_ERROR ("Allocating cmd output");
 			return;
 		}
 		(void)r_socket_read_block (s, (ut8*)cmd_output, maxlen);
@@ -1068,10 +1065,10 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 		char *uri = r_str_newf ("http://%s:%d/cmd/%s", rh->host, rh->port, cmd);
 		char *str = r_socket_http_get (uri, NULL, &len);
 		if (!str) {
-			eprintf ("Cannot find '%s'\n", uri);
+			R_LOG_ERROR ("Cannot find '%s'", uri);
 			return;
 		}
-		core->num->value = 0;
+		r_core_return_value (core, R_CMD_RC_SUCCESS);
 		str[len] = 0;
 		r_cons_print (str);
 		free ((void *)str);
@@ -1080,7 +1077,7 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 	}
 
 	if (rtr_host[rtr_n].proto == RTR_PROTOCOL_RAP) {
-		core->num->value = 0; // that's fine
+		r_core_return_value (core, R_CMD_RC_SUCCESS);
 		cmd = r_str_trim_head_ro (cmd);
 		RSocket *fh = rtr_host[rtr_n].fd;
 		if (!strlen (cmd)) {
@@ -1093,11 +1090,11 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 		free (cmd_output);
 		return;
 	}
-	eprintf ("Error: Unknown protocol\n");
+	R_LOG_ERROR ("Unknown protocol");
 }
 
 // TODO: support len for binary data?
-R_API char *r_core_rtr_cmds_query (RCore *core, const char *host, const char *port, const char *cmd) {
+R_API char *r_core_rtr_cmds_query(RCore *core, const char *host, const char *port, const char *cmd) {
 	RSocket *s = r_socket_new (0);
 	const int timeout = 0;
 	char *rbuf = NULL;
@@ -1123,7 +1120,7 @@ R_API char *r_core_rtr_cmds_query (RCore *core, const char *host, const char *po
 			rbuf = r_str_append (rbuf, (const char *)buf);
 		}
 	} else {
-		eprintf ("Cannot connect\n");
+		R_LOG_ERROR ("Cannot connect");
 	}
 	r_socket_free (s);
 	return rbuf;
@@ -1150,7 +1147,7 @@ static void rtr_cmds_client_close(uv_tcp_t *client, bool remove) {
 	rtr_cmds_context *context = loop->data;
 	if (remove) {
 		size_t i;
-		for (i = 0; i < r_pvector_len (&context->clients); i++) {
+		for (i = 0; i < r_pvector_length (&context->clients); i++) {
 			if (r_pvector_at (&context->clients, i) == client) {
 				r_pvector_remove_at (&context->clients, i);
 				break;
@@ -1171,11 +1168,9 @@ static void rtr_cmds_alloc_buffer(uv_handle_t *handle, size_t suggested_size, uv
 
 static void rtr_cmds_write(uv_write_t *req, int status) {
 	rtr_cmds_client_context *context = req->data;
-
 	if (status) {
-		eprintf ("Write error: %s\n", uv_strerror (status));
+		R_LOG_ERROR ("Cannot write %s", uv_strerror (status));
 	}
-
 	free (req);
 	rtr_cmds_client_close (context->client, true);
 }
@@ -1186,7 +1181,7 @@ static void rtr_cmds_read(uv_stream_t *client, ssize_t nread, const uv_buf_t *bu
 
 	if (nread < 0) {
 		if (nread != UV_EOF) {
-			eprintf ("Failed to read: %s\n", uv_err_name ((int) nread));
+			R_LOG_ERROR ("Failed to read: %s", uv_err_name ((int) nread));
 		}
 		rtr_cmds_client_close ((uv_tcp_t *) client, true);
 		return;
@@ -1210,7 +1205,7 @@ static void rtr_cmds_read(uv_stream_t *client, ssize_t nread, const uv_buf_t *bu
 		client_context->res = strdup ("\n");
 	}
 
-	if (!client_context->res || (!r_config_get_i (client_context->core->config, "scr.prompt") &&
+	if (!client_context->res || (!r_config_get_b (client_context->core->config, "scr.prompt") &&
 				 !strcmp ((char *)buf, "q!")) ||
 				 !strcmp ((char *)buf, ".--")) {
 		rtr_cmds_client_close ((uv_tcp_t *) client, true);
@@ -1228,7 +1223,7 @@ static void rtr_cmds_read(uv_stream_t *client, ssize_t nread, const uv_buf_t *bu
 
 static void rtr_cmds_new_connection(uv_stream_t *server, int status) {
 	if (status < 0) {
-		eprintf ("New connection error: %s\n", uv_strerror (status));
+		R_LOG_ERROR ("New connection failed: %s", uv_strerror (status));
 		return;
 	}
 
@@ -1307,7 +1302,7 @@ R_API int r_core_rtr_cmds(RCore *core, const char *port) {
 	uv_tcp_bind (&context.server, (const struct sockaddr *) &addr, 0);
 	int r = uv_listen ((uv_stream_t *)&context.server, 32, rtr_cmds_new_connection);
 	if (r) {
-		eprintf ("Failed to listen: %s\n", uv_strerror (r));
+		R_LOG_ERROR ("Failed to listen: %s", uv_strerror (r));
 		goto beach;
 	}
 
@@ -1329,8 +1324,8 @@ beach:
 
 #else
 
-R_API int r_core_rtr_cmds (RCore *core, const char *port) {
-	unsigned char buf[4097];
+R_API int r_core_rtr_cmds(RCore *core, const char *port) {
+	ut8 buf[4097];
 	RSocket *ch = NULL;
 	int i, ret;
 	char *str;
@@ -1344,12 +1339,12 @@ R_API int r_core_rtr_cmds (RCore *core, const char *port) {
 	s->local = r_config_get_i (core->config, "tcp.islocal");
 
 	if (!r_socket_listen (s, port, NULL)) {
-		eprintf ("Error listening on port %s\n", port);
+		R_LOG_ERROR ("listening on port %s", port);
 		r_socket_free (s);
 		return false;
 	}
 
-	eprintf ("Listening for commands on port %s\n", port);
+	R_LOG_INFO ("Listening for commands on port %s", port);
 	listenport = port;
 	r_cons_break_push ((RConsBreak)r_core_rtr_http_stop, core);
 	for (;;) {
@@ -1368,9 +1363,7 @@ R_API int r_core_rtr_cmds (RCore *core, const char *port) {
 					buf[i] = buf[i + 1]? ';': '\0';
 				}
 			}
-			if ((!r_config_get_i (core->config, "scr.prompt") &&
-			     !strcmp ((char *)buf, "q!")) ||
-			    !strcmp ((char *)buf, ".--")) {
+			if ((!r_config_get_b (core->config, "scr.prompt") && !strcmp ((char *)buf, "q!")) || !strcmp ((char *)buf, ".--")) {
 				r_socket_close (ch);
 				break;
 			}
