@@ -1,6 +1,10 @@
 /* radare - LGPL - Copyright 2019 - pancake */
 
 #include <r_core.h>
+#include <r_vec.h>
+
+R_VEC_TYPE(RVecAnalRef, RAnalRef);
+
 #define SORT_ADDRESS 0
 #define SORT_NAME 1
 
@@ -67,13 +71,17 @@ static char *print_item(void *_core, void *_item, bool selected) {
 
 static RList *__xrefs(RCore *core, ut64 addr) {
 	RList *r = r_list_newf (free);
-	RListIter *iter;
+	RVecAnalRef *xrefs = r_anal_xrefs_get (core->anal, addr);
+	if (!xrefs) {
+		return r;
+	}
+
 	RAnalRef *ref;
-	RList *xrefs = r_anal_xrefs_get (core->anal, addr);
-	r_list_foreach (xrefs, iter, ref) {
+	R_VEC_FOREACH (xrefs, ref) {
 		if (ref->type != 'C') {
 			continue;
 		}
+
 		RCoreVisualViewGraphItem *item = R_NEW0 (RCoreVisualViewGraphItem);
 		RFlagItem *f = r_flag_get_at (core->flags, ref->addr, 0);
 		item->addr = ref->addr;
@@ -85,22 +93,29 @@ static RList *__xrefs(RCore *core, ut64 addr) {
 		}
 		r_list_append (r, item);
 	}
+
+	RVecAnalRef_free (xrefs);
 	return r;
 }
 
 static RList *__refs(RCore *core, ut64 addr) {
 	RList *r = r_list_newf (free);
-	RListIter *iter;
-	RAnalRef *ref;
 	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, addr, 0);
 	if (!fcn) {
 		return r;
 	}
-	RList *refs = r_anal_fcn_get_refs (core->anal, fcn);
-	r_list_foreach (refs, iter, ref) {
+
+	RVecAnalRef *refs = r_anal_function_get_refs (fcn);
+	if (!refs) {
+		return r;
+	}
+
+	RAnalRef *ref;
+	R_VEC_FOREACH (refs, ref) {
 		if (ref->type != 'C') {
 			continue;
 		}
+
 		RCoreVisualViewGraphItem *item = R_NEW0 (RCoreVisualViewGraphItem);
 		RFlagItem *f = r_flag_get_at (core->flags, ref->addr, 0);
 		item->addr = ref->addr;
@@ -112,6 +127,8 @@ static RList *__refs(RCore *core, ut64 addr) {
 		}
 		r_list_append (r, item);
 	}
+
+	RVecAnalRef_free (refs);
 	return r;
 }
 
@@ -149,12 +166,12 @@ static void __seek_cursor(RCoreVisualViewGraph *status) {
 	return;
 }
 
-static int cmpaddr (const void *_a, const void *_b) {
+static int cmpaddr(const void *_a, const void *_b) {
 	const RCoreVisualViewGraphItem *a = _a, *b = _b;
 	return a->addr - b->addr;
 }
 
-static int cmpname (const void *_a, const void *_b) {
+static int cmpname(const void *_a, const void *_b) {
 	const RCoreVisualViewGraphItem *a = _a, *b = _b;
 	if (!a || !b || !a->name || !b->name) {
 		return 0;
@@ -162,15 +179,15 @@ static int cmpname (const void *_a, const void *_b) {
 	return (int)strcmp (a->name, b->name);
 }
 
-static void __sort (RCoreVisualViewGraph *status, RList *list) {
-	r_return_if_fail (status && list);
+static void __sort(RCoreVisualViewGraph *status, RList *list) {
+	R_RETURN_IF_FAIL (status && list);
 	RListComparator cmp = (status->cur_sort == SORT_ADDRESS)? cmpaddr: cmpname;
 	list->sorted = false;
 	r_list_sort (list, cmp);
 }
 
-static void __toggleSort (RCoreVisualViewGraph *status) {
-	r_return_if_fail (status);
+static void __toggleSort(RCoreVisualViewGraph *status) {
+	R_RETURN_IF_FAIL (status);
 	status->cur_sort = (status->cur_sort == SORT_ADDRESS)? SORT_NAME: SORT_ADDRESS;
 	__sort (status, status->mainCol);
 	__sort (status, status->refsCol);
@@ -229,21 +246,21 @@ R_API int __core_visual_view_graph_update(RCore *core, RCoreVisualViewGraph *sta
 	/* 	__reset_status (status); */
 	/* } */
 
-	char *title = r_str_newf ("[r2-visual-browser] addr=0x%08"PFMT64x" faddr=0x%08"PFMT64x"", status->addr, status->fcn ? status->fcn->addr : 0);
+	char *title = r_str_newf ("[r2-visual-browser] addr=0x%08"PFMT64x" faddr=0x%08"PFMT64x, status->addr, status->fcn ? status->fcn->addr : 0);
 	if (title) {
-		r_cons_strcat_at (title, 0, 0, w - 1, 2);
+		r_cons_print_at (title, 0, 0, w - 1, 2);
 		free (title);
 	}
-	r_cons_strcat_at (xrefsColstr, 0, 2, colw, colh);
-	r_cons_strcat_at (mainColstr, colx, 2, colw*2, colh);
-	r_cons_strcat_at (refsColstr, colx * 2, 2, colw, colh);
+	r_cons_print_at (xrefsColstr, 0, 2, colw, colh);
+	r_cons_print_at (mainColstr, colx, 2, colw*2, colh);
+	r_cons_print_at (refsColstr, colx * 2, 2, colw, colh);
 
-	char *output = r_core_cmd_strf (core, "pd %d @e:asm.flags=0@ 0x%08"PFMT64x"; pds 256 @ 0x%08"PFMT64x"\n",
+	char *output = r_core_cmd_strf (core, "pd %d @e:asm.flags=0@ 0x%08"PFMT64x"; pds 256 @ 0x%08"PFMT64x,
 		32, status->addr, status->addr);
 	int disy = colh + 2;
-	r_cons_strcat_at (output, 10, disy, w, h - disy);
+	r_cons_print_at (output, 10, disy, w, h - disy);
 	free (output);
-	r_cons_flush();
+	r_cons_flush ();
 
 	free (xrefsColstr);
 	free (mainColstr);
@@ -323,7 +340,9 @@ R_API int r_core_visual_view_graph(RCore *core) {
 		case '\n':
 			{
 				RCoreVisualViewGraphItem *item = r_list_get_n (status.mainCol, status.cur);
-				r_core_seek (core, item->addr, 1);
+				if (item) {
+					r_core_seek (core, item->addr, true);
+				}
 			}
 			return true;
 			break;
