@@ -1,41 +1,47 @@
-/* radare - LGPL - Copyright 2016-2017 - rakholiyajenish.07 */
+/* radare - LGPL - Copyright 2016-2024 - rakholiyajenish.07 */
 
 #include <r_lib.h>
+#include <r_util/r_log.h>
+#include <r_util/r_assert.h>
 #include <r_crypto.h>
 #include "crypto_aes_algo.h"
 
 #define BLOCK_SIZE 16
 
-static bool aes_cbc_set_key(RCrypto *cry, const ut8 *key, int keylen, int mode, int direction) {
+static bool aes_cbc_set_key(RCryptoJob *cj, const ut8 *key, int keylen, int mode, int direction) {
 	if (!(keylen == 128 / 8 || keylen == 192 / 8 || keylen == 256 / 8)) {
 		return false;
 	}
-	cry->key_len = keylen;
-	memcpy (cry->key, key, keylen);
-	cry->dir = direction;
+	cj->key_len = keylen;
+	memcpy (cj->key, key, keylen);
+	cj->dir = direction;
 	return true;
 }
 
-static int aes_cbc_get_key_size(RCrypto *cry) {
-	return cry->key_len;
+static int aes_cbc_get_key_size(RCryptoJob *cj) {
+	R_RETURN_VAL_IF_FAIL (cj, -1);
+	return cj->key_len;
 }
 
-static bool aes_cbc_set_iv(RCrypto *cry, const ut8 *iv_src, int ivlen) {
+static bool aes_cbc_set_iv(RCryptoJob *cj, const ut8 *iv_src, int ivlen) {
 	if (ivlen != BLOCK_SIZE) {
 		return false;
 	}
-	cry->iv = calloc (1, BLOCK_SIZE);
-	memcpy (cry->iv, iv_src, BLOCK_SIZE);
+	cj->iv = calloc (1, BLOCK_SIZE);
+	if (!cj->iv) {
+		return false;
+	}
+	memcpy (cj->iv, iv_src, BLOCK_SIZE);
 	return true;
 }
 
-static bool aes_cbc_use(const char *algo) {
+static bool aes_cbc_check(const char *algo) {
 	return algo && !strcmp (algo, "aes-cbc");
 }
 
-static bool update(RCrypto *cry, const ut8 *buf, int len) {
-	if (NULL == cry->iv) {
-		eprintf ("IV not set. Use -I [iv]\n");
+static bool update(RCryptoJob *cj, const ut8 *buf, int len) {
+	if (!cj->iv) {
+		R_LOG_ERROR ("AES CBC IV is not defined. Use rahash2 -I [iv]");
 		return false;
 	}
 	struct aes_state st;
@@ -61,48 +67,51 @@ static bool update(RCrypto *cry, const ut8 *buf, int len) {
 		ibuf[len] = 8; // 0b1000;
 	}
 
-	st.key_size = cry->key_len;
+	st.key_size = cj->key_len;
 	st.rounds = 6 + (int)(st.key_size / 4);
 	st.columns = (int)(st.key_size / 4);
-	memcpy (st.key, cry->key, st.key_size);
+	memcpy (st.key, cj->key, st.key_size);
 
 	int i, j;
-	if (cry->dir == 0) {
+	switch (cj->dir) {
+	case R_CRYPTO_DIR_ENCRYPT:
 		for (i = 0; i < blocks; i++) {
 			for (j = 0; j < BLOCK_SIZE; j++) {
-				ibuf[i * BLOCK_SIZE + j] ^= cry->iv[j];
+				ibuf[i * BLOCK_SIZE + j] ^= cj->iv[j];
 			}
 			aes_encrypt (&st, ibuf + BLOCK_SIZE * i, obuf + BLOCK_SIZE * i);
-			memcpy (cry->iv, obuf + BLOCK_SIZE * i, BLOCK_SIZE);
+			memcpy (cj->iv, obuf + BLOCK_SIZE * i, BLOCK_SIZE);
 		}
-	} else if (cry->dir == 1) {
+		break;
+	case R_CRYPTO_DIR_DECRYPT:
 		for (i = 0; i < blocks; i++) {
 			aes_decrypt (&st, ibuf + BLOCK_SIZE * i, obuf + BLOCK_SIZE * i);
 			for (j = 0; j < BLOCK_SIZE; j++) {
-				obuf[i * BLOCK_SIZE + j] ^= cry->iv[j];
+				obuf[i * BLOCK_SIZE + j] ^= cj->iv[j];
 			}
-			memcpy(cry->iv, buf + BLOCK_SIZE * i, BLOCK_SIZE);
+			memcpy (cj->iv, buf + BLOCK_SIZE * i, BLOCK_SIZE);
 		}
+		break;
 	}
 
-	r_crypto_append (cry, obuf, size);
+	r_crypto_job_append (cj, obuf, size);
 	free (obuf);
 	free (ibuf);
 	return true;
 }
 
-static bool final(RCrypto *cry, const ut8 *buf, int len) {
-	return update (cry, buf, len);
-}
-
 RCryptoPlugin r_crypto_plugin_aes_cbc = {
-	.name = "aes-cbc",
+	.meta = {
+		.name = "aes-cbc",
+		.author = "pancake",
+		.license = "LGPL",
+	},
 	.set_key = aes_cbc_set_key,
 	.get_key_size = aes_cbc_get_key_size,
 	.set_iv = aes_cbc_set_iv,
-	.use = aes_cbc_use,
+	.check = aes_cbc_check,
 	.update = update,
-	.final = final
+	.end = update
 };
 
 #ifndef R2_PLUGIN_INCORE
