@@ -1,32 +1,39 @@
+/* radare - LGPL - Copyright 2017-2024 - pancake */
+
 #include <r_lib.h>
 #include <r_crypto.h>
 #include "crypto_serpent_algo.h"
 
-static struct serpent_state st = {{0}};
-
-static bool serpent_set_key(RCrypto *cry, const ut8 *key, int keylen, int mode, int direction) {
-	eprintf ("key_size: %d\n", keylen);
+static bool serpent_set_key(RCryptoJob *cj, const ut8 *key, int keylen, int mode, int direction) {
+	free (cj->data);
+	cj->data = R_NEW0 (struct serpent_state);
+	struct serpent_state *st = cj->data;
+	if (!st) {
+		return false;
+	}
+	R_LOG_INFO ("key_length: %d", keylen);
 	if ((keylen != 128 / 8) && (keylen != 192 / 8) && (keylen != 256 / 8)) {
 		return false;
 	}
-	st.key_size = keylen * 8;
-	eprintf ("key_size: %d\n", st.key_size);
-	memcpy (st.key, key, keylen);
-	cry->dir = direction;
+	st->key_size = keylen * 8;
+	R_LOG_INFO ("key_size: %d", st->key_size);
+	memcpy (st->key, key, keylen);
+	cj->dir = direction;
 	return true;
 }
 
-static int serpent_get_key_size(RCrypto *cry) {
-	return st.key_size;
+static int serpent_get_key_size(RCryptoJob *cj) {
+	struct serpent_state *st = cj->data;
+	return st? st->key_size: 0;
 }
 
-static bool serpent_use(const char *algo) {
+static bool serpent_check(const char *algo) {
 	return !strcmp (algo, "serpent-ecb");
 }
 
 #define BLOCK_SIZE 16
 
-static bool update(RCrypto *cry, const ut8 *buf, int len) {
+static bool update(RCryptoJob *cj, const ut8 *buf, int len) {
 	// Pad to the block size, do not append dummy block
 	const int diff = (BLOCK_SIZE - (len % BLOCK_SIZE)) % BLOCK_SIZE;
 	const int size = len + diff;
@@ -59,18 +66,29 @@ static bool update(RCrypto *cry, const ut8 *buf, int len) {
 		ibuf[len / 4] = r_read_le32 (tail);
 	}
 
-	if (cry->dir == 0) {
+	struct serpent_state *st = cj->data;
+	if (!st) {
+		R_LOG_ERROR ("No state");
+		free (obuf);
+		free (ibuf);
+		free (tmp);
+		return false;
+	}
+	switch (cj->dir) {
+	case R_CRYPTO_DIR_ENCRYPT:
 		for (i = 0; i < blocks; i++) {
 			// delta in number of ut32
 			const int delta = (BLOCK_SIZE * i) / 4;
-			serpent_encrypt (&st, ibuf + delta, tmp + delta);
+			serpent_encrypt (st, ibuf + delta, tmp + delta);
 		}
-	} else if (cry->dir > 0) {
+		break;
+	case R_CRYPTO_DIR_DECRYPT:
 		for (i = 0; i < blocks; i++) {
 			// delta in number of ut32
 			const int delta = (BLOCK_SIZE * i) / 4;
-			serpent_decrypt (&st, ibuf + delta, tmp + delta);
+			serpent_decrypt (st, ibuf + delta, tmp + delta);
 		}
+		break;
 	}
 
 	// Construct ut32 blocks from byte stream
@@ -83,24 +101,28 @@ static bool update(RCrypto *cry, const ut8 *buf, int len) {
 		obuf[k + 3] = (tmp[j] >> 24) & 0xff;
 	}
 
-	r_crypto_append (cry, obuf, size);
+	r_crypto_job_append (cj, obuf, size);
 	free (obuf);
 	free (ibuf);
 	free (tmp);
 	return true;
 }
 
-static bool final(RCrypto *cry, const ut8 *buf, int len) {
-	return update (cry, buf, len);
+static bool end(RCryptoJob *cj, const ut8 *buf, int len) {
+	return update (cj, buf, len);
 }
 
 RCryptoPlugin r_crypto_plugin_serpent = {
-	.name = "serpent-ecb",
+	.meta = {
+		.name = "serpent-ecb",
+		.license = "LGPL",
+		.author = "pancake",
+	},
 	.set_key = serpent_set_key,
 	.get_key_size = serpent_get_key_size,
-	.use = serpent_use,
+	.check = serpent_check,
 	.update = update,
-	.final = final
+	.end = end
 };
 
 #ifndef R2_PLUGIN_INCORE
