@@ -1,8 +1,7 @@
-/* radare - LGPL - Copyright 2016-2021 pancake */
+/* radare - LGPL - Copyright 2016-2022 pancake */
 
-#include <r_io.h>
-#include <r_asm.h>
 #include <r_debug.h>
+#include <r_asm.h>
 
 static bool __io_step(RDebug *dbg) {
 	free (dbg->iob.system (dbg->iob.io, "ds"));
@@ -48,11 +47,17 @@ static RList *__io_maps(RDebug *dbg) {
 			if (_s_) {
 				memmove (_s_, _s_ + 2, strlen (_s_));
 			}
-			sscanf (str, "0x%"PFMT64x" - 0x%"PFMT64x" %s %s",
-				&map_start, &map_end, perm, name);
+			if (r_str_scanf (str, "0x%Lx - 0x%Lx %.s %.s", &map_start, &map_end, sizeof (perm), perm, sizeof (name), name) > 4) {
+				break;
+			}
 			if (map_end != 0LL) {
-				RDebugMap *map = r_debug_map_new (name, map_start, map_end, r_str_rwx (perm), 0);
-				r_list_append (list, map);
+				int sperm = r_str_rwx (perm);
+				if (sperm >= 0) {
+					RDebugMap *map = r_debug_map_new (name, map_start, map_end, sperm, 0);
+					r_list_append (list, map);
+				} else {
+					R_LOG_WARN ("Invalid permission string (%s)", perm);
+				}
 			}
 			str = nl + 1;
 		} else {
@@ -90,13 +95,13 @@ static char *__io_reg_profile(RDebug *dbg) {
 }
 
 // "dr8" read register state
-static int __reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
+static bool __reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	char *dr8 = dbg->iob.system (dbg->iob.io, "dr8");
 	if (!dr8) {
 		const char *fb = r_cons_get_buffer ();
-		if (!fb || !*fb) {
-			eprintf ("debug.io: Failed to get dr8 from io\n");
-			return -1;
+		if (R_STR_ISEMPTY (fb)) {
+			R_LOG_ERROR ("Failed to get dr8 from io");
+			return false;
 		}
 		dr8 = strdup (fb);
 		r_cons_reset ();
@@ -104,21 +109,16 @@ static int __reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	ut8 *bregs = calloc (1, strlen (dr8));
 	if (!bregs) {
 		free (dr8);
-		return -1;
+		return false;
 	}
 	r_str_trim ((char *)bregs);
 	int sz = r_hex_str2bin (dr8, bregs);
 	if (sz > 0) {
 		memcpy (buf, bregs, R_MIN (size, sz));
-		free (bregs);
-		free (dr8);
-		return size;
-	} else {
-		// eprintf ("SIZE %d (%s)\n", sz, regs);
 	}
 	free (bregs);
 	free (dr8);
-	return -1;
+	return (sz > 0);
 }
 
 // "dc" continue execution
@@ -130,15 +130,19 @@ static bool __io_continue(RDebug *dbg, int pid, int tid, int sig) {
 
 // "dk" send kill signal
 static bool __io_kill(RDebug *dbg, int pid, int tid, int sig) {
-	const char *cmd = sdb_fmt ("dk %d", sig);
+	r_strf_var (cmd, 32, "dk %d", sig);
 	dbg->iob.system (dbg->iob.io, cmd);
 	r_cons_flush ();
 	return true;
 }
 
 RDebugPlugin r_debug_plugin_io = {
-	.name = "io",
-	.license = "MIT",
+	.meta = {
+		.name = "io",
+		.author = "pancake",
+		.license = "MIT",
+		.desc = "io debug plugin",
+	},
 	.arch = "any", // TODO: exception!
 	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
 	.step = __io_step,
@@ -152,7 +156,7 @@ RDebugPlugin r_debug_plugin_io = {
 	.step_over = __io_step_over,
 	.canstep = 1,
 #if 0
-	.init = __esil_init,
+	.init_debugger = __esil_init,
 	.contsc = __esil_continue_syscall,
 	.detach = &__esil_detach,
 	.stop = __esil_stop,
