@@ -1,11 +1,9 @@
-/* radare - LGPL - Copyright 2007-2023 - pancake, ret2libc */
+/* radare - LGPL - Copyright 2007-2024 - pancake, ret2libc */
 
 #include <r_flag.h>
-#include <r_util.h>
 #include <r_cons.h>
-#include <stdio.h>
 
-R_LIB_VERSION(r_flag);
+R_LIB_VERSION (r_flag);
 
 #define IS_FI_NOTIN_SPACE(f, i) (r_flag_space_cur (f) && (i)->space != r_flag_space_cur (f))
 #define IS_FI_IN_SPACE(fi, sp) (!(sp) || (fi)->space == (sp))
@@ -93,7 +91,7 @@ static RFlagsAtOffset *r_flag_get_nearest_list(RFlag *f, ut64 off, int dir) {
 }
 
 static void remove_offsetmap(RFlag *f, RFlagItem *item) {
-	r_return_if_fail (f && item);
+	R_RETURN_IF_FAIL (f && item);
 	RFlagsAtOffset *flags = r_flag_get_nearest_list (f, item->offset, 0);
 	if (flags) {
 		r_list_delete_data (flags->flags, item);
@@ -142,7 +140,7 @@ static char *filter_item_name(const char *name) {
 }
 
 static void set_name(RFlagItem *item, char *name) {
-	r_return_if_fail (item && name);
+	R_RETURN_IF_FAIL (item && name);
 	free_item_name (item);
 	item->name = name;
 	free_item_realname (item);
@@ -170,7 +168,7 @@ static bool update_flag_item_offset(RFlag *f, RFlagItem *item, ut64 newoff, bool
 }
 
 static bool update_flag_item_name(RFlag *f, RFlagItem *item, const char *newname, bool force) {
-	r_return_val_if_fail (f && item && newname, false);
+	R_RETURN_VAL_IF_FAIL (f && item && newname, false);
 	if (!force && (item->name == newname || (item->name && !strcmp (item->name, newname)))) {
 		return false;
 	}
@@ -250,7 +248,7 @@ R_API RFlag *r_flag_new(void) {
 }
 
 R_API RFlagItem *r_flag_item_clone(RFlagItem *item) {
-	r_return_val_if_fail (item, NULL);
+	R_RETURN_VAL_IF_FAIL (item, NULL);
 
 	RFlagItem *n = R_NEW0 (RFlagItem);
 	if (!n) {
@@ -387,7 +385,7 @@ static bool print_flag_orig_name(RFlagItem *flag, void *user) {
 
 /* print with r_cons the flag items in the flag f, given as a parameter */
 R_API void r_flag_list(RFlag *f, int rad, const char *pfx) {
-	r_return_if_fail (f);
+	R_RETURN_IF_FAIL (f);
 	bool in_range = false;
 	ut64 range_from = UT64_MAX;
 	ut64 range_to = UT64_MAX;
@@ -478,7 +476,7 @@ R_API void r_flag_list(RFlag *f, int rad, const char *pfx) {
 }
 
 static RFlagItem *evalFlag(RFlag *f, RFlagItem *item) {
-	r_return_val_if_fail (f && item, NULL);
+	R_RETURN_VAL_IF_FAIL (f && item, NULL);
 	if (item->alias) {
 		item->offset = r_num_math (f->num, item->alias);
 	}
@@ -488,7 +486,7 @@ static RFlagItem *evalFlag(RFlag *f, RFlagItem *item) {
 /* return true if flag.* exist at offset. Otherwise, false is returned.
  * For example (f, "sym", 3, 0x1000)*/
 R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64 off) {
-	r_return_val_if_fail (f && flag_prefix, false);
+	R_RETURN_VAL_IF_FAIL (f && flag_prefix, false);
 	RListIter *iter = NULL;
 	RFlagItem *item = NULL;
 	if (f->mask) {
@@ -508,14 +506,14 @@ R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64
 /* return the flag item with name "name" in the RFlag "f", if it exists.
  * Otherwise, NULL is returned. */
 R_API RFlagItem *r_flag_get(RFlag *f, const char *name) {
-	r_return_val_if_fail (f, NULL);
+	R_RETURN_VAL_IF_FAIL (f, NULL);
 	RFlagItem *r = ht_pp_find (f->ht_name, name, NULL);
 	return r? evalFlag (f, r): NULL;
 }
 
 /* return the first flag item that can be found at offset "off", or NULL otherwise */
 R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
-	r_return_val_if_fail (f, NULL);
+	R_RETURN_VAL_IF_FAIL (f, NULL);
 	if (f->mask) {
 		off &= f->mask;
 	}
@@ -527,7 +525,7 @@ R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
  * operands to the function.
  * Pass in the name of each space, in order, followed by a NULL */
 R_API RFlagItem *r_flag_get_by_spaces(RFlag *f, ut64 off, ...) {
-	r_return_val_if_fail (f, NULL);
+	R_RETURN_VAL_IF_FAIL (f, NULL);
 	if (f->mask) {
 		off &= f->mask;
 	}
@@ -612,7 +610,7 @@ static bool isFunctionFlag(const char *n) {
 /* returns the last flag item defined before or at the given offset.
  * NULL is returned if such a item is not found. */
 R_API RFlagItem *r_flag_get_at(RFlag *f, ut64 off, bool closest) {
-	r_return_val_if_fail (f, NULL);
+	R_RETURN_VAL_IF_FAIL (f, NULL);
 	R_CRITICAL_ENTER (f);
 	if (f->mask) {
 		off &= f->mask;
@@ -714,7 +712,7 @@ R_API char *r_flag_get_liststr(RFlag *f, ut64 off) {
 // Set a new flag named `name` at offset `off`. If there's already a flag with
 // the same name, slightly change the name by appending ".%d" as suffix
 R_API RFlagItem *r_flag_set_next(RFlag *f, const char *name, ut64 off, ut32 size) {
-	r_return_val_if_fail (f && name, NULL);
+	R_RETURN_VAL_IF_FAIL (f && name, NULL);
 	if (f->mask) {
 		off &= f->mask;
 	}
@@ -757,7 +755,7 @@ R_API RFlagItem *r_flag_set_inspace(RFlag *f, const char *space, const char *nam
  * The realname of the item will be the same as the name.
  * NULL is returned in case of any errors during the process. */
 R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
-	r_return_val_if_fail (f && name && *name, NULL);
+	R_RETURN_VAL_IF_FAIL (f && name && *name, NULL);
 	if (f->mask) {
 		off &= f->mask;
 	}
@@ -799,28 +797,28 @@ R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
 
 /* add/replace/remove the alias of a flag item */
 R_API void r_flag_item_set_alias(RFlagItem *item, const char *alias) {
-	r_return_if_fail (item);
+	R_RETURN_IF_FAIL (item);
 	free (item->alias);
 	item->alias = R_STR_ISEMPTY (alias)? NULL: strdup (alias);
 }
 
 /* add/replace/remove the comment of a flag item */
 R_API void r_flag_item_set_comment(RFlagItem *item, const char *comment) {
-	r_return_if_fail (item);
+	R_RETURN_IF_FAIL (item);
 	free (item->comment);
 	item->comment = R_STR_ISEMPTY (comment)? NULL: strdup (comment);
 }
 
 /* add/replace/remove the realname of a flag item */
 R_API void r_flag_item_set_realname(RFlagItem *item, const char *realname) {
-	r_return_if_fail (item);
+	R_RETURN_IF_FAIL (item);
 	free_item_realname (item);
 	item->realname = R_STR_ISEMPTY (realname)? NULL: strdup (realname);
 }
 
 /* add/replace/remove the color of a flag item */
 R_API const char *r_flag_item_set_color(RFlagItem *item, const char *color) {
-	r_return_val_if_fail (item, NULL);
+	R_RETURN_VAL_IF_FAIL (item, NULL);
 	free (item->color);
 	item->color = (color && *color) ? strdup (color) : NULL;
 	return item->color;
@@ -829,12 +827,12 @@ R_API const char *r_flag_item_set_color(RFlagItem *item, const char *color) {
 /* change the name of a flag item, if the new name is available.
  * true is returned if everything works well, false otherwise */
 R_API int r_flag_rename(RFlag *f, RFlagItem *item, const char *name) {
-	r_return_val_if_fail (f && item && name && *name, false);
+	R_RETURN_VAL_IF_FAIL (f && item && name && *name, false);
 	return update_flag_item_name (f, item, name, false);
 }
 
 R_API void r_flag_item_set_type(RFlagItem *fi, const char *type) {
-	r_return_if_fail (fi && type);
+	R_RETURN_IF_FAIL (fi && type);
 	free (fi->type);
 	fi->type = strdup (type);
 }
@@ -844,7 +842,7 @@ R_API void r_flag_item_set_type(RFlagItem *fi, const char *type) {
  *
  * NOTE: the item is freed. */
 R_API bool r_flag_unset(RFlag *f, RFlagItem *item) {
-	r_return_val_if_fail (f && item, false);
+	R_RETURN_VAL_IF_FAIL (f && item, false);
 	remove_offsetmap (f, item);
 	ht_pp_delete (f->ht_name, item->name);
 	R_DIRTY (f);
@@ -854,7 +852,7 @@ R_API bool r_flag_unset(RFlag *f, RFlagItem *item) {
 /* unset the first flag item found at offset off.
  * return true if such a flag is found and unset, false otherwise. */
 R_API bool r_flag_unset_off(RFlag *f, ut64 off) {
-	r_return_val_if_fail (f, false);
+	R_RETURN_VAL_IF_FAIL (f, false);
 	RFlagItem *item = r_flag_get_i (f, off);
 	if (item && r_flag_unset (f, item)) {
 		return true;
@@ -881,7 +879,7 @@ static bool unset_foreach(RFlagItem *fi, void *user) {
  * return the number of unset items. -1 on error */
 // XXX This is O(n^n) because unset_globa iterates all flags and unset too.
 R_API int r_flag_unset_glob(RFlag *f, const char *glob) {
-	r_return_val_if_fail (f, -1);
+	R_RETURN_VAL_IF_FAIL (f, -1);
 
 	struct unset_foreach_t u = { .f = f, .n = 0 };
 	r_flag_foreach_glob (f, glob, unset_foreach, &u);
@@ -892,7 +890,7 @@ R_API int r_flag_unset_glob(RFlag *f, const char *glob) {
 /* unset the flag item with the given name.
  * returns true if the item is found and unset, false otherwise. */
 R_API bool r_flag_unset_name(RFlag *f, const char *name) {
-	r_return_val_if_fail (f, false);
+	R_RETURN_VAL_IF_FAIL (f, false);
 	RFlagItem *item = ht_pp_find (f->ht_name, name, NULL);
 	R_DIRTY (f);
 	return item && r_flag_unset (f, item);
@@ -900,7 +898,7 @@ R_API bool r_flag_unset_name(RFlag *f, const char *name) {
 
 /* unset all flag items in the RFlag f */
 R_API void r_flag_unset_all(RFlag *f) {
-	r_return_if_fail (f);
+	R_RETURN_IF_FAIL (f);
 	ht_pp_free (f->ht_name);
 	f->ht_name = ht_pp_new (NULL, ht_free_flag, NULL);
 	r_skiplist_purge (f->by_off);
@@ -932,7 +930,7 @@ static bool flag_relocate_foreach(RFlagItem *fi, void *user) {
 }
 
 R_API int r_flag_relocate(RFlag *f, ut64 off, ut64 off_mask, ut64 to) {
-	r_return_val_if_fail (f, -1);
+	R_RETURN_VAL_IF_FAIL (f, -1);
 	struct flag_relocate_t u = {
 		.f = f,
 		.off = off,
@@ -946,7 +944,7 @@ R_API int r_flag_relocate(RFlag *f, ut64 off, ut64 off_mask, ut64 to) {
 }
 
 R_API bool r_flag_move(RFlag *f, ut64 at, ut64 to) {
-	r_return_val_if_fail (f, false);
+	R_RETURN_VAL_IF_FAIL (f, false);
 	RFlagItem *item = r_flag_get_i (f, at);
 	if (item) {
 		r_flag_set (f, item->name, to, item->size);
@@ -957,7 +955,7 @@ R_API bool r_flag_move(RFlag *f, ut64 at, ut64 to) {
 
 // BIND
 R_API void r_flag_bind(RFlag *f, RFlagBind *fb) {
-	r_return_if_fail (f && fb);
+	R_RETURN_IF_FAIL (f && fb);
 	fb->f = f;
 	fb->exist_at = r_flag_exist_at;
 	fb->get = r_flag_get;
@@ -980,7 +978,7 @@ static bool flag_count_foreach(RFlagItem *fi, void *user) {
 
 R_API int r_flag_count(RFlag *f, const char *glob) {
 	int count = 0;
-	r_return_val_if_fail (f, -1);
+	R_RETURN_VAL_IF_FAIL (f, -1);
 	r_flag_foreach_glob (f, glob, flag_count_foreach, &count);
 	return count;
 }
