@@ -1,14 +1,12 @@
-/* radare - LGPL - Copyright 2016-2021 - pancake */
+/* radare - LGPL - Copyright 2016-2023 - pancake */
 
 #include <r_flag.h>
-#include <r_util.h>
 
-#define DB f->zones
-
-static RFlagZoneItem *r_flag_zone_get (RFlag *f, const char *name) {
+static RFlagZoneItem *r_flag_zone_get(RFlag *f, const char *name) {
 	RListIter *iter;
 	RFlagZoneItem *zi;
-	r_list_foreach (DB, iter, zi) {
+	RList *db = f->zones;
+	r_list_foreach (db, iter, zi) {
 		if (!strcmp (name, zi->name)) {
 			return zi;
 		}
@@ -16,10 +14,11 @@ static RFlagZoneItem *r_flag_zone_get (RFlag *f, const char *name) {
 	return NULL;
 }
 
-static RFlagZoneItem *r_flag_zone_get_inrange (RFlag *f, ut64 from, ut64 to) {
+static RFlagZoneItem *r_flag_zone_get_inrange(RFlag *f, ut64 from, ut64 to) {
 	RListIter *iter;
 	RFlagZoneItem *zi;
-	r_list_foreach (DB, iter, zi) {
+	RList *db = f->zones;
+	r_list_foreach (db, iter, zi) {
 		if (R_BETWEEN (from, zi->from, to)) {
 			return zi;
 		}
@@ -28,7 +27,7 @@ static RFlagZoneItem *r_flag_zone_get_inrange (RFlag *f, ut64 from, ut64 to) {
 }
 
 R_API bool r_flag_zone_add(RFlag *f, const char *name, ut64 addr) {
-	r_return_val_if_fail (f && name && *name, false);
+	R_RETURN_VAL_IF_FAIL (f && name && *name, false);
 	RFlagZoneItem *zi = r_flag_zone_get (f, name);
 	if (zi) {
 		if (addr < zi->from) {
@@ -38,13 +37,15 @@ R_API bool r_flag_zone_add(RFlag *f, const char *name, ut64 addr) {
 			zi->to = addr;
 		}
 	} else {
-		if (!DB) {
+		if (!f->zones) {
 			r_flag_zone_reset (f);
 		}
 		zi = R_NEW0 (RFlagZoneItem);
-		zi->name = strdup (name);
-		zi->from = zi->to = addr;
-		r_list_append (DB, zi);
+		if (zi) {
+			zi->name = strdup (name);
+			zi->from = zi->to = addr;
+			r_list_append (f->zones, zi);
+		}
 	}
 	return true;
 }
@@ -58,9 +59,10 @@ R_API bool r_flag_zone_reset(RFlag *f) {
 R_API bool r_flag_zone_del(RFlag *f, const char *name) {
 	RListIter *iter;
 	RFlagZoneItem *zi;
-	r_list_foreach (DB, iter, zi) {
+	RList *db = f->zones;
+	r_list_foreach (db, iter, zi) {
 		if (!strcmp (name, zi->name)) {
-			r_list_delete (DB, iter);
+			r_list_delete (db, iter);
 			return true;
 		}
 	}
@@ -79,8 +81,9 @@ R_API bool r_flag_zone_around(RFlag *f, ut64 addr, const char **prev, const char
 	RFlagZoneItem *zi;
 	*prev = *next = NULL;
 	ut64 h = UT64_MAX, l = 0LL;
+	RList *db = f->zones;
 
-	r_list_foreach (DB, iter, zi) {
+	r_list_foreach (db, iter, zi) {
 		if (zi->from > addr) {
 			if (h == UT64_MAX) {
 				h = zi->from;
@@ -147,7 +150,8 @@ R_API RList *r_flag_zone_barlist(RFlag *f, ut64 from, ut64 bsize, int rows) {
 R_API bool r_flag_zone_list(RFlag *f, int mode) {
 	RListIter *iter;
 	RFlagZoneItem *zi;
-	r_list_foreach (DB, iter, zi) {
+	RList *db = f->zones;
+	r_list_foreach (db, iter, zi) {
 		if (mode == '*') {
 			f->cb_printf ("fz %s @ 0x08%"PFMT64x"\n", zi->name, zi->from);
 			f->cb_printf ("f %s %"PFMT64d" 0x08%"PFMT64x"\n", zi->name,
