@@ -1,25 +1,23 @@
-// Copyright (c) 2016-2021 - LGPL, SkUaTeR, All rights reserved.
+// Copyright (c) 2016-2022 - LGPL, SkUaTeR, All rights reserved.
 
 #include <r_io.h>
 #include <r_lib.h>
-#include <r_util.h>
 #include <libbochs.h>
 
 typedef struct {
 	libbochs_t desc;
 } RIOBochs;
 
-static libbochs_t *desc = NULL;
-static RIODesc *riobochs = NULL;
+static R_TH_LOCAL libbochs_t *desc = NULL;
+static R_TH_LOCAL RIODesc *riobochs = NULL;
 extern RIOPlugin r_io_plugin_bochs; // forward declaration
 
 static bool __plugin_open(RIO *io, const char *file, bool many) {
-	return !strncmp (file, "bochs://", strlen ("bochs://"));
+	return r_str_startswith (file, "bochs://");
 }
 
 static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	RIOBochs  *riob;
-	lprintf("io_open\n");
 	const char *i;
 	char * fileBochs = NULL;
 	char * fileCfg = NULL;
@@ -28,7 +26,6 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		return NULL;
 	}
 	if (r_sandbox_enable (false)) {
-		eprintf ("sandbox exit\n");
 		return NULL;
 	}
 	if (riobochs) {
@@ -42,7 +39,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		fileCfg = strdup (i + 1);
 	} else {
 		free (fileCfg);
-		eprintf ("Error can't find :\n");
+		R_LOG_ERROR ("can't find :");
 		return NULL;
 	}
 	riob = R_NEW0 (RIOBochs);
@@ -52,11 +49,10 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		desc = &riob->desc;
 		riobochs = r_io_desc_new (io, &r_io_plugin_bochs, file, rw, mode, riob);
 		//riogdb = r_io_desc_new (&r_io_plugin_gdb, riog->desc.sock->fd, file, rw, mode, riog);
-		free(fileBochs);
-		free(fileCfg);
+		free (fileBochs);
+		free (fileCfg);
 		return riobochs;
 	}
-	lprintf ("bochsio.open: Cannot connect to bochs.\n");
 	free (riob);
 	free (fileBochs);
 	free (fileCfg);
@@ -64,12 +60,10 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 }
 
 static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
-	lprintf("io_write\n");
 	return -1;
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
-	lprintf("io_seek %016"PFMT64x" \n",offset);
 	return offset;
 }
 
@@ -79,7 +73,6 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	if (!desc || !desc->data) {
 		return -1;
 	}
-	lprintf ("io_read ofs= %016"PFMT64x" count= %x\n", io->off, count);
 	bochs_read (desc,addr,count,buf);
 	return count;
 }
@@ -90,25 +83,25 @@ static bool __close(RIODesc *fd) {
 }
 
 static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
-        lprintf ("system command (%s)\n", cmd);
-        if (!strcmp (cmd, "help")) {
-                lprintf ("Usage: =!cmd args\n"
-                        " =!:<bochscmd>      - Send a bochs command.\n"
-                        " =!dobreak          - pause bochs.\n");
-		lprintf ("io_system: Enviando commando bochs\n");
+	if (*cmd == '?' || !strcmp (cmd, "help")) {
+		eprintf ("Usage: :cmd args\n"
+			" ::<bochscmd>      - Send a bochs command.\n"
+			" :dobreak	  - pause bochs.\n");
 		bochs_send_cmd (desc, &cmd[1], true);
 		io->cb_printf ("%s\n", desc->data);
-	} else if (!strncmp (cmd, "dobreak", 7)) {
+	} else if (r_str_startswith (cmd, "dobreak")) {
 		bochs_cmd_stop (desc);
 		io->cb_printf ("%s\n", desc->data);
 	}
-        return NULL;
+	return NULL;
 }
 
 RIOPlugin r_io_plugin_bochs = {
-	.name = "bochs",
-	.desc = "Attach to a BOCHS debugger instance",
-	.license = "LGPL3",
+	.meta = {
+		.name = "bochs",
+		.desc = "Attach to a BOCHS debugger instance",
+		.license = "LGPL3",
+	},
 	.uris = "bochs://",
 	.open = __open,
 	.close = __close,
