@@ -1,9 +1,7 @@
-/* radare - LGPL - Copyright 2008-2021 - pancake */
+/* radare - LGPL - Copyright 2008-2022 - pancake */
 
-#include <r_userconf.h>
 #include <r_io.h>
 #include <r_lib.h>
-#include <stdio.h>
 
 typedef struct r_io_mmo_t {
 	char *filename;
@@ -13,7 +11,7 @@ typedef struct r_io_mmo_t {
 	bool rawio;
 	bool nocache;
 	RBuffer *buf;
-	RIO * io_backref;
+	RIO *io_backref;
 } RIOMMapFileObj;
 
 static int __io_posix_open(const char *file, int perm, int mode) {
@@ -24,7 +22,7 @@ static int __io_posix_open(const char *file, int perm, int mode) {
 	if (r_file_is_directory (file)) {
 		return -1;
 	}
-#if __WINDOWS__
+#if R2__WINDOWS__
 	// probably unnecessary to have this ifdef nowadays windows is posix enough
 	if (perm & R_PERM_W) {
 		fd = r_sandbox_open (file, O_RDWR, 0);
@@ -45,14 +43,14 @@ static int __io_posix_open(const char *file, int perm, int mode) {
 
 static ut64 r_io_def_mmap_seek(RIO *io, RIOMMapFileObj *mmo, ut64 offset, int whence) {
 	if (!mmo) {
-		return UT64_MAX;
+		return UT64_MAX - 1;
 	}
 	if (mmo->rawio) {
 		io->off = lseek (mmo->fd, offset, whence);
 		return io->off;
 	}
 	if (!mmo->buf) {
-		return UT64_MAX;
+		return UT64_MAX - 1;
 	}
 	io->off = r_buf_seek (mmo->buf, offset, whence);
 	return io->off;
@@ -60,13 +58,11 @@ static ut64 r_io_def_mmap_seek(RIO *io, RIOMMapFileObj *mmo, ut64 offset, int wh
 
 static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 	RIO* io = mmo->io_backref;
-	ut64 cur;
+	ut64 cur = 0;
 	if (mmo->buf) {
 		cur = r_buf_tell (mmo->buf);
 		r_buf_free (mmo->buf);
 		mmo->buf = NULL;
-	} else {
-		cur = 0;
 	}
 	st64 sz = r_file_size (mmo->filename);
 	if (sz > ST32_MAX) {
@@ -86,20 +82,18 @@ static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 	if (mmo->buf) {
 		r_io_def_mmap_seek (io, mmo, cur, SEEK_SET);
 		return true;
-	} else {
-		mmo->rawio = true;
-		mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
-		if (mmo->nocache) {
-#ifdef F_NOCACHE
-			fcntl (mmo->fd, F_NOCACHE, 1);
-#endif
-		}
-		return mmo->fd != -1;
 	}
-	return false;
+	mmo->rawio = true;
+	mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
+	if (mmo->nocache) {
+#ifdef F_NOCACHE
+		fcntl (mmo->fd, F_NOCACHE, 1);
+#endif
+	}
+	return mmo->fd != -1;
 }
 
-static void r_io_def_mmap_free (RIOMMapFileObj *mmo) {
+static void r_io_def_mmap_free(RIOMMapFileObj *mmo) {
 	if (mmo) {
 		free (mmo->filename);
 		r_buf_free (mmo->buf);
@@ -109,7 +103,7 @@ static void r_io_def_mmap_free (RIOMMapFileObj *mmo) {
 }
 
 RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, int perm, int mode) {
-	r_return_val_if_fail (io && filename, NULL);
+	R_RETURN_VAL_IF_FAIL (io && filename, NULL);
 	RIOMMapFileObj *mmo = R_NEW0 (RIOMMapFileObj);
 	if (!mmo) {
 		return NULL;
@@ -148,17 +142,17 @@ RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, in
 }
 
 static bool r_io_def_mmap_check_default(const char *filename) {
-	r_return_val_if_fail (filename && *filename, false);
+	R_RETURN_VAL_IF_FAIL (filename, false);
 	if (r_str_startswith (filename, "file://")) {
 		filename += strlen ("file://");
 	}
-	const char * peekaboo = (!strncmp (filename, "nocache://", 10))
+	const char * peekaboo = r_str_startswith (filename, "nocache://")
 		? NULL : strstr (filename, "://");
 	return (!peekaboo || (peekaboo - filename) > 10);
 }
 
 static int r_io_def_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
-	r_return_val_if_fail (fd && fd->data && buf, -1);
+	R_RETURN_VAL_IF_FAIL (fd && fd->data && buf, -1);
 	if (io->off == UT64_MAX) {
 		memset (buf, 0xff, count);
 		return count;
@@ -187,7 +181,7 @@ static int r_io_def_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 }
 
 static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
-	r_return_val_if_fail (io && fd && fd->data && buf, -1);
+	R_RETURN_VAL_IF_FAIL (io && fd && fd->data && buf, -1);
 
 	int len = -1;
 	ut64 addr = io->off;
@@ -204,7 +198,7 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 		if (!(mmo->perm & R_PERM_W)) {
 			return -1;
 		}
-		if ( (count + addr > r_buf_size (mmo->buf)) || r_buf_size (mmo->buf) == 0) {
+		if ((count + addr > r_buf_size (mmo->buf)) || r_buf_size (mmo->buf) == 0) {
 			ut64 sz = count + addr;
 			r_file_truncate (mmo->filename, sz);
 		}
@@ -219,14 +213,14 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 		len = write (fd->fd, buf, count);
 	}
 	if (!r_io_def_mmap_refresh_def_mmap_buf (mmo) ) {
-		eprintf ("io_def_mmap: failed to refresh the def_mmap backed buffer.\n");
+		R_LOG_ERROR ("io_def_mmap: failed to refresh the def_mmap backed buffer");
 		// XXX - not sure what needs to be done here (error handling).
 	}
 	return len;
 }
 
 static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int perm, int mode) {
-	r_return_val_if_fail (io && file, NULL);
+	R_RETURN_VAL_IF_FAIL (io && file, NULL);
 #if __wasi__
 	RIOPlugin *_plugin = r_io_plugin_resolve (io, (const char *)"slurp://", false);
 	if (!_plugin || !_plugin->open) {
@@ -257,10 +251,10 @@ static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int perm, int mode
 static int r_io_def_mmap_truncate(RIOMMapFileObj *mmo, ut64 size) {
 	bool res = r_file_truncate (mmo->filename, size);
 	if (res && !r_io_def_mmap_refresh_def_mmap_buf (mmo) ) {
-		eprintf ("r_io_def_mmap_truncate: Error trying to refresh the def_mmap'ed file.");
+		R_LOG_ERROR ("r_io_def_mmap_truncate: Can't refresh the def_mmap'ed file");
 		res = false;
 	} else if (!res) {
-		eprintf ("r_io_def_mmap_truncate: Error trying to resize the file.");
+		R_LOG_ERROR ("r_io_def_mmap_truncate: Error trying to resize the file");
 	}
 	return res;
 }
@@ -271,7 +265,7 @@ static bool __plugin_open_default(RIO *io, const char *file, bool many) {
 
 // default open should permit opening
 static RIODesc *__open_default(RIO *io, const char *file, int perm, int mode) {
-	if (r_io_def_mmap_check_default (file)) {
+	if (*file && r_io_def_mmap_check_default (file)) {
 		return r_io_def_mmap_open (io, file, perm, mode);
 	}
 	return NULL;
@@ -286,12 +280,12 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int len) {
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
-	r_return_val_if_fail (fd && fd->data, UT64_MAX);
+	R_RETURN_VAL_IF_FAIL (fd && fd->data, UT64_MAX);
 	return r_io_def_mmap_seek (io, (RIOMMapFileObj *)fd->data, offset, whence);
 }
 
 static bool __close(RIODesc *fd) {
-	r_return_val_if_fail (fd, false);
+	R_RETURN_VAL_IF_FAIL (fd, false);
 	if (fd->data) {
 		r_io_def_mmap_free ((RIOMMapFileObj *) fd->data);
 		fd->data = NULL;
@@ -300,7 +294,7 @@ static bool __close(RIODesc *fd) {
 }
 
 static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
-	r_return_val_if_fail (io && fd && fd->data, false);
+	R_RETURN_VAL_IF_FAIL (io && fd && fd->data, false);
 	RIOMMapFileObj *mmo = fd->data;
 	if (!(mmo->perm & R_PERM_W)) {
 		return false;
@@ -308,9 +302,9 @@ static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
 	return r_io_def_mmap_truncate (mmo, size);
 }
 
-#if __UNIX__
-static bool __is_blockdevice (RIODesc *desc) {
-	r_return_val_if_fail (desc && desc->data, false);
+#if R2__UNIX__
+static bool __is_blockdevice(RIODesc *desc) {
+	R_RETURN_VAL_IF_FAIL (desc && desc->data, false);
 	RIOMMapFileObj *mmo = desc->data;
 	struct stat buf;
 	if (fstat (mmo->fd, &buf) == -1) {
@@ -321,9 +315,11 @@ static bool __is_blockdevice (RIODesc *desc) {
 #endif
 
 RIOPlugin r_io_plugin_default = {
-	.name = "default",
-	.desc = "Open local files",
-	.license = "LGPL3",
+	.meta = {
+		.name = "default",
+		.desc = "Open local files",
+		.license = "LGPL3",
+	},
 	.uris = "file://,nocache://",
 	.open = __open_default,
 	.close = __close,
@@ -332,7 +328,7 @@ RIOPlugin r_io_plugin_default = {
 	.seek = __lseek,
 	.write = __write,
 	.resize = __resize,
-#if __UNIX__
+#if R2__UNIX__
 	.is_blockdevice = __is_blockdevice,
 #endif
 };
