@@ -6,7 +6,7 @@
 #include <r_util.h>
 #include <sys/types.h>
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 #include "io_r2k_windows.h"
 #elif defined (__linux__) && !defined (__GNU__)
 #include "io_r2k_linux.h"
@@ -16,7 +16,7 @@ int r2k_struct; // dummy
 #endif
 
 int r2k__write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	//eprintf("writing to: 0x%"PFMT64x" len: %x\n",io->off, count);
 	return WriteKernelMemory (io->off, buf, count);
 #elif defined (__linux__) && !defined (__GNU__)
@@ -38,7 +38,7 @@ int r2k__write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 }
 
 static int r2k__read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	return ReadKernelMemory (io->off, buf, count);
 #elif defined (__linux__) && !defined (__GNU__)
 	switch (r2k_struct.beid) {
@@ -61,7 +61,7 @@ static int r2k__read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 }
 
 static bool r2k__close(RIODesc *fd) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (gHandleDriver) {
 		CloseHandle (gHandleDriver);
 		StartStopService (TEXT ("r2k"),TRUE);
@@ -71,26 +71,26 @@ static bool r2k__close(RIODesc *fd) {
 		close ((int)(size_t)fd->data);
 	}
 #else
-	eprintf ("TODO: r2k not implemented for this plataform.\n");
+	R_LOG_TODO ("r2k not implemented for this plataform");
 #endif
 	return true;
 }
 
 static ut64 r2k__lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	return (!whence) ? offset : whence == 1
-		? io->off + offset : UT64_MAX;
+		? io->off + offset : UT64_MAX - 1;
 }
 
 static bool r2k__plugin_open(RIO *io, const char *pathname, bool many) {
-	return (!strncmp (pathname, "r2k://", 6));
+	return r_str_startswith (pathname, "r2k://");
 }
 
 static char *r2k__system(RIO *io, RIODesc *fd, const char *cmd) {
 	if (!strcmp (cmd, "")) {
 		return NULL;
 	}
-	if (!strncmp (cmd, "mod", 3)) {
-#if __WINDOWS__
+	if (r_str_startswith (cmd, "mod")) {
+#if R2__WINDOWS__
 		GetSystemModules (io);
 #endif
 	} else {
@@ -98,19 +98,19 @@ static char *r2k__system(RIO *io, RIODesc *fd, const char *cmd) {
 		(void)run_ioctl_command (io, fd, cmd);
 		return NULL;
 #else
-		eprintf ("Try: '=!mod'\n    '.=!mod'\n");
+		eprintf ("Try: ':mod'\n    '.:mod'\n");
 #endif
 	}
 	return NULL;
 }
 
 static RIODesc *r2k__open(RIO *io, const char *pathname, int rw, int mode) {
-	if (!strncmp (pathname, "r2k://", 6)) {
+	if (r_str_startswith (pathname, "r2k://")) {
 		rw |= R_PERM_WX;
-#if __WINDOWS__
+#if R2__WINDOWS__
 		RIOW32 *w32 = R_NEW0 (RIOW32);
-		if (Init (&pathname[6]) == FALSE) {
-			eprintf ("r2k__open: Error cant init driver: %s\n", &pathname[6]);
+		if (!w32 || !Init (pathname + 6)) {
+			R_LOG_ERROR ("r2k__open: Error cant init driver: %s", pathname + 6);
 			free (w32);
 			return NULL;
 		}
@@ -135,10 +135,12 @@ static RIODesc *r2k__open(RIO *io, const char *pathname, int rw, int mode) {
 }
 
 RIOPlugin r_io_plugin_r2k = {
-	.name = "r2k",
-	.desc = "Kernel access API io",
+	.meta = {
+		.name = "r2k",
+		.desc = "Kernel access API io",
+		.license = "LGPL3",
+	},
 	.uris = "r2k://",
-	.license = "LGPL3",
 	.open = r2k__open,
 	.close = r2k__close,
 	.read = r2k__read,
