@@ -81,17 +81,17 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	switch (whence) {
 	case SEEK_SET: return offset;
 	case SEEK_CUR: return io->off + offset;
-	case SEEK_END: return UT64_MAX;
+	case SEEK_END: return UT64_MAX - 1;
 	}
 	return offset;
 }
 
 static bool __plugin_open(RIO *io, const char *pathname, bool many) {
 	const char *uri = "r2web://";
-	return (!strncmp (pathname, uri, strlen (uri)));
+	return r_str_startswith (pathname, uri);
 }
 
-static inline int getmalfd (RIOR2Web *mal) {
+static inline int getmalfd(RIOR2Web *mal) {
 	return 0xfffffff & (int)(size_t)mal;
 }
 
@@ -128,7 +128,7 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 		free (mal);
 		free (out);
 		free (path);
-		eprintf ("Error: Try http://localhost:9090/cmd/");
+		R_LOG_WARN ("Try http://localhost:9090/cmd/");
 	}
 	return NULL;
 }
@@ -151,10 +151,12 @@ static char *__system(RIO *io, RIODesc *fd, const char *command) {
 }
 
 RIOPlugin r_io_plugin_r2web = {
-	.name = "r2web",
-	.desc = "r2web io client plugin",
+	.meta = {
+		.name = "r2web",
+		.desc = "r2web io client plugin",
+		.license = "LGPL3",
+	},
 	.uris = "r2web://",
-	.license = "LGPL3",
 	.open = __open,
 	.close = __close,
 	.read = __read,
