@@ -7,7 +7,7 @@
 #include <r_cons.h>
 #include <r_util.h>
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 
 #include <windows.h>
 #include <tlhelp32.h>
@@ -89,19 +89,17 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int len) {
 }
 
 static bool __plugin_open(RIO *io, const char *file, bool many) {
-	if (!strncmp (file, "attach://", 9)) {
+	if (r_str_startswith (file, "attach://")) {
 		return true;
 	}
-	return !strncmp (file, "w32dbg://", 9);
+	return r_str_startswith (file, "w32dbg://");
 }
 
 static int __w32_first_thread(int pid) {
-	HANDLE th;
-	HANDLE thid;
-	THREADENTRY32 te32;
-	te32.dwSize = sizeof (THREADENTRY32);
-
-	th = CreateToolhelp32Snapshot (TH32CS_SNAPTHREAD, pid);
+	THREADENTRY32 te32 = {
+		.dwSize = sizeof (THREADENTRY32)
+	};
+	HANDLE th = CreateToolhelp32Snapshot (TH32CS_SNAPTHREAD, pid);
 	if (th == INVALID_HANDLE_VALUE) {
 		return -1;
 	}
@@ -112,7 +110,7 @@ static int __w32_first_thread(int pid) {
 	do {
 		/* get all threads of process */
 		if (te32.th32OwnerProcessID == pid) {
-			thid = OpenThread (THREAD_ALL_ACCESS, 0, te32.th32ThreadID);
+			HANDLE thid = OpenThread (THREAD_ALL_ACCESS, 0, te32.th32ThreadID);
 			if (!thid) {
 				r_sys_perror ("__w32_first_thread/OpenThread");
 				goto err_first_th;
@@ -122,7 +120,7 @@ static int __w32_first_thread(int pid) {
 		}
 	} while (Thread32Next (th, &te32));
 err_first_th:
-	eprintf ("Could not find an active thread for pid %d\n", pid);
+	R_LOG_ERROR ("Could not find an active thread for pid %d", pid);
 	CloseHandle (th);
 	return pid;
 }
@@ -143,7 +141,7 @@ static int __open_proc(RIO *io, int pid, bool attach) {
 	RW32Dw *wrap = (RW32Dw *)io->dbgwrap;
 	wrap->pi.dwProcessId = pid;
 	if (attach) {
-		/* Attach to the process */	
+		/* Attach to the process */
 		wrap->params.type = W32_ATTACH;
 		r_w32dw_waitret (wrap);
 		if (!r_w32dw_ret (wrap)) {
@@ -162,7 +160,7 @@ static int __open_proc(RIO *io, int pid, bool attach) {
 			goto att_exit;
 		}
 		if (de.dwDebugEventCode != CREATE_PROCESS_DEBUG_EVENT) {
-			eprintf ("exception code 0x%04x\n", (ut32)de.dwDebugEventCode);
+			R_LOG_ERROR ("exception code 0x%04x", (ut32)de.dwDebugEventCode);
 			goto att_exit;
 		}
 		wrap->winbase = (ut64)de.u.CreateProcessInfo.lpBaseOfImage;
@@ -181,7 +179,11 @@ att_exit:
 static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	if (__plugin_open (io, file, 0)) {
 		RIODesc *ret;
-		if (__open_proc (io, atoi (file + 9), !strncmp (file, "attach://", 9)) == -1) {
+		if (!r_sandbox_check (R_SANDBOX_GRAIN_EXEC)) {
+			return NULL;
+		}
+		bool must_attach = r_str_startswith (file, "attach://");
+		if (__open_proc (io, atoi (file + 9), must_attach) == -1) {
 			return NULL;
 		}
 		RW32Dw *wrap = io->dbgwrap;
@@ -208,7 +210,7 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 		io->off += (int)offset;
 		break;
 	case 2: // end
-		io->off = UT64_MAX;
+		io->off = UT64_MAX - 1; // UT64_MAX reserved for error case
 		break;
 	}
 	return io->off;
@@ -230,7 +232,7 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	/* XXX ugly hack for testing purposes */
 	if (!strcmp (cmd, "")) {
 		// do nothing
-	} else if (!strncmp (cmd, "pid", 3)) {
+	} else if (r_str_startswith (cmd, "pid")) {
 		if (cmd[3] == ' ') {
 			int pid = atoi (cmd + 3);
 			if (pid > 0 && pid != wrap->pi.dwThreadId && pid != wrap->pi.dwProcessId) {
@@ -238,13 +240,13 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 				if (ht) {
 					wrap->pi.hThread = ht;
 				} else {
-					eprintf ("Cannot attach to %d (%s)\n", pid, cmd);
+					R_LOG_ERROR ("Cannot attach to %d (%s)", pid, cmd);
 				}
 			}
 		}
 		return r_str_newf ("%lu", wrap->pi.dwProcessId);
 	} else {
-		eprintf ("Try: '=!pid'\n");
+		eprintf ("Try: ':pid'\n");
 	}
 	return NULL;
 }
@@ -272,9 +274,11 @@ static bool __getbase(RIODesc *fd, ut64 *base) {
 }
 
 RIOPlugin r_io_plugin_w32dbg = {
-	.name = "w32dbg",
-	.desc = "w32 debugger io plugin",
-	.license = "LGPL3",
+	.meta = {
+		.name = "w32dbg",
+		.desc = "w32 debugger io plugin",
+		.license = "LGPL3",
+	},
 	.uris = "w32dbg://,attach://",
 	.open = __open,
 	.close = __close,
@@ -290,7 +294,9 @@ RIOPlugin r_io_plugin_w32dbg = {
 };
 #else
 RIOPlugin r_io_plugin_w32dbg = {
-	.name = NULL
+	.meta = {
+		.name = NULL
+	},
 };
 #endif
 
