@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2018 - pancake */
+/* radare - LGPL - Copyright 2007-2022 - pancake */
 
 #include <r_io.h>
 
@@ -127,7 +127,7 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 
 	idx = io->undo.idx;
 	start = (idx - undos + R_IO_UNDOS) % R_IO_UNDOS;
-	end = (idx + redos + 1 - 1) % R_IO_UNDOS; // +1 slot for current position, -1 due to inclusive end
+	end = (idx + redos) % R_IO_UNDOS;
 
 	j = 0;
 	switch (mode) {
@@ -149,6 +149,22 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 				io->cb_printf ("0x%"PFMT64x"%s", addr, notLast? " > ": "");
 			}
 			break;
+		case 'r':
+			{
+				char *cmt = io->coreb.cmdstrf (io->coreb.core, "fd 0x%08"PFMT64x, addr);
+				r_str_trim (cmt);
+				if (j < undos) {
+					io->cb_printf ("0x%08"PFMT64x" ; %ds- # %s\n", addr, idx + 1, cmt);
+				} else if (j == undos && j != 0 && redos != 0) {
+					io->cb_printf ("0x%08"PFMT64x" ; # CUR %s\n", addr, cmt);
+				} else if (j != undos) {
+					io->cb_printf ("0x%08"PFMT64x" ; %ds+ # %s\n", addr, idx + 1, cmt);
+				} else if (addr != 0) {
+					io->cb_printf ("0x%08"PFMT64x" ; # CUR %s\n", addr, cmt);
+				}
+				free (cmt);
+			}
+			break;
 		case '*':
 			if (j < undos) {
 				io->cb_printf ("f undo_%d @ 0x%"PFMT64x"\n", idx, addr);
@@ -160,7 +176,7 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 			break;
 		case 0:
 			if (list) {
-				RIOUndos  *u = R_NEW0 (RIOUndos);
+				RIOUndos *u = R_NEW0 (RIOUndos);
 				if (u) {
 					if (!(j == undos && redos == 0)) {
 						// Current position gets pushed before seek, so there
@@ -237,7 +253,7 @@ R_API void r_io_wundo_list(RIO *io) {
 
 	if (io->undo.w_init) {
 		r_list_foreach (io->undo.w_list, iter, u) {
-			io->cb_printf ("%02d %c %zu %08" PFMT64x ": ", i, u->set ? '+' : '-', u->len, u->off);
+			io->cb_printf ("%02d %c %u %08" PFMT64x ": ", i, u->set ? '+' : '-', (int)u->len, u->off);
 			len = (u->len > BW) ? BW : u->len;
 			for (j = 0; j < len; j++) {
 				io->cb_printf ("%02x ", u->o[j]);
@@ -277,13 +293,13 @@ R_API void r_io_wundo_apply_all(RIO *io, int set) {
 	RIOUndoWrite *u;
 
 	r_list_foreach_prev (io->undo.w_list, iter, u) {
-		r_io_wundo_apply (io, u, set); //UNDO_WRITE_UNSET);
-		eprintf ("%s 0x%08"PFMT64x"\n", set?"redo":"undo", u->off);
+		r_io_wundo_apply (io, u, set);
+		R_LOG_INFO ("%s 0x%08"PFMT64x, set? "redo": "undo", u->off);
 	}
 }
 
 /* sets or unsets the writes done */
-/* if ( set == 0 ) unset(n) */
+/* if (set == 0) unset(n) */
 R_API int r_io_wundo_set(RIO *io, int n, int set) {
 	RListIter *iter;
 	RIOUndoWrite *u = NULL;
@@ -298,9 +314,9 @@ R_API int r_io_wundo_set(RIO *io, int n, int set) {
 			r_io_wundo_apply (io, u, set);
 			return true;
 		}
-		eprintf ("invalid undo-write index\n");
+		R_LOG_ERROR ("invalid undo-write index");
 	} else {
-		eprintf ("no writes done\n");
+		R_LOG_INFO ("no writes done");
 	}
 	return false;
 }
