@@ -1,10 +1,9 @@
-/* radare - LGPL - Copyright 2014-2021 - pancake */
+/* radare - LGPL - Copyright 2014-2024 - pancake */
 
 /* this helper api is here because it depends on r_util and r_socket */
 /* we should find a better place for it. r_io? */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#define R_LOG_ORIGIN "socket.run"
+
 #include <fcntl.h>
 #include <r_socket.h>
 #include <r_util.h>
@@ -12,6 +11,7 @@
 #include <r_cons.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <errno.h>
 
 #if __APPLE__ && LIBC_HAVE_FORK
 #if !__POWERPC__
@@ -31,10 +31,10 @@
 #include <mach-o/nlist.h>
 #endif
 
-#if __UNIX__
+#if R2__UNIX__
 #include <sys/ioctl.h>
-#include <sys/resource.h>
 #ifndef __wasi__
+#include <sys/resource.h>
 #include <grp.h>
 #endif
 #include <errno.h>
@@ -59,11 +59,10 @@
 #define pid_t int
 #endif
 
-
 #if HAVE_PTY
-static int (*dyn_openpty)(int *amaster, int *aslave, char *name, struct termios *termp, struct winsize *winp) = NULL;
-static int (*dyn_login_tty)(int fd) = NULL;
-static id_t (*dyn_forkpty)(int *amaster, char *name, struct termios *termp, struct winsize *winp) = NULL;
+static int(*dyn_openpty)(int *amaster, int *aslave, char *name, struct termios *termp, struct winsize *winp) = NULL;
+static int(*dyn_login_tty)(int fd) = NULL;
+static id_t(*dyn_forkpty)(int *amaster, char *name, struct termios *termp, struct winsize *winp) = NULL;
 static void dyn_init(void) {
 	if (!dyn_openpty) {
 		dyn_openpty = r_lib_dl_sym (NULL, "openpty");
@@ -74,12 +73,12 @@ static void dyn_init(void) {
 	if (!dyn_forkpty) {
 		dyn_forkpty = r_lib_dl_sym (NULL, "forkpty");
 	}
-#if __UNIX__
+#if R2__UNIX__
 	// attempt to fall back on libutil if we failed to load anything
 	if (!(dyn_openpty && dyn_login_tty && dyn_forkpty)) {
 		void *libutil;
 		if (!(libutil = r_lib_dl_open ("libutil." R_LIB_EXT))) {
-			eprintf ("[ERROR] rarun2: Could not find PTY utils, failed to load %s\n", "libutil." R_LIB_EXT);
+			R_LOG_ERROR ("rarun2: Could not find PTY utils, failed to load libutil" R_LIB_EXT);
 			return;
 		}
 		if (!dyn_openpty) {
@@ -98,10 +97,10 @@ static void dyn_init(void) {
 
 #endif
 
-R_API RRunProfile *r_run_new(const char *str) {
+R_API RRunProfile *r_run_new(R_NULLABLE const char *str) {
 	RRunProfile *p = R_NEW0 (RRunProfile);
 	if (p) {
-		r_run_reset (p);
+		r_run_reset (p); // TODO: rename to r_run_init
 		if (str) {
 			r_run_parsefile (p, str);
 		}
@@ -110,13 +109,37 @@ R_API RRunProfile *r_run_new(const char *str) {
 }
 
 R_API void r_run_reset(RRunProfile *p) {
-	r_return_if_fail (p);
+	R_RETURN_IF_FAIL (p);
+	int i;
+	for (i = 0; i < R_RUN_PROFILE_NARGS; i++) {
+		R_FREE (p->_args[i]);
+	}
+	R_FREE (p->_system);
+	R_FREE (p->_program);
+	R_FREE (p->_runlib);
+	R_FREE (p->_runlib_fcn);
+	R_FREE (p->_stdio);
+	R_FREE (p->_stdin);
+	R_FREE (p->_stdout);
+	R_FREE (p->_stderr);
+	R_FREE (p->_chgdir);
+	R_FREE (p->_chroot);
+	R_FREE (p->_libpath);
+	R_FREE (p->_preload);
+	R_FREE (p->_pidfile);
+	R_FREE (p->_connect);
+	R_FREE (p->_listen);
+	R_FREE (p->_input);
+	R_FREE (p->_setuid);
+	R_FREE (p->_seteuid);
+	R_FREE (p->_setgid);
+	R_FREE (p->_setegid);
 	memset (p, 0, sizeof (RRunProfile));
 	p->_aslr = -1;
 }
 
 R_API bool r_run_parse(RRunProfile *pf, const char *profile) {
-	r_return_val_if_fail (pf && profile, false);
+	R_RETURN_VAL_IF_FAIL (pf && profile, false);
 	char *p, *o, *str = strdup (profile);
 	if (!str) {
 		return false;
@@ -135,6 +158,7 @@ R_API bool r_run_parse(RRunProfile *pf, const char *profile) {
 }
 
 R_API void r_run_free(RRunProfile *r) {
+	int i;
 	if (r) {
 		free (r->_system);
 		free (r->_program);
@@ -147,14 +171,24 @@ R_API void r_run_free(RRunProfile *r) {
 		free (r->_chgdir);
 		free (r->_chroot);
 		free (r->_libpath);
-		free (r->_preload);
+		r_list_free (r->_preload);
+		free (r->_pidfile);
+		free (r->_connect);
+		free (r->_listen);
+		free (r->_input);
+		free (r->_setuid);
+		free (r->_seteuid);
+		free (r->_setgid);
+		free (r->_setegid);
+		for (i = 0; i < R_RUN_PROFILE_NARGS; i++) {
+			free (r->_args[i]);
+		}
 		free (r);
 	}
 }
 
-#if __UNIX__
+#if R2__UNIX__ && !__wasi__ && !defined(__serenity__)
 static void set_limit(int n, int a, ut64 b) {
-#ifndef __wasi__
 	if (n) {
 		struct rlimit cl = {b, b};
 		setrlimit (RLIMIT_CORE, &cl);
@@ -162,32 +196,31 @@ static void set_limit(int n, int a, ut64 b) {
 		struct rlimit cl = {0, 0};
 		setrlimit (a, &cl);
 	}
-#endif
 }
 #endif
 
-static char *getstr(const char *src) {
-	int len;
+static char *getstr(const char *src, R_NULLABLE size_t *out_len) {
+	size_t len = 0;
 	char *ret = NULL;
 
-	switch (*src) {
+	switch (*(src++)) {
 	case '\'':
-		ret = strdup (src+1);
+		ret = strdup (src);
 		if (ret) {
 			len = strlen (ret);
 			if (len > 0) {
 				len--;
 				if (ret[len] == '\'') {
 					ret[len] = 0;
-					return ret;
+					goto beach;
 				}
-				eprintf ("Missing \"\n");
+				R_LOG_ERROR ("Unterminated string literal in input: ' expected");
 			}
 			free (ret);
 		}
 		return NULL;
 	case '"':
-		ret = strdup (src + 1);
+		ret = strdup (src);
 		if (ret) {
 			len = strlen (ret);
 			if (len > 0) {
@@ -195,83 +228,113 @@ static char *getstr(const char *src) {
 				if (ret[len] == '"') {
 					ret[len] = 0;
 					r_str_unescape (ret);
-					return ret;
+					goto beach;
 				}
-				eprintf ("Missing \"\n");
+				R_LOG_ERROR ("Unterminated string literal in input: \" expected");
 			}
 			free (ret);
 		}
 		return NULL;
 	case '@':
 		{
-			char *pat = strchr (src + 1, '@');
-			if (pat) {
-				size_t len;
-				long i, rep;
+			char *pat, *endptr;
+			if ((pat = strchr (src, '@'))) {
+				size_t i, pat_len;
 				*pat++ = 0;
-				rep = strtol (src + 1, NULL, 10);
-				len = strlen (pat);
-				if (rep > 0) {
-					char *buf = malloc (rep);
-					if (buf) {
-						for (i = 0; i < rep; i++) {
-							buf[i] = pat[i % len];
+				len = strtoul (src, &endptr, 10);
+				if (*endptr != 0) {
+					R_LOG_ERROR ("Invalid num in @<num>@<pattern> expr");
+					return NULL;
+				}
+				if (errno == EINVAL || errno == ERANGE || len > 64000000) {
+					R_LOG_ERROR ("Out-of-bounds num in @<num>@<pattern> expr");
+					return NULL;
+				}
+				pat_len = strlen (pat);
+				if (pat_len == 0) {
+					R_LOG_ERROR ("Missing pattern in @<num>@<pattern> expr");
+					return NULL;
+				}
+				if (len > 0) {
+					ret = malloc (len + 1);
+					if (ret) {
+						for (i = 0; i < len; i++) {
+							ret[i] = pat[i % pat_len];
 						}
+						ret[len] = 0;
 					}
-					return buf;
+					goto beach;
 				}
 			}
 			// slurp file
-			return r_file_slurp (src + 1, NULL);
+			ret = r_file_slurp (src, &len);
+			break;
 		}
 	case '`':
 		{
-		char *msg = strdup (src + 1);
-		int msg_len = strlen (msg);
-		if (msg_len > 0) {
-			msg [msg_len - 1] = 0;
-			char *ret = r_sys_cmd_str (msg, NULL, NULL);
-			r_str_trim_tail (ret);
-			free (msg);
-			return ret;
+		size_t msg_len = strlen (src);
+		if (msg_len == 0) {
+			R_LOG_ERROR ("Invalid backtick expression in input");
+			return NULL;
 		}
+		if (src [msg_len - 1] != '`') {
+			R_LOG_ERROR ("Unterminated backtick expr in input");
+			return NULL;
+		}
+		char *msg = strdup (src);
+		if (!msg) {
+			return NULL;
+		}
+		msg [msg_len - 1] = 0;
+		int cmd_len = 0;
+		ret = r_sys_cmd_str (msg, NULL, &cmd_len);
+		len = (size_t)cmd_len;
+		r_str_trim_tail (ret);
 		free (msg);
-		return strdup ("");
+		break;
 		}
 	case '!':
 		{
-		char *a = r_sys_cmd_str (src + 1, NULL, NULL);
-		r_str_trim_tail (a);
-		return a;
+		ret = r_sys_cmd_str (src, NULL, NULL);
+		if (!ret) {
+			return NULL;
+		}
+		r_str_trim_tail (ret);
+		len = strlen (ret);
+		break;
 		}
 	case ':':
-		if (src[1] == '!') {
-			ret = r_sys_cmd_str (src + 1, NULL, NULL);
-			r_str_trim_tail (ret); // why no head :?
-		} else {
-			ret = strdup (src);
+		{
+		char *hex = getstr (src, NULL);
+		if (!hex) {
+			return NULL;
 		}
-		len = r_hex_str2bin (src + 1, (ut8*)ret);
-		if (len > 0) {
+		int hexlen = r_hex_str2bin (hex, NULL);
+		if (hexlen <= 0) {
+			R_LOG_ERROR ("Invalid hexpair string");
+			free (hex);
+			return NULL;
+		}
+		len = (size_t)hexlen;
+		ret = malloc (len + 1);
+		if (ret) {
 			ret[len] = 0;
-			return ret;
+			r_hex_str2bin (hex, (ut8*)ret);
 		}
-		eprintf ("Invalid hexpair string\n");
-		free (ret);
-		return NULL;
+		free (hex);
+		}
+		break;
+	default:
+		len = r_str_unescape ((ret = strdup (src - 1)));
+		break;
+	}
+beach:
+	if (out_len) {
+		*out_len = len;
 	}
-	r_str_unescape ((ret = strdup (src)));
 	return ret;
 }
 
-static int parseBool(const char *e) {
-	return (strcmp (e, "yes")?
-		(strcmp (e, "on")?
-		(strcmp (e, "true")?
-		(strcmp (e, "1")?
-		0: 1): 1): 1): 1);
-}
-
 // TODO: move into r_util? r_run_... ? with the rest of funcs?
 static void setASLR(RRunProfile *r, int enabled) {
 #if __linux__
@@ -290,8 +353,7 @@ static void setASLR(RRunProfile *r, int enabled) {
 		: r->_program ? r->_program
 		: r->_args[0] ? r->_args[0]
 		: "/path/to/exec";
-	eprintf ("To disable aslr patch mach0.hdr.flags with:\n"
-		"r2 -qwnc 'wx 000000 @ 0x18' %s\n", argv0);
+	R_LOG_INFO ("To disable aslr patch mach0.hdr.flags with: r2 -qwnc 'wx 000000 @ 0x18' %s", argv0);
 	// f MH_PIE=0x00200000; wB-MH_PIE @ 24\n");
 	// for osxver>=10.7
 	// "unset the MH_PIE bit in an already linked executable" with --no-pie flag of the script
@@ -334,7 +396,6 @@ static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err)
 		close (saved_stdin);
 		return -1;
 	}
-	
 	int fdm, pid = dyn_forkpty (&fdm, NULL, NULL, NULL);
 	if (pid == -1) {
 		close (saved_stdin);
@@ -403,37 +464,37 @@ static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err)
 }
 #endif
 
-static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
+static bool handle_redirection(const char *cmd, bool in, bool out, bool err) {
 #if __APPLE__ && !__POWERPC__
 	//XXX handle this in other layer since things changes a little bit
 	//this seems like a really good place to refactor stuff
-	return 0;
+	return true;
 #else
-	if (!cmd || !*cmd) {
-		return 0;
+	if (R_STR_ISEMPTY (cmd)) {
+		return true;
 	}
 	if (cmd[0] == '"') {
 #ifdef __wasi__
-		eprintf ("[ERROR] rarun2: Cannot create pipe\n");
-#elif __UNIX__
+		R_LOG_ERROR ("Cannot create pipe");
+#elif R2__UNIX__
 		if (in) {
-			int pipes[2];
+			int pipes[2] = { -1, -1 };
 			if (pipe (pipes) != -1) {
 				size_t cmdl = strlen (cmd)-2;
 				if (write (pipes[1], cmd + 1, cmdl) != cmdl) {
-					eprintf ("[ERROR] rarun2: Cannot write to the pipe\n");
+					R_LOG_ERROR ("Cannot write to the pipe");
 					close (0);
-					return 1;
+					return false;
 				}
 				if (write (pipes[1], "\n", 1) != 1) {
-					eprintf ("[ERROR] rarun2: Cannot write to the pipe\n");
+					R_LOG_ERROR ("Cannot write to the pipe");
 					close (0);
-					return 1;
+					return false;
 				}
 				close (0);
 				dup2 (pipes[0], 0);
 			} else {
-				eprintf ("[ERROR] rarun2: Cannot create pipe\n");
+				R_LOG_ERROR ("Cannot create pipe");
 			}
 		}
 #else
@@ -452,36 +513,36 @@ static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
 		flag |= in ? O_RDONLY : 0;
 		flag |= out ? O_WRONLY | O_CREAT : 0;
 		flag |= err ? O_WRONLY | O_CREAT : 0;
-#ifdef __WINDOWS__
+#ifdef R2__WINDOWS__
 		mode = _S_IREAD | _S_IWRITE;
 #else
 		mode = S_IRUSR | S_IWUSR;
 #endif
 		f = open (cmd, flag, mode);
 		if (f < 0) {
-			eprintf ("[ERROR] rarun2: Cannot open: %s\n", cmd);
-			return 1;
+			R_LOG_ERROR ("Cannot open: %s", cmd);
+			return false;
 		}
 #ifndef __wasi__
 #define DUP(x) { close(x); dup2(f,x); }
 		if (in) {
-			DUP(0);
+			DUP (0);
 		}
 		if (out) {
-			DUP(1);
+			DUP (1);
 		}
 		if (err) {
-			DUP(2);
+			DUP (2);
 		}
 #endif
 		close (f);
 	}
-	return 0;
+	return true;
 #endif
 }
 
 R_API bool r_run_parsefile(RRunProfile *p, const char *b) {
-	r_return_val_if_fail (p && b, false);
+	R_RETURN_VAL_IF_FAIL (p && b, false);
 	char *s = r_file_slurp (b, NULL);
 	if (s) {
 		bool ret = r_run_parse (p, s);
@@ -506,7 +567,8 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 		return 0;
 	}
 	if (!strcmp (b, "program")) {
-		p->_args[0] = p->_program = strdup (e);
+		p->_args[0] = strdup (e);
+		p->_program = strdup (e);
 	} else if (!strcmp (b, "daemon")) {
 		p->_daemon = true;
 	} else if (!strcmp (b, "system")) {
@@ -516,9 +578,12 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 	} else if (!strcmp (b, "runlib.fcn")) {
 		p->_runlib_fcn = strdup (e);
 	} else if (!strcmp (b, "aslr")) {
-		p->_aslr = parseBool (e);
+		p->_aslr = r_str_is_true (e);
 	} else if (!strcmp (b, "pid")) {
 		p->_pid = atoi (e);
+		if (!p->_pid) {
+			p->_pid = r_str_is_true (e);
+		}
 	} else if (!strcmp (b, "pidfile")) {
 		p->_pidfile = strdup (e);
 	} else if (!strcmp (b, "connect")) {
@@ -526,7 +591,7 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 	} else if (!strcmp (b, "listen")) {
 		p->_listen = strdup (e);
 	} else if (!strcmp (b, "pty")) {
-		p->_pty = parseBool (e);
+		p->_pty = r_str_is_true (e);
 	} else if (!strcmp (b, "stdio")) {
 		if (e[0] == '!') {
 			p->_stdio = strdup (e);
@@ -546,9 +611,9 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 	} else if (!strcmp (b, "chdir")) {
 		p->_chgdir = strdup (e);
 	} else if (!strcmp (b, "core")) {
-		p->_docore = parseBool (e);
+		p->_docore = r_str_is_true (e);
 	} else if (!strcmp (b, "fork")) {
-		p->_dofork = parseBool (e);
+		p->_dofork = r_str_is_true (e);
 	} else if (!strcmp (b, "sleep")) {
 		p->_r2sleep = atoi (e);
 	} else if (!strcmp (b, "maxstack")) {
@@ -559,14 +624,19 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 		p->_maxfd = atoi (e);
 	} else if (!strcmp (b, "bits")) {
 		p->_bits = atoi (e);
+	} else if (!strcmp (b, "time")) {
+		p->_time = true;
 	} else if (!strcmp (b, "chroot")) {
 		p->_chroot = strdup (e);
 	} else if (!strcmp (b, "libpath")) {
 		p->_libpath = strdup (e);
 	} else if (!strcmp (b, "preload")) {
-		p->_preload = strdup (e);
+		if (!p->_preload) {
+			p->_preload = r_list_newf (free);
+		}
+		r_list_append (p->_preload, strdup (e));
 	} else if (!strcmp (b, "r2preload")) {
-		p->_r2preload = parseBool (e);
+		p->_r2preload = r_str_is_true (e);
 	} else if (!strcmp (b, "r2preweb")) {
 		r_sys_setenv ("RARUN2_WEB", "yes");
 	} else if (!strcmp (b, "setuid")) {
@@ -583,20 +653,21 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 		p->_timeout = atoi (e);
 	} else if (!strcmp (b, "timeoutsig")) {
 		p->_timeout_sig = r_signal_from_string (e);
-	} else if (!memcmp (b, "arg", 3)) {
+	} else if (r_str_startswith (b, "arg")) {
 		int n = atoi (b + 3);
 		if (n >= 0 && n < R_RUN_PROFILE_NARGS) {
-			p->_args[n] = getstr (e);
+			free (p->_args[n]);
+			p->_args[n] = getstr (e, NULL);
 			p->_argc++;
 		} else {
-			eprintf ("Out of bounds args index: %d\n", n);
+			R_LOG_ERROR ("Out of bounds args index: %d", n);
 		}
 	} else if (!strcmp (b, "envfile")) {
 		char *p, buf[1024];
 		size_t len;
 		FILE *fd = r_sandbox_fopen (e, "r");
 		if (!fd) {
-			eprintf ("Cannot open '%s'\n", e);
+			R_LOG_ERROR ("Cannot open '%s'", e);
 			if (must_free == true) {
 				free (e);
 			}
@@ -629,11 +700,11 @@ R_API bool r_run_parseline(RRunProfile *p, const char *b) {
 		char *V, *v = strchr (e, '=');
 		if (v) {
 			*v++ = 0;
-			V = getstr (v);
+			V = getstr (v, NULL);
 			r_sys_setenv (e, V);
 			free (V);
 		}
-	} else if (!strcmp(b, "clearenv")) {
+	} else if (!strcmp (b, "clearenv")) {
 		r_sys_clearenv ();
 	}
 	if (must_free == true) {
@@ -683,7 +754,7 @@ R_API const char *r_run_help(void) {
 	"# chroot=/mnt/chroot\n"
 	"# libpath=$PWD:/tmp/lib\n"
 	"# r2preload=yes\n"
-	"# preload=/lib/libfoo.so\n"
+	"# preload=/lib/libfoo.so # you can load more than one lib by using this directive many times\n"
 	"# setuid=2000\n"
 	"# seteuid=2000\n"
 	"# setgid=2001\n"
@@ -696,7 +767,7 @@ static int fd_forward(int in_fd, int out_fd, char **buff) {
 	int size = 0;
 
 	if (ioctl (in_fd, FIONREAD, &size) == -1) {
-		perror ("ioctl");
+		r_sys_perror ("ioctl");
 		return -1;
 	}
 	if (!size) { // child process exited or socket is closed
@@ -705,16 +776,16 @@ static int fd_forward(int in_fd, int out_fd, char **buff) {
 
 	char *new_buff = realloc (*buff, size);
 	if (!new_buff) {
-		eprintf ("Failed to allocate buffer for redirection");
+		R_LOG_ERROR ("Failed to allocate buffer for redirection");
 		return -1;
 	}
 	*buff = new_buff;
 	if (read (in_fd, *buff, size) != size) {
-		perror ("read");
+		r_sys_perror ("read");
 		return -1;
 	}
 	if (write (out_fd, *buff, size) != size) {
-		perror ("write");
+		r_sys_perror ("write");
 		return -1;
 	}
 
@@ -734,35 +805,34 @@ static int redirect_socket_to_stdio(RSocket *sock) {
 	return 0;
 }
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 static RThreadFunctionRet exit_process(RThread *th) {
-	// eprintf ("\nrarun2: Interrupted by timeout\n");
 	exit (0);
 }
 #endif
 
-static int redirect_socket_to_pty(RSocket *sock) {
+static bool redirect_socket_to_pty(RSocket *sock) {
 #if HAVE_PTY
 	// directly duplicating the fds using dup2() creates problems
 	// in case of interactive applications
 	int fdm = -1, fds = -1;
 
 	if (dyn_openpty && dyn_openpty (&fdm, &fds, NULL, NULL, NULL) == -1) {
-		perror ("opening pty");
-		return -1;
+		r_sys_perror ("opening pty");
+		return false;
 	}
 
 	pid_t child_pid = r_sys_fork ();
 
 	if (child_pid == -1) {
-		eprintf ("cannot fork\n");
+		R_LOG_ERROR ("cannot fork");
 		if (fdm != -1) {
 			close (fdm);
 		}
 		if (fds != -1) {
 			close (fds);
 		}
-		return -1;
+		return false;
 	}
 
 	if (child_pid == 0) {
@@ -780,7 +850,7 @@ static int redirect_socket_to_pty(RSocket *sock) {
 			FD_SET (sockfd, &readfds);
 
 			if (select (max_fd + 1, &readfds, NULL, NULL, NULL) == -1) {
-				perror ("select error");
+				r_sys_perror ("select error");
 				break;
 			}
 
@@ -821,41 +891,44 @@ static int redirect_socket_to_pty(RSocket *sock) {
 	cfmakeraw (&t);
 	tcsetattr (0, TCSANOW, &t);
 
-	return 0;
+	return true;
 #else
 	// Fallback to socket to I/O redirection
 	return redirect_socket_to_stdio (sock);
 #endif
 }
 
-R_API int r_run_config_env(RRunProfile *p) {
+R_API bool r_run_config_env(RRunProfile *p) {
 	int ret;
 
 #if HAVE_PTY
 	dyn_init ();
 #endif
-
 	if (!p->_program && !p->_system && !p->_runlib) {
-		eprintf ("No program, system or runlib rule defined\n");
-		return 1;
+		R_LOG_ERROR ("No program, system or runlib rule defined");
+		return false;
 	}
 	// when IO is redirected to a process, handle them together
-	if (handle_redirection (p->_stdio, true, true, false) != 0) {
-		return 1;
+	if (!handle_redirection (p->_stdio, true, true, false)) {
+		R_LOG_WARN ("cannot handle stdio redirection");
+		return false;
 	}
-	if (handle_redirection (p->_stdin, true, false, false) != 0) {
-		return 1;
+	if (!handle_redirection (p->_stdin, true, false, false)) {
+		R_LOG_WARN ("cannot handle stdin redirection");
+		return false;
 	}
-	if (handle_redirection (p->_stdout, false, true, false) != 0) {
-		return 1;
+	if (!handle_redirection (p->_stdout, false, true, false)) {
+		R_LOG_WARN ("cannot handle stdout redirection");
+		return false;
 	}
-	if (handle_redirection (p->_stderr, false, false, true) != 0) {
-		return 1;
+	if (!handle_redirection (p->_stderr, false, false, true)) {
+		R_LOG_WARN ("cannot handle stderr redirection");
+		return false;
 	}
 	if (p->_aslr != -1) {
 		setASLR (p, p->_aslr);
 	}
-#if __UNIX__ && !__wasi__ && !defined(serenity)
+#if R2__UNIX__ && !__wasi__ && !defined(__serenity__)
 	set_limit (p->_docore, RLIMIT_CORE, RLIM_INFINITY);
 	if (p->_maxfd) {
 		set_limit (p->_maxfd, RLIMIT_NOFILE, p->_maxfd);
@@ -870,7 +943,7 @@ R_API int r_run_config_env(RRunProfile *p) {
 	}
 #else
 	if (p->_docore || p->_maxfd || p->_maxproc || p->_maxstack)
-		eprintf ("Warning: setrlimits not supported for this platform\n");
+		R_LOG_WARN ("setrlimits not supported for this platform");
 #endif
 	if (p->_connect) {
 		char *q = strchr (p->_connect, ':');
@@ -878,59 +951,63 @@ R_API int r_run_config_env(RRunProfile *p) {
 			RSocket *fd = r_socket_new (0);
 			*q = 0;
 			if (!r_socket_connect_tcp (fd, p->_connect, q+1, 30)) {
-				eprintf ("Cannot connect\n");
+				R_LOG_ERROR ("Cannot connect");
 				r_socket_free (fd);
-				return 1;
+				return false;
 			}
 			if (p->_pty) {
-				if (redirect_socket_to_pty (fd) != 0) {
-					eprintf ("socket redirection failed\n");
+				if (!redirect_socket_to_pty (fd)) {
+					R_LOG_ERROR ("socket redirection failed");
 					r_socket_free (fd);
-					return 1;
+					return false;
 				}
 			} else {
 				redirect_socket_to_stdio (fd);
 			}
 		} else {
-			eprintf ("Invalid format for connect. missing ':'\n");
-			return 1;
+			R_LOG_ERROR ("Invalid format for connect. missing ':'");
+			return false;
 		}
 	}
 	if (p->_listen) {
 		RSocket *child, *fd = r_socket_new (0);
 		bool is_child = false;
 		if (!r_socket_listen (fd, p->_listen, NULL)) {
-			eprintf ("rarun2: cannot listen\n");
+			R_LOG_ERROR ("Cannot listen");
 			r_socket_free (fd);
-			return 1;
+			return false;
 		}
 		while (true) {
 			child = r_socket_accept (fd);
 			if (child) {
 				is_child = true;
 
-				if (p->_dofork && !p->_dodebug) {
+				if (p->_dofork) {
 					pid_t child_pid = r_sys_fork ();
 					if (child_pid == -1) {
-						eprintf("rarun2: cannot fork\n");
+						R_LOG_ERROR ("Cannot fork");
 						r_socket_free (child);
 						r_socket_free (fd);
-						return 1;
-					} else if (child_pid != 0){
+						return false;
+					}
+					if (child_pid != 0) {
 						// parent code
 						is_child = false;
+						if (p->_pid) {
+							R_LOG_INFO ("pid = %d", child_pid);
+						}
 					}
 				}
 
 				if (is_child) {
 					r_socket_close_fd (fd);
-					eprintf ("connected\n");
+					R_LOG_INFO ("connected");
 					if (p->_pty) {
-						if (redirect_socket_to_pty (child) != 0) {
-							eprintf ("socket redirection failed\n");
+						if (!redirect_socket_to_pty (child)) {
+							R_LOG_ERROR ("socket redirection failed");
 							r_socket_free (child);
 							r_socket_free (fd);
-							return 1;
+							return false;
 						}
 					} else {
 						redirect_socket_to_stdio (child);
@@ -949,64 +1026,60 @@ R_API int r_run_config_env(RRunProfile *p) {
 	if (p->_r2sleep != 0) {
 		r_sys_sleep (p->_r2sleep);
 	}
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 	if (p->_chroot) {
 		if (chdir (p->_chroot) == -1) {
-			eprintf ("Cannot chdir to chroot in %s\n", p->_chroot);
-			return 1;
-		} else {
-			if (chroot (".") == -1) {
-				eprintf ("Cannot chroot to %s\n", p->_chroot);
-				return 1;
-			} else {
-				// Silenting pedantic meson flags...
-				if (chdir ("/") == -1) {
-					eprintf ("Cannot chdir to /\n");
-					return 1;
-				}
-				if (p->_chgdir) {
-					if (chdir (p->_chgdir) == -1) {
-						eprintf ("Cannot chdir after chroot to %s\n", p->_chgdir);
-						return 1;
-					}
-				}
-			}
+			R_LOG_ERROR ("Cannot chdir to chroot in %s", p->_chroot);
+			return false;
+		}
+		if (chroot (".") == -1) {
+			R_LOG_ERROR ("Cannot chroot to %s", p->_chroot);
+			return false;
+		}
+		// Silenting pedantic meson flags...
+		if (chdir ("/") == -1) {
+			R_LOG_ERROR ("Cannot chdir to /");
+			return false;
+		}
+		if (p->_chgdir && chdir (p->_chgdir) == -1) {
+			R_LOG_ERROR ("Cannot chdir after chroot to %s", p->_chgdir);
+			return false;
 		}
 	} else if (p->_chgdir) {
 		if (chdir (p->_chgdir) == -1) {
-			eprintf ("Cannot chdir after chroot to %s\n", p->_chgdir);
-			return 1;
+			R_LOG_ERROR ("Cannot chdir after chroot to %s", p->_chgdir);
+			return false;
 		}
 	}
 #else
 	if (p->_chgdir) {
 		ret = chdir (p->_chgdir);
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 	}
 	if (p->_chroot) {
 		ret = chdir (p->_chroot);
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 	}
 #endif
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 	if (p->_setuid) {
 		ret = setgroups (0, NULL);
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 		ret = setuid (atoi (p->_setuid));
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 	}
 	if (p->_seteuid) {
 		ret = seteuid (atoi (p->_seteuid));
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 	}
 	if (p->_setgid) {
@@ -1016,47 +1089,47 @@ R_API int r_run_config_env(RRunProfile *p) {
 		ret = setgid (atoi (p->_setgid));
 #endif
 		if (ret < 0) {
-			return 1;
+			return false;
 		}
 	}
 	if (p->_input) {
 		char *inp;
-		int f2[2];
+		int f2[2] = { -1, -1 };
 		if (pipe (f2) != -1) {
 			close (0);
 #if !__wasi__
 			dup2 (f2[0], 0);
 #endif
 		} else {
-			eprintf ("[ERROR] rarun2: Cannot create pipe\n");
-			return 1;
+			R_LOG_ERROR ("Cannot create pipe");
+			return false;
 		}
-		inp = getstr (p->_input);
+		size_t inpl;
+		inp = getstr (p->_input, &inpl);
 		if (inp) {
-			size_t inpl = strlen (inp);
 			if  (write (f2[1], inp, inpl) != inpl) {
-				eprintf ("[ERROR] rarun2: Cannot write to the pipe\n");
+				R_LOG_ERROR ("Cannot write to the pipe");
 			}
-			close (f2[1]);
 			free (inp);
 		} else {
-			eprintf ("Invalid input\n");
+			R_LOG_ERROR ("Invalid input");
 		}
+		close (f2[1]);
 	}
 #endif
 	if (p->_r2preload) {
-		if (p->_preload) {
-			eprintf ("Warning: Only one library can be opened at a time\n");
-		}
-#ifdef __WINDOWS__
-		p->_preload = r_str_r2_prefix (R_JOIN_2_PATHS (R2_LIBDIR, "libr2."R_LIB_EXT));
+#if R2__WINDOWS__
+		R_LOG_ERROR ("r2preload is not supported in this platform");
 #else
-		p->_preload = strdup (R2_LIBDIR"/libr2."R_LIB_EXT);
+		if (!p->_preload) {
+			p->_preload = r_list_newf (free);
+		}
+		r_list_append (p->_preload, strdup (R2_LIBDIR"/libr2."R_LIB_EXT));
 #endif
 	}
 	if (p->_libpath) {
-#if __WINDOWS__
-		eprintf ("rarun2: libpath unsupported for this platform\n");
+#if R2__WINDOWS__
+		R_LOG_ERROR ("libpath is not supported in this platform");
 #elif __HAIKU__
 		char *orig = r_sys_getenv ("LIBRARY_PATH");
 		char *newlib = r_str_newf ("%s:%s", p->_libpath, orig);
@@ -1070,22 +1143,26 @@ R_API int r_run_config_env(RRunProfile *p) {
 #endif
 	}
 	if (p->_preload) {
-#if __APPLE__
+		char *ps = r_str_list_join (p->_preload, ":");
+#if R2__WINDOWS__
+		R_LOG_WARN ("The preload directive doesn't work on windows");
+#elif __APPLE__
 		// 10.6
 #ifndef __MAC_10_7
-		r_sys_setenv ("DYLD_PRELOAD", p->_preload);
+		r_sys_setenv ("DYLD_PRELOAD", ps);
 #endif
-		r_sys_setenv ("DYLD_INSERT_LIBRARIES", p->_preload);
+		r_sys_setenv ("DYLD_INSERT_LIBRARIES", ps);
 		// 10.8
 		r_sys_setenv ("DYLD_FORCE_FLAT_NAMESPACE", "1");
 #else
-		r_sys_setenv ("LD_PRELOAD", p->_preload);
+		r_sys_setenv ("LD_PRELOAD", ps);
 #endif
+		free (ps);
 	}
 	if (p->_timeout) {
 #if __wasi__
 		// do nothing
-#elif __UNIX__
+#elif R2__UNIX__
 		int mypid = r_sys_getpid ();
 		if (!r_sys_fork ()) {
 			int use_signal = p->_timeout_sig;
@@ -1103,27 +1180,39 @@ R_API int r_run_config_env(RRunProfile *p) {
 		if (p->_timeout_sig < 1 || p->_timeout_sig == 9) {
 			r_th_new (exit_process, NULL, p->_timeout);
 		} else {
-			eprintf ("timeout with signal not supported for this platform\n");
+			R_LOG_ERROR ("timeout with signal not supported for this platform");
 		}
 #endif
 	}
-	return 0;
+	return true;
+}
+
+static void time_end(bool chk, ut64 time_begin) {
+	if (chk) {
+		ut64 now = r_time_now ();
+		R_LOG_INFO ("%"PFMT64d, now - time_begin);
+	}
 }
 
 // NOTE: return value is like in unix return code (0 = ok, 1 = not ok)
-R_API int r_run_start(RRunProfile *p) {
+R_API bool r_run_start(RRunProfile *p) {
+	R_RETURN_VAL_IF_FAIL (p, false);
 #if LIBC_HAVE_FORK
 	if (p->_execve) {
 		exit (execv (p->_program, (char* const*)p->_args));
 	}
 #endif
+	ut64 time_begin = 0;
+	if (p->_time) {
+		time_begin = r_time_now ();
+	}
 #if __APPLE__ && !__POWERPC__ && LIBC_HAVE_FORK
 	posix_spawnattr_t attr = {0};
 	pid_t pid = -1;
 	int ret;
 	posix_spawnattr_init (&attr);
 	if (p->_args[0]) {
-		char **envp = r_sys_get_environ();
+		char **envp = r_sys_get_environ ();
 		ut32 spflags = 0; //POSIX_SPAWN_START_SUSPENDED;
 		spflags |= POSIX_SPAWN_SETEXEC;
 		if (p->_aslr == 0) {
@@ -1145,54 +1234,64 @@ R_API int r_run_start(RRunProfile *p) {
 			posix_spawnattr_setbinpref_np (
 					&attr, 1, &cpu, &copied);
 		}
-		ret = posix_spawnp (&pid, p->_args[0],
-			NULL, &attr, p->_args, envp);
+		if (p->_pid) {
+			R_LOG_INFO ("pid = %d", r_sys_getpid ());
+		}
+		ret = posix_spawnp (&pid, p->_args[0], NULL, &attr, p->_args, envp);
+		if (p->_pid) {
+			R_LOG_INFO ("pid = %d", pid);
+		}
 		switch (ret) {
 		case 0:
 			break;
 		case 22:
-			eprintf ("posix_spawnp: Invalid argument\n");
+			R_LOG_ERROR ("posix_spawnp: Invalid argument");
 			break;
 		case 86:
-			eprintf ("posix_spawnp: Unsupported architecture\n");
+			R_LOG_ERROR ("posix_spawnp: Unsupported architecture");
 			break;
 		default:
-			eprintf ("posix_spawnp: unknown error %d\n", ret);
-			perror ("posix_spawnp");
+			R_LOG_ERROR ("posix_spawnp: unknown error %d", ret);
+			r_sys_perror ("posix_spawnp");
 			break;
 		}
 		exit (ret);
 	}
 #endif
 	if (p->_system) {
+		int rc = 0;
 		if (p->_pid) {
-			eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
+			R_LOG_ERROR ("PID: Cannot determine pid with 'system' directive. Use 'program'");
 		}
 		if (p->_daemon) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	//		eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
 #else
 			pid_t child = r_sys_fork ();
 			if (child == -1) {
-				perror ("fork");
+				r_sys_perror ("fork");
 				exit (1);
 			}
 			if (child) {
+				if (p->_pid) {
+					R_LOG_INFO ("pid = %d", child);
+				}
 				if (p->_pidfile) {
-					char pidstr[32];
-					snprintf (pidstr, sizeof (pidstr), "%d\n", child);
-					r_file_dump (p->_pidfile,
-							(const ut8*)pidstr,
-							strlen (pidstr), 0);
+					r_strf_var (pidstr, 32, "%d\n", (int)child);
+					r_file_dump (p->_pidfile, (const ut8*)pidstr,
+						strlen (pidstr), 0);
 				}
 				exit (0);
 			}
 #if !__wasi__
 			setsid ();
 #endif
+			// setvbuf (stdout, NULL, _IONBF, 0);
 			if (p->_timeout) {
-#if __UNIX__
+#if R2__UNIX__
+#if !__wasi__
 				int mypid = r_sys_getpid ();
+#endif
 				if (!r_sys_fork ()) {
 					int use_signal = p->_timeout_sig;
 					if (use_signal < 1) {
@@ -1205,46 +1304,50 @@ R_API int r_run_start(RRunProfile *p) {
 					}
 					kill (mypid, use_signal);
 #endif
+					time_end (p->_time, time_begin);
 					exit (0);
 				}
 #else
-				eprintf ("timeout not supported for this platform\n");
+				R_LOG_ERROR ("timeout not supported for this platform");
 #endif
 			}
 #endif
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 			close (0);
 			close (1);
 			char *bin_sh = r_file_binsh ();
 			if (bin_sh) {
-				exit (execl (bin_sh, bin_sh, "-c", p->_system, NULL));
+				rc = execl (bin_sh, bin_sh, "-c", p->_system, NULL);
 			} else {
-				exit (r_sys_cmd (p->_system));
+				rc = r_sys_cmd (p->_system);
 			}
 			free (bin_sh);
 #else
-			exit (r_sys_cmd (p->_system));
+			rc = r_sys_cmd (p->_system);
 #endif
 		} else {
 			if (p->_pidfile) {
-				eprintf ("Warning: pidfile doesnt work with 'system'.\n");
+				R_LOG_WARN ("pidfile doesnt work with 'system'");
 			}
-			exit (r_sys_cmd (p->_system));
+			if (p->_pid) {
+				R_LOG_WARN ("Use 'program' instead of 'system' to show the pid");
+			}
+			rc = r_sys_cmd (p->_system);
 		}
+		time_end (p->_time, time_begin);
+		exit (rc);
 	}
 	if (p->_program) {
 		if (!r_file_exists (p->_program)) {
 			char *progpath = r_file_path (p->_program);
-			if (progpath && *progpath) {
-				free (p->_program);
-				p->_program = progpath;
-			} else {
-				free (progpath);
-				eprintf ("rarun2: %s: file not found\n", p->_program);
-				return 1;
+			if (!progpath) {
+				R_LOG_ERROR ("file not found: %s", p->_program);
+				return false;
 			}
+			free (p->_program);
+			p->_program = progpath;
 		}
-#if __UNIX__
+#if R2__UNIX__
 		// XXX HACK close all non-tty fds
 		{ int i;
 			for (i = 3; i < 1024; i++) {
@@ -1259,11 +1362,11 @@ R_API int r_run_start(RRunProfile *p) {
 			int ret = setgid (atoi (p->_setgid));
 #endif
 			if (ret < 0) {
-				return 1;
+				return false;
 			}
 		}
 		if (p->_pid) {
-			eprintf ("PID: %d\n", r_sys_getpid ());
+			R_LOG_INFO ("pid = %d", r_sys_getpid ());
 		}
 		if (p->_pidfile) {
 			char pidstr[32];
@@ -1275,27 +1378,27 @@ R_API int r_run_start(RRunProfile *p) {
 #endif
 
 		if (p->_nice) {
-#if __UNIX__ && !defined(__HAIKU__) && !defined(__serenity__) && !__wasi__
+#if R2__UNIX__ && !defined(__HAIKU__) && !defined(__serenity__) && !__wasi__
 			if (nice (p->_nice) == -1) {
-				return 1;
+				return false;
 			}
 #else
-			eprintf ("nice not supported for this platform\n");
+			R_LOG_ERROR ("nice not supported for this platform");
 #endif
 		}
 		if (p->_daemon) {
-#if __WINDOWS__
-			eprintf ("PID: Cannot determine pid with 'system' directive. Use 'program'.\n");
+#if R2__WINDOWS__
+			R_LOG_ERROR ("PID: Cannot determine pid with 'system' directive. Use 'program'");
 #else
 			pid_t child = r_sys_fork ();
 			if (child == -1) {
-				perror ("fork");
+				r_sys_perror ("fork");
 				exit (1);
 			}
 			if (child) {
 				if (p->_pidfile) {
 					char pidstr[32];
-					snprintf (pidstr, sizeof (pidstr), "%d\n", child);
+					snprintf (pidstr, sizeof (pidstr), "%d\n", (int)child);
 					r_file_dump (p->_pidfile,
 							(const ut8*)pidstr,
 							strlen (pidstr), 0);
@@ -1312,23 +1415,24 @@ R_API int r_run_start(RRunProfile *p) {
 		}
 // TODO: must be HAVE_EXECVE
 #if LIBC_HAVE_FORK
+		time_end (p->_time, time_begin);
 		exit (execv (p->_program, (char* const*)p->_args));
 #endif
 	}
 	if (p->_runlib) {
 		if (!p->_runlib_fcn) {
-			eprintf ("No function specified. Please set runlib.fcn\n");
-			return 1;
+			R_LOG_ERROR ("No function specified. Please set runlib.fcn");
+			return false;
 		}
 		void *addr = r_lib_dl_open (p->_runlib);
 		if (!addr) {
-			eprintf ("Could not load the library '%s'\n", p->_runlib);
-			return 1;
+			R_LOG_ERROR ("Could not load the library '%s'", p->_runlib);
+			return false;
 		}
 		void (*fcn)(void) = r_lib_dl_sym (addr, p->_runlib_fcn);
 		if (!fcn) {
-			eprintf ("Could not find the function '%s'\n", p->_runlib_fcn);
-			return 1;
+			R_LOG_ERROR ("Could not find the function '%s'", p->_runlib_fcn);
+			return false;
 		}
 		switch (p->_argc) {
 		case 0:
@@ -1371,12 +1475,13 @@ R_API int r_run_start(RRunProfile *p) {
 				p->_args[5], p->_args[6], p->_args[7], p->_args[8], p->_args[9], p->_args[10]);
 			break;
 		default:
-			eprintf ("Too many arguments.\n");
-			return 1;
+			R_LOG_ERROR ("Too many arguments");
+			return false;
 		}
 		r_lib_dl_close (addr);
 	}
-	return 0;
+	time_end (p->_time, time_begin);
+	return true;
 }
 
 R_API char *r_run_get_environ_profile(char **env) {
