@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2021 - ret2libc, pancake */
+/* radare - LGPL - Copyright 2009-2024 - ret2libc, pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -11,6 +11,9 @@ typedef enum {
 	R_BUFFER_MMAP,
 	R_BUFFER_SPARSE,
 	R_BUFFER_REF,
+#if R2_USE_NEW_ABI
+	R_BUFFER_CACHE,
+#endif
 } RBufferType;
 
 #include "buf_file.c"
@@ -20,9 +23,14 @@ typedef enum {
 #include "buf_io.c"
 #include "buf_ref.c"
 
+#if R2_USE_NEW_ABI
+#include "buf_cache.c"
+#endif
+
 static bool buf_init(RBuffer *b, const void *user) {
-	r_return_val_if_fail (b && b->methods, false);
-	return b->methods->init? b->methods->init (b, user): true;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, false);
+	const RBufferInit init = b->methods->init;
+	return init? init (b, user): true;
 }
 
 static void buf_wholefree(RBuffer *b) {
@@ -32,40 +40,51 @@ static void buf_wholefree(RBuffer *b) {
 }
 
 static bool buf_fini(RBuffer *b) {
-	r_return_val_if_fail (b && b->methods, false);
-	return b->methods->fini? b->methods->fini (b): true;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, false);
+	const RBufferFini fini = b->methods->fini;
+	return fini? fini (b): true;
 }
 
 static ut64 buf_get_size(RBuffer *b) {
-	r_return_val_if_fail (b && b->methods, UT64_MAX);
-	return b->methods->get_size? b->methods->get_size (b): UT64_MAX;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, UT64_MAX);
+	const RBufferGetSize get_size = b->methods->get_size;
+	return get_size? get_size (b): UT64_MAX;
 }
 
 static st64 buf_read(RBuffer *b, ut8 *buf, size_t len) {
-	r_return_val_if_fail (b && b->methods, -1);
-	return b->methods->read? b->methods->read (b, buf, len): -1;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, -1);
+	const RBufferRead bufread = b->methods->read;
+	R_RETURN_VAL_IF_FAIL (bufread, -1);
+	return bufread (b, buf, len);
 }
 
 static st64 buf_write(RBuffer *b, const ut8 *buf, size_t len) {
-	r_return_val_if_fail (b && b->methods, -1);
+	R_RETURN_VAL_IF_FAIL (b && b->methods, -1);
 	buf_wholefree (b);
-	return b->methods->write? b->methods->write (b, buf, len): -1;
+	const RBufferWrite bufwrite = b->methods->write;
+	R_RETURN_VAL_IF_FAIL (bufwrite, -1);
+	return bufwrite (b, buf, len);
 }
 
 static st64 buf_seek(RBuffer *b, st64 addr, int whence) {
-	r_return_val_if_fail (b && b->methods, -1);
-	return b->methods->seek? b->methods->seek (b, addr, whence): -1;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, -1);
+	const RBufferSeek bufseek = b->methods->seek;
+	R_RETURN_VAL_IF_FAIL (bufseek, -1);
+	return bufseek (b, addr, whence);
 }
 
 static bool buf_resize(RBuffer *b, ut64 newsize) {
-	r_return_val_if_fail (b && b->methods, -1);
-	return b->methods->resize? b->methods->resize (b, newsize): false;
+	R_RETURN_VAL_IF_FAIL (b && b->methods, -1);
+	const RBufferResize bufresize = b->methods->resize;
+	R_RETURN_VAL_IF_FAIL (bufresize, false);
+	return bufresize (b, newsize);
 }
 
 static ut8 *get_whole_buf(RBuffer *b, ut64 *sz) {
-	r_return_val_if_fail (b && b->methods, NULL);
-	if (b->methods->get_whole_buf) {
-		return b->methods->get_whole_buf (b, sz);
+	R_RETURN_VAL_IF_FAIL (b && b->methods, NULL);
+	RBufferGetWholeBuf bufwhole = b->methods->get_whole_buf;
+	if (bufwhole) {
+		return bufwhole (b, sz);
 	}
 	ut64 bsz = r_buf_size (b);
 	// bsz = 4096; // FAKE MINIMUM SIZE TO READ THE BIN HEADER
@@ -96,6 +115,11 @@ static RBuffer *new_buffer(RBufferType type, const void *user) {
 	case R_BUFFER_MMAP:
 		b->methods = &buffer_mmap_methods;
 		break;
+#if R2_USE_NEW_ABI
+	case R_BUFFER_CACHE:
+		b->methods = &buffer_cache_methods;
+		break;
+#endif
 	case R_BUFFER_SPARSE:
 		b->methods = &buffer_sparse_methods;
 		break;
@@ -125,15 +149,15 @@ static RBuffer *new_buffer(RBufferType type, const void *user) {
 // copied from libr/io/cache.c:r_io_cache_read
 // ret # of bytes copied
 R_API RBuffer *r_buf_new_with_io(void *iob, int fd) {
-	r_return_val_if_fail (iob && fd >= 0, NULL);
-	struct buf_io_user u = { 0 };
+	R_RETURN_VAL_IF_FAIL (iob && fd >= 0, NULL);
+	struct buf_io_user u = {0};
 	u.iob = (RIOBind *)iob;
 	u.fd = fd;
 	return new_buffer (R_BUFFER_IO, &u);
 }
 
 R_API RBuffer *r_buf_new_with_pointers(const ut8 *bytes, ut64 len, bool steal) {
-	struct buf_bytes_user u = { 0 };
+	struct buf_bytes_user u = {0};
 	u.data_steal = bytes;
 	u.length = len;
 	u.steal = steal;
@@ -146,7 +170,7 @@ R_API RBuffer *r_buf_new_empty(ut64 len) {
 		return NULL;
 	}
 
-	struct buf_bytes_user u = { 0 };
+	struct buf_bytes_user u = {0};
 	u.data_steal = buf;
 	u.length = len;
 	u.steal = true;
@@ -158,14 +182,14 @@ R_API RBuffer *r_buf_new_empty(ut64 len) {
 }
 
 R_API RBuffer *r_buf_new_with_bytes(const ut8 *bytes, ut64 len) {
-	struct buf_bytes_user u = { 0 };
+	struct buf_bytes_user u = {0};
 	u.data = bytes;
 	u.length = len;
 	return new_buffer (R_BUFFER_BYTES, &u);
 }
 
 R_API RBuffer *r_buf_new_slice(RBuffer *b, ut64 offset, ut64 size) {
-	struct buf_ref_user u = { 0 };
+	struct buf_ref_user u = {0};
 	u.parent = b;
 	u.offset = offset;
 	u.size = size;
@@ -190,35 +214,53 @@ R_API RBuffer *r_buf_new_sparse(ut8 Oxff) {
 	return b;
 }
 
+#if R2_USE_NEW_ABI
+R_API RBuffer *r_buf_new_with_cache(RBuffer *sb, bool steal) {
+	RBuffer *b = new_buffer (R_BUFFER_CACHE, NULL);
+	if (b) {
+		struct minicachebuf {
+			RBuffer *sb;
+			bool owned;
+			ut64 length;
+		};
+		struct minicachebuf *mcb = b->priv;
+		mcb->sb = sb;
+		mcb->owned = steal;
+		mcb->length = r_buf_size (sb);
+	}
+	return b;
+}
+#endif
+
 R_API RBuffer *r_buf_new(void) {
-	struct buf_bytes_user u = { 0 };
+	struct buf_bytes_user u = {0};
 	u.data = NULL;
 	u.length = 0;
 	return new_buffer (R_BUFFER_BYTES, &u);
 }
 
 R_DEPRECATE R_API const ut8 *r_buf_data(RBuffer *b, ut64 *size) {
-	r_return_val_if_fail (b, NULL);
+	R_RETURN_VAL_IF_FAIL (b, NULL);
 	b->whole_buf = get_whole_buf (b, size);
 	return b->whole_buf;
 }
 
 R_API ut64 r_buf_size(RBuffer *b) {
-	r_return_val_if_fail (b, 0);
+	R_RETURN_VAL_IF_FAIL (b, 0);
 	return buf_get_size (b);
 }
 
 // rename to new?
 R_API RBuffer *r_buf_new_mmap(const char *filename, int perm) {
-	r_return_val_if_fail (filename, NULL);
-	struct buf_mmap_user u = { 0 };
+	R_RETURN_VAL_IF_FAIL (filename, NULL);
+	struct buf_mmap_user u = {0};
 	u.filename = filename;
 	u.perm = perm;
 	return new_buffer (R_BUFFER_MMAP, &u);
 }
 
 R_API RBuffer *r_buf_new_file(const char *file, int perm, int mode) {
-	struct buf_file_user u = { 0 };
+	struct buf_file_user u = {0};
 	u.file = file;
 	u.perm = perm;
 	u.mode = mode;
@@ -233,7 +275,7 @@ R_API RBuffer *r_buf_new_slurp(const char *file) {
 		return NULL;
 	}
 
-	struct buf_bytes_user u = { 0 };
+	struct buf_bytes_user u = {0};
 	u.data_steal = (ut8 *)tmp;
 	u.length = (ut64)len;
 	u.steal = true;
@@ -251,7 +293,7 @@ R_API bool r_buf_dump(RBuffer *b, const char *file) {
 }
 
 R_API st64 r_buf_seek(RBuffer *b, st64 addr, int whence) {
-	r_return_val_if_fail (b, -1);
+	R_RETURN_VAL_IF_FAIL (b, -1);
 	return buf_seek (b, addr, whence);
 }
 
@@ -260,25 +302,25 @@ R_API ut64 r_buf_tell(RBuffer *b) {
 }
 
 R_API bool r_buf_set_bytes(RBuffer *b, const ut8 *buf, ut64 length) {
-	r_return_val_if_fail (b && buf && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && buf && !b->readonly, false);
 	if (!r_buf_resize (b, 0)) {
 		return false;
 	}
-	if (r_buf_seek (b, 0, R_BUF_SET) < 0) {
+	if (r_buf_seek (b, 0, R_BUF_SET) == -1) {
 		return false;
 	}
 	if (!r_buf_append_bytes (b, buf, length)) {
 		return false;
 	}
-	return r_buf_seek (b, 0, R_BUF_SET) >= 0;
+	return r_buf_seek (b, 0, R_BUF_SET) != -1;
 }
 
 R_API bool r_buf_prepend_bytes(RBuffer *b, const ut8 *buf, ut64 length) {
-	r_return_val_if_fail (b && buf && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && buf && !b->readonly, false);
 	return r_buf_insert_bytes (b, 0, buf, length) >= 0;
 }
 
-R_API char *r_buf_to_string(RBuffer *b) {
+R_API char *r_buf_tostring(RBuffer *b) {
 	ut64 sz = r_buf_size (b);
 	char *s = malloc (sz + 1);
 	if (!s) {
@@ -293,17 +335,16 @@ R_API char *r_buf_to_string(RBuffer *b) {
 }
 
 R_API bool r_buf_append_bytes(RBuffer *b, const ut8 *buf, ut64 length) {
-	r_return_val_if_fail (b && buf && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && buf && !b->readonly, false);
 
-	if (r_buf_seek (b, 0, R_BUF_END) < 0) {
+	if (r_buf_seek (b, 0, R_BUF_END) == -1) {
 		return false;
 	}
-
 	return r_buf_write (b, buf, length) >= 0;
 }
 
 R_API bool r_buf_append_nbytes(RBuffer *b, ut64 length) {
-	r_return_val_if_fail (b && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, false);
 	ut8 *buf = R_NEWS0 (ut8, length);
 	if (!buf) {
 		return false;
@@ -314,14 +355,14 @@ R_API bool r_buf_append_nbytes(RBuffer *b, ut64 length) {
 }
 
 R_API st64 r_buf_insert_bytes(RBuffer *b, ut64 addr, const ut8 *buf, ut64 length) {
-	r_return_val_if_fail (b && !b->readonly, -1);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, -1);
 	st64 pos, r = r_buf_seek (b, 0, R_BUF_CUR);
-	if (r < 0) {
+	if (r == -1) {
 		return r;
 	}
 	pos = r;
 	r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		goto restore_pos;
 	}
 
@@ -336,7 +377,7 @@ R_API st64 r_buf_insert_bytes(RBuffer *b, ut64 addr, const ut8 *buf, ut64 length
 		goto free_tmp;
 	}
 	r = r_buf_seek (b, addr + length, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		goto free_tmp;
 	}
 	r = r_buf_write (b, tmp, tmp_length);
@@ -344,7 +385,7 @@ R_API st64 r_buf_insert_bytes(RBuffer *b, ut64 addr, const ut8 *buf, ut64 length
 		goto free_tmp;
 	}
 	r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		goto free_tmp;
 	}
 	r = r_buf_write (b, buf, length);
@@ -356,47 +397,45 @@ restore_pos:
 }
 
 R_API bool r_buf_append_ut8(RBuffer *b, ut8 n) {
-	r_return_val_if_fail (b && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
 }
 
 R_API bool r_buf_append_ut16(RBuffer *b, ut16 n) {
-	r_return_val_if_fail (b && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
 }
 
 R_API bool r_buf_append_ut32(RBuffer *b, ut32 n) {
-	r_return_val_if_fail (b && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
 }
 
 R_API bool r_buf_append_ut64(RBuffer *b, ut64 n) {
-	r_return_val_if_fail (b && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)&n, sizeof (n));
 }
 
 R_API bool r_buf_append_buf(RBuffer *b, RBuffer *a) {
-	r_return_val_if_fail (b && a && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && a && !b->readonly, false);
 	ut64 sz = 0;
 	const ut8 *tmp = r_buf_data (a, &sz);
 	return r_buf_append_bytes (b, tmp, sz);
 }
 
 R_API bool r_buf_append_buf_slice(RBuffer *b, RBuffer *a, ut64 offset, ut64 size) {
-	r_return_val_if_fail (b && a && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && a && !b->readonly, false);
 	ut8 *tmp = R_NEWS (ut8, size);
 	bool res = false;
-
-	if (!tmp) {
-		return false;
-	}
-	st64 r = r_buf_read_at (a, offset, tmp, size);
-	if (r < 0) {
-		goto err;
-	}
-	res = r_buf_append_bytes (b, tmp, r);
+	if (tmp) {
+		st64 r = r_buf_read_at (a, offset, tmp, size);
+		if (r < 0) {
+			goto err;
+		}
+		res = r_buf_append_bytes (b, tmp, r);
 err:
-	free (tmp);
+		free (tmp);
+	}
 	return res;
 }
 
@@ -434,14 +473,21 @@ R_API char *r_buf_get_string(RBuffer *b, ut64 addr) {
 }
 
 R_API ut8 *r_buf_read_all(RBuffer *b, int *blen) {
-	r_return_val_if_fail (b, NULL);
+	R_RETURN_VAL_IF_FAIL (b, NULL);
 	int buflen = r_buf_size (b);
 	if (buflen < 0) {
 		return NULL;
 	}
 	ut8 *buf = malloc (buflen + 1);
 	buf_seek (b, 0, R_BUF_SET);
-	buf_read (b, buf, buflen);
+	st64 newlen = buf_read (b, buf, buflen);
+	if (newlen != buflen) {
+		if (newlen > 0) {
+			buflen = newlen;
+		} else {
+			R_LOG_WARN ("buf_read_all fails");
+		}
+	}
 	if (blen) {
 		*blen = buflen;
 	}
@@ -449,7 +495,7 @@ R_API ut8 *r_buf_read_all(RBuffer *b, int *blen) {
 }
 
 R_API st64 r_buf_read(RBuffer *b, ut8 *buf, ut64 len) {
-	r_return_val_if_fail (b && buf, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf, -1);
 	st64 r = buf_read (b, buf, len);
 	if (r >= 0 && r < len) {
 		memset (buf + r, b->Oxff_priv, len - r);
@@ -458,7 +504,7 @@ R_API st64 r_buf_read(RBuffer *b, ut8 *buf, ut64 len) {
 }
 
 R_API st64 r_buf_write(RBuffer *b, const ut8 *buf, ut64 len) {
-	r_return_val_if_fail (b && buf && !b->readonly, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && !b->readonly, -1);
 	return buf_write (b, buf, len);
 }
 
@@ -471,7 +517,7 @@ R_API ut8 r_buf_read8(RBuffer *b) {
 R_API ut8 r_buf_read8_at(RBuffer *b, ut64 addr) {
 	ut8 res;
 	st64 r = r_buf_read_at (b, addr, &res, sizeof (res));
-	return r == sizeof (res)? res: b->Oxff_priv;
+	return (r == sizeof (res))? res: b->Oxff_priv;
 }
 
 static st64 buf_format(RBuffer *dst, RBuffer *src, const char *fmt, int n) {
@@ -517,10 +563,9 @@ static st64 buf_format(RBuffer *dst, RBuffer *src, const char *fmt, int n) {
 				ut32 d3;
 				ut64 d4;
 				st64 r = r_buf_read (src, tmp, tsize);
-				if (r < tsize) {
+				if (r != tsize) {
 					return -1;
 				}
-
 				switch (tsize) {
 				case 1:
 					d1 = r_read_ble8 (tmp);
@@ -550,29 +595,34 @@ static st64 buf_format(RBuffer *dst, RBuffer *src, const char *fmt, int n) {
 	return res;
 }
 
+// TODO: add r_buf_fnread or nfread for safety reasons. callers never know what they are doing
 R_API st64 r_buf_fread(RBuffer *b, ut8 *buf, const char *fmt, int n) {
-	r_return_val_if_fail (b && buf && fmt, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && fmt, -1);
 	// XXX: we assume the caller knows what he's doing
 	RBuffer *dst = r_buf_new_with_pointers (buf, UT64_MAX, false);
-	st64 res = buf_format (dst, b, fmt, n);
-	r_buf_free (dst);
-	return res;
+	if (dst) {
+		st64 res = buf_format (dst, b, fmt, n);
+		r_buf_free (dst);
+		return res;
+	}
+	return -1;
 }
 
+// UNSAFE
 R_API st64 r_buf_fread_at(RBuffer *b, ut64 addr, ut8 *buf, const char *fmt, int n) {
-	r_return_val_if_fail (b && buf && fmt, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && fmt, -1);
 	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		return r;
 	}
 	r = r_buf_fread (b, buf, fmt, n);
-	r_buf_seek (b, o_addr, R_BUF_SET);
+	(void)r_buf_seek (b, o_addr, R_BUF_SET);
 	return r;
 }
 
 R_API st64 r_buf_fwrite(RBuffer *b, const ut8 *buf, const char *fmt, int n) {
-	r_return_val_if_fail (b && buf && fmt && !b->readonly, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && fmt && !b->readonly, -1);
 	// XXX: we assume the caller knows what he's doing
 	RBuffer *src = r_buf_new_with_pointers (buf, UT64_MAX, false);
 	st64 res = buf_format (b, src, fmt, n);
@@ -581,10 +631,10 @@ R_API st64 r_buf_fwrite(RBuffer *b, const ut8 *buf, const char *fmt, int n) {
 }
 
 R_API st64 r_buf_fwrite_at(RBuffer *b, ut64 addr, const ut8 *buf, const char *fmt, int n) {
-	r_return_val_if_fail (b && buf && fmt && !b->readonly, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && fmt && !b->readonly, -1);
 	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		return r;
 	}
 	r = r_buf_fwrite (b, buf, fmt, n);
@@ -593,31 +643,30 @@ R_API st64 r_buf_fwrite_at(RBuffer *b, ut64 addr, const ut8 *buf, const char *fm
 }
 
 R_API st64 r_buf_read_at(RBuffer *b, ut64 addr, ut8 *buf, ut64 len) {
-	r_return_val_if_fail (b && buf, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf, -1);
 	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		return r;
 	}
-
 	r = r_buf_read (b, buf, len);
-	r_buf_seek (b, o_addr, R_BUF_SET);
+	r_buf_seek (b, (ut64)o_addr, R_BUF_SET);
 	return r;
 }
 
 R_API st64 r_buf_write_at(RBuffer *b, ut64 addr, const ut8 *buf, ut64 len) {
-	r_return_val_if_fail (b && buf && !b->readonly, -1);
+	R_RETURN_VAL_IF_FAIL (b && buf && !b->readonly, -1);
 	st64 o_addr = r_buf_seek (b, 0, R_BUF_CUR);
 	st64 r = r_buf_seek (b, addr, R_BUF_SET);
-	if (r < 0) {
+	if (r == -1) {
 		return r;
 	}
-
 	r = r_buf_write (b, buf, len);
 	r_buf_seek (b, o_addr, R_BUF_SET);
 	return r;
 }
 
+// XXX 580 use r_ref api instead
 R_API void r_buf_fini(RBuffer *b) {
 	if (!b) {
 		return;
@@ -626,7 +675,6 @@ R_API void r_buf_fini(RBuffer *b) {
 		b->refctr--;
 		return;
 	}
-
 	// free the whole_buf only if it was initially allocated by the buf types
 	if (b->methods->get_whole_buf) {
 		if (b->methods->free_whole_buf) {
@@ -638,24 +686,28 @@ R_API void r_buf_fini(RBuffer *b) {
 	buf_fini (b);
 }
 
+// XXX 580 use r_ref api instead
 R_API void r_buf_free(RBuffer *b) {
-	bool unreferenced = b && b->refctr == 0;
-	r_buf_fini (b);
-	if (unreferenced) {
-		free (b);
+	if (b) {
+		bool unreferenced = b && b->refctr == 0;
+		r_buf_fini (b);
+		if (unreferenced) {
+			free (b);
+		}
 	}
 }
 
 R_API st64 r_buf_append_string(RBuffer *b, const char *str) {
-	r_return_val_if_fail (b && str && !b->readonly, false);
+	R_RETURN_VAL_IF_FAIL (b && str && !b->readonly, false);
 	return r_buf_append_bytes (b, (const ut8 *)str, strlen (str));
 }
 
 R_API bool r_buf_resize(RBuffer *b, ut64 newsize) {
-	r_return_val_if_fail (b, false);
+	R_RETURN_VAL_IF_FAIL (b, false);
 	return buf_resize (b, newsize);
 }
 
+// XXX 580 use r_ref api instead
 R_API RBuffer *r_buf_ref(RBuffer *b) {
 	if (b) {
 		b->refctr++;
@@ -664,7 +716,8 @@ R_API RBuffer *r_buf_ref(RBuffer *b) {
 }
 
 R_API RList *r_buf_nonempty_list(RBuffer *b) {
-	return b->methods->nonempty_list? b->methods->nonempty_list (b): NULL;
+	const RBufferNonEmptyList nelist = b->methods->nonempty_list;
+	return nelist ? nelist (b): NULL;
 }
 
 R_API st64 r_buf_uleb128(RBuffer *b, ut64 *v) {
