@@ -298,7 +298,7 @@ R_API void r_graph_dfs_node_reverse(RGraph *g, RGraphNode *n, RGraphVisitor *vis
 }
 
 R_API void r_graph_dfs(RGraph *g, RGraphVisitor *vis) {
-	r_return_if_fail (g && vis);
+	R_RETURN_IF_FAIL (g && vis);
 	RGraphNode *n;
 	RListIter *it;
 
@@ -378,7 +378,7 @@ static void _dfs_ins_edge(const RGraphEdge *e, RGraphVisitor *vi) {
 }
 
 R_API RGraph *r_graph_dom_tree(RGraph *graph, RGraphNode *root) {
-	r_return_val_if_fail (graph && root, NULL);
+	R_RETURN_VAL_IF_FAIL (graph && root, NULL);
 	RGraph *g = r_graph_new ();
 	if (!g) {
 		return NULL;
@@ -465,7 +465,7 @@ static void _invert_edges (RGraph *g) {
 }
 
 R_API RGraph *r_graph_pdom_tree(RGraph *graph, RGraphNode *root) {
-	r_return_val_if_fail (graph && root, NULL);
+	R_RETURN_VAL_IF_FAIL (graph && root, NULL);
 	_invert_edges (graph);
 	RGraph *g = r_graph_dom_tree (graph, root);
 	_invert_edges (graph);
