@@ -1,10 +1,8 @@
-/* radare2 - LGPL - Copyright 2017-2018 - wargio */
+/* radare2 - LGPL - Copyright 2017-2022 - wargio */
 
-#include <r_util.h>
 #include <r_cons.h>
-#include <stdlib.h>
-#include <string.h>
-#include "./x509.h"
+#include <r_util.h>
+#include "x509.h"
 
 static bool r_x509_parse_validity(RX509Validity *validity, RASN1Object *object) {
 	RASN1Object *o;
@@ -23,7 +21,7 @@ static bool r_x509_parse_validity(RX509Validity *validity, RASN1Object *object)
 			}
 		}
 		o = object->list.objects[1];
-		if (o->klass == CLASS_UNIVERSAL && o->form == FORM_PRIMITIVE) {
+		if (o && o->klass == CLASS_UNIVERSAL && o->form == FORM_PRIMITIVE) {
 			if (o->tag == TAG_UTCTIME) {
 				validity->notAfter = r_asn1_stringify_utctime (o->sector, o->length);
 			} else if (o->tag == TAG_GENERALIZEDTIME) {
@@ -34,19 +32,26 @@ static bool r_x509_parse_validity(RX509Validity *validity, RASN1Object *object)
 	return true;
 }
 
-bool r_x509_parse_algorithmidentifier (RX509AlgorithmIdentifier *ai, RASN1Object *object) {
-	if (!ai || !object || object->list.length < 1 || !object->list.objects) {
-		return false;
-	}
-	if (object->list.objects[0] && object->list.objects[0]->klass == CLASS_UNIVERSAL && object->list.objects[0]->tag == TAG_OID) {
-		ai->algorithm = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
+static inline bool is_oid_object(RASN1Object *object) {
+	return object->list.objects[0] &&
+		object->list.objects[0]->klass == CLASS_UNIVERSAL &&
+		object->list.objects[0]->tag == TAG_OID;
+}
+
+R_API bool r_x509_parse_algorithmidentifier(RX509AlgorithmIdentifier *ai, RASN1Object *object) {
+	R_RETURN_VAL_IF_FAIL (ai && object, false);
+
+	if (object->list.length < 1 || !object->list.objects || !is_oid_object (object)) {
+			return false;
 	}
+
+	ai->algorithm = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
 	ai->parameters = NULL; // TODO
 	//ai->parameters = asn1_stringify_sector (object->list.objects[1]);
 	return true;
 }
 
-bool r_x509_parse_subjectpublickeyinfo (RX509SubjectPublicKeyInfo *spki, RASN1Object *object) {
+R_API bool r_x509_parse_subjectpublickeyinfo(RX509SubjectPublicKeyInfo *spki, RASN1Object *object) {
 	RASN1Object *o;
 	if (!spki || !object || object->list.length != 2) {
 		return false;
@@ -55,20 +60,11 @@ bool r_x509_parse_subjectpublickeyinfo (RX509SubjectPublicKeyInfo *spki, RASN1Ob
 	if (object->list.objects[1]) {
 		o = object->list.objects[1];
 		spki->subjectPublicKey = r_asn1_create_binary (o->sector, o->length);
-		if (o->list.length == 1 && o->list.objects[0] && o->list.objects[0]->list.length == 2) {
-			o = o->list.objects[0];
-			if (o->list.objects[0]) {
-				spki->subjectPublicKeyExponent = r_asn1_create_binary (o->list.objects[0]->sector, o->list.objects[0]->length);
-			}
-			if (o->list.objects[1]) {
-				spki->subjectPublicKeyModule = r_asn1_create_binary (o->list.objects[1]->sector, o->list.objects[1]->length);
-			}
-		}
 	}
 	return true;
 }
 
-bool r_x509_parse_name (RX509Name *name, RASN1Object *object) {
+R_API bool r_x509_parse_name(RX509Name *name, RASN1Object *object) {
 	ut32 i;
 	if (!name || !object || !object->list.length) {
 		return false;
@@ -111,17 +107,17 @@ bool r_x509_parse_name (RX509Name *name, RASN1Object *object) {
 	return true;
 }
 
-bool r_x509_parse_extension (RX509Extension *ext, RASN1Object *object) {
-	RASN1Object *o;
-	if (!ext || !object || object->list.length < 2) {
+R_API bool r_x509_parse_extension(RX509Extension *ext, RASN1Object *object) {
+	R_RETURN_VAL_IF_FAIL (ext && object, false);
+	if (object->list.length != 2) {
 		return false;
 	}
-	o = object->list.objects[0];
+	RASN1Object *o = object->list.objects[0];
 	if (o && o->tag == TAG_OID) {
 		ext->extnID = r_asn1_stringify_oid (o->sector, o->length);
 		o = object->list.objects[1];
 		if (o->tag == TAG_BOOLEAN && object->list.length > 2) {
-			//This field is optional (so len must be 3)
+			// This field is optional (so len must be 3)
 			ext->critical = o->sector[0] != 0;
 			o = object->list.objects[2];
 		}
@@ -132,7 +128,7 @@ bool r_x509_parse_extension (RX509Extension *ext, RASN1Object *object) {
 	return true;
 }
 
-bool r_x509_parse_extensions (RX509Extensions *ext, RASN1Object *object) {
+R_API bool r_x509_parse_extensions(RX509Extensions *ext, RASN1Object *object) {
 	ut32 i;
 	if (!ext || !object || object->list.length != 1 || !object->list.objects[0]->length) {
 		return false;
@@ -153,7 +149,7 @@ bool r_x509_parse_extensions (RX509Extensions *ext, RASN1Object *object) {
 	return true;
 }
 
-bool r_x509_parse_tbscertificate (RX509TBSCertificate *tbsc, RASN1Object *object) {
+R_API bool r_x509_parse_tbscertificate(RX509TBSCertificate *tbsc, RASN1Object *object) {
 	RASN1Object **elems;
 	ut32 i;
 	ut32 shift = 0;
@@ -176,6 +172,9 @@ bool r_x509_parse_tbscertificate (RX509TBSCertificate *tbsc, RASN1Object *object
 	if (shift < object->list.length && elems[shift]->klass == CLASS_UNIVERSAL && elems[shift]->tag == TAG_INTEGER) {
 		tbsc->serialNumber = r_asn1_stringify_integer (elems[shift]->sector, elems[shift]->length);
 	}
+	if (object->list.length < shift + 6) {
+		return false;
+	}
 	r_x509_parse_algorithmidentifier (&tbsc->signature, elems[shift + 1]);
 	r_x509_parse_name (&tbsc->issuer, elems[shift + 2]);
 	r_x509_parse_validity (&tbsc->validity, elems[shift + 3]);
@@ -206,7 +205,7 @@ bool r_x509_parse_tbscertificate (RX509TBSCertificate *tbsc, RASN1Object *object
 	return true;
 }
 
-RX509Certificate *r_x509_parse_certificate (RASN1Object *object) {
+R_API RX509Certificate *r_x509_parse_certificate(RASN1Object *object) {
 	if (!object) {
 		return NULL;
 	}
@@ -234,52 +233,58 @@ RX509Certificate *r_x509_parse_certificate (RASN1Object *object) {
 		R_FREE (cert);
 	}
 fail:
-	r_asn1_free_object (object);
+	r_asn1_object_free (object);
 	return cert;
 }
 
-RX509Certificate *r_x509_parse_certificate2 (const ut8 *buffer, ut32 length) {
-	RX509Certificate *certificate;
-	RASN1Object *object;
-	if (!buffer || !length) {
-		return NULL;
-	}
-	object = r_asn1_create_object (buffer, length, buffer);
-	certificate = r_x509_parse_certificate (object);
-	//object freed by r_x509_parse_certificate
-	return certificate;
-}
-
-RX509CRLEntry *r_x509_parse_crlentry (RASN1Object *object) {
-	RX509CRLEntry *entry;
+R_API RX509CRLEntry *r_x509_parse_crlentry(RASN1Object *object) {
 	if (!object || object->list.length != 2) {
 		return NULL;
 	}
-	entry = (RX509CRLEntry *)malloc (sizeof (RX509CRLEntry));
+	RX509CRLEntry *entry = R_NEW0 (RX509CRLEntry);
 	if (!entry) {
 		return NULL;
 	}
-	entry->userCertificate = r_asn1_create_binary (object->list.objects[0]->sector, object->list.objects[0]->length);
-	entry->revocationDate = r_asn1_stringify_utctime (object->list.objects[1]->sector, object->list.objects[1]->length);
+	struct r_asn1_object_t *obj0 = object->list.objects[0];
+	if (!obj0) {
+		free (entry);
+		return NULL;
+	}
+	entry->userCertificate = r_asn1_create_binary (obj0->sector, obj0->length);
+	struct r_asn1_object_t *obj1 = object->list.objects[1];
+	if (!obj1) {
+		free (entry);
+		return NULL;
+	}
+	entry->revocationDate = r_asn1_stringify_utctime (obj1->sector, obj1->length);
 	return entry;
 }
 
 R_API RX509CertificateRevocationList *r_x509_parse_crl(RASN1Object *object) {
-	RX509CertificateRevocationList *crl;
-	RASN1Object **elems;
 	if (!object || object->list.length < 4) {
 		return NULL;
 	}
-	crl = (RX509CertificateRevocationList *)malloc (sizeof (RX509CertificateRevocationList));
+	RX509CertificateRevocationList *crl = R_NEW0 (RX509CertificateRevocationList);
 	if (!crl) {
 		return NULL;
 	}
-	memset (crl, 0, sizeof (RX509CertificateRevocationList));
-	elems = object->list.objects;
-	r_x509_parse_algorithmidentifier (&crl->signature, elems[0]);
-	r_x509_parse_name (&crl->issuer, elems[1]);
-	crl->lastUpdate = r_asn1_stringify_utctime (elems[2]->sector, elems[2]->length);
-	crl->nextUpdate = r_asn1_stringify_utctime (elems[3]->sector, elems[3]->length);
+	RASN1Object **elems = object->list.objects;
+	if (!elems) {
+		free (crl);
+		return NULL;
+	}
+	if (elems[0]) {
+		r_x509_parse_algorithmidentifier (&crl->signature, elems[0]);
+	}
+	if (elems[1]) {
+		r_x509_parse_name (&crl->issuer, elems[1]);
+	}
+	if (elems[2]) {
+		crl->lastUpdate = r_asn1_stringify_utctime (elems[2]->sector, elems[2]->length);
+	}
+	if (elems[3]) {
+		crl->nextUpdate = r_asn1_stringify_utctime (elems[3]->sector, elems[3]->length);
+	}
 	if (object->list.length > 4 && object->list.objects[4]) {
 		ut32 i;
 		crl->revokedCertificates = calloc (object->list.objects[4]->list.length, sizeof (RX509CRLEntry *));
@@ -295,31 +300,31 @@ R_API RX509CertificateRevocationList *r_x509_parse_crl(RASN1Object *object) {
 	return crl;
 }
 
-void r_x509_free_algorithmidentifier (RX509AlgorithmIdentifier *ai) {
+R_API void r_x509_free_algorithmidentifier(RX509AlgorithmIdentifier *ai) {
 	if (ai) {
 		// no need to free ai, since this functions is used internally
-		r_asn1_free_string (ai->algorithm);
-		r_asn1_free_string (ai->parameters);
+		r_asn1_string_free (ai->algorithm);
+		r_asn1_string_free (ai->parameters);
 	}
 }
 
 static void r_x509_free_validity(RX509Validity *validity) {
 	if (validity) {
 		// not freeing validity since it's not allocated dinamically
-		r_asn1_free_string (validity->notAfter);
-		r_asn1_free_string (validity->notBefore);
+		r_asn1_string_free (validity->notAfter);
+		r_asn1_string_free (validity->notBefore);
 	}
 }
 
-void r_x509_free_name (RX509Name *name) {
+R_API void r_x509_free_name(RX509Name *name) {
 	ut32 i;
 	if (!name) {
 		return;
 	}
 	if (name->names) {
 		for (i = 0; i < name->length; i++) {
-			r_asn1_free_string (name->oids[i]);
-			r_asn1_free_string (name->names[i]);
+			r_asn1_string_free (name->oids[i]);
+			r_asn1_string_free (name->names[i]);
 		}
 		R_FREE (name->names);
 		R_FREE (name->oids);
@@ -327,10 +332,10 @@ void r_x509_free_name (RX509Name *name) {
 	// not freeing name since it's not allocated dinamically
 }
 
-void r_x509_free_extension (RX509Extension *ex) {
+void r_x509_free_extension(RX509Extension *ex) {
 	if (ex) {
-		r_asn1_free_string (ex->extnID);
-		r_asn1_free_binary (ex->extnValue);
+		r_asn1_string_free (ex->extnID);
+		r_asn1_binary_free (ex->extnValue);
 		//this is allocated dinamically so, i'll free
 		free (ex);
 	}
@@ -353,9 +358,7 @@ void r_x509_free_extensions (RX509Extensions *ex) {
 void r_x509_free_subjectpublickeyinfo (RX509SubjectPublicKeyInfo *spki) {
 	if (spki) {
 		r_x509_free_algorithmidentifier (&spki->algorithm);
-		r_asn1_free_binary (spki->subjectPublicKey);
-		r_asn1_free_binary (spki->subjectPublicKeyExponent);
-		r_asn1_free_binary (spki->subjectPublicKeyModule);
+		r_asn1_binary_free (spki->subjectPublicKey);
 		// No need to free spki, since it's a static variable.
 	}
 }
@@ -363,14 +366,14 @@ void r_x509_free_subjectpublickeyinfo (RX509SubjectPublicKeyInfo *spki) {
 void r_x509_free_tbscertificate (RX509TBSCertificate *tbsc) {
 	if (tbsc) {
 		//  version is ut32
-		r_asn1_free_string (tbsc->serialNumber);
+		r_asn1_string_free (tbsc->serialNumber);
 		r_x509_free_algorithmidentifier (&tbsc->signature);
 		r_x509_free_name (&tbsc->issuer);
 		r_x509_free_validity (&tbsc->validity);
 		r_x509_free_name (&tbsc->subject);
 		r_x509_free_subjectpublickeyinfo (&tbsc->subjectPublicKeyInfo);
-		r_asn1_free_binary (tbsc->subjectUniqueID);
-		r_asn1_free_binary (tbsc->issuerUniqueID);
+		r_asn1_binary_free (tbsc->subjectUniqueID);
+		r_asn1_binary_free (tbsc->issuerUniqueID);
 		r_x509_free_extensions (&tbsc->extensions);
 		//no need to free tbsc, since this functions is used internally
 	}
@@ -378,7 +381,7 @@ void r_x509_free_tbscertificate (RX509TBSCertificate *tbsc) {
 
 void r_x509_free_certificate (RX509Certificate *certificate) {
 	if (certificate) {
-		r_asn1_free_binary (certificate->signature);
+		r_asn1_binary_free (certificate->signature);
 		r_x509_free_algorithmidentifier (&certificate->algorithmIdentifier);
 		r_x509_free_tbscertificate (&certificate->tbsCertificate);
 		free (certificate);
@@ -387,8 +390,8 @@ void r_x509_free_certificate (RX509Certificate *certificate) {
 
 static void r_x509_free_crlentry(RX509CRLEntry *entry) {
 	if (entry) {
-		r_asn1_free_binary (entry->userCertificate);
-		r_asn1_free_string (entry->revocationDate);
+		r_asn1_binary_free (entry->userCertificate);
+		r_asn1_string_free (entry->revocationDate);
 		free (entry);
 	}
 }
@@ -398,8 +401,8 @@ void r_x509_free_crl (RX509CertificateRevocationList *crl) {
 	if (crl) {
 		r_x509_free_algorithmidentifier (&crl->signature);
 		r_x509_free_name (&crl->issuer);
-		r_asn1_free_string (crl->nextUpdate);
-		r_asn1_free_string (crl->lastUpdate);
+		r_asn1_string_free (crl->nextUpdate);
+		r_asn1_string_free (crl->lastUpdate);
 		if (crl->revokedCertificates) {
 			for (i = 0; i < crl->length; i++) {
 				r_x509_free_crlentry (crl->revokedCertificates[i]);
@@ -423,7 +426,7 @@ static void r_x509_validity_dump(RX509Validity *validity, const char *pad, RStrB
 	r_strbuf_appendf (sb, "%sNot Before: %s\n%sNot After: %s\n", pad, b, pad, a);
 }
 
-void r_x509_name_dump (RX509Name *name, const char *pad, RStrBuf *sb) {
+R_API void r_x509_name_dump(RX509Name *name, const char *pad, RStrBuf *sb) {
 	ut32 i;
 	if (!name) {
 		return;
@@ -447,18 +450,13 @@ static void r_x509_subjectpublickeyinfo_dump(RX509SubjectPublicKeyInfo *spki, co
 	if (!pad) {
 		pad = "";
 	}
-	a = spki->algorithm.algorithm ? spki->algorithm.algorithm->string : "Missing";
-	RASN1String *m = NULL;
-	if (spki->subjectPublicKeyModule) {
-		m = r_asn1_stringify_integer (spki->subjectPublicKeyModule->binary, spki->subjectPublicKeyModule->length);
+	a = spki->algorithm.algorithm? spki->algorithm.algorithm->string: "Missing";
+	RASN1String *pubkey = NULL;
+	if (spki->subjectPublicKey) {
+		pubkey = r_asn1_stringify_integer (spki->subjectPublicKey->binary, spki->subjectPublicKey->length);
 	}
-	//	RASN1String* e = r_asn1_stringify_bytes (spki->subjectPublicKeyExponent->sector, spki->subjectPublicKeyExponent->length);
-	//	r = snprintf (buffer, length, "%sAlgorithm: %s\n%sModule: %s\n%sExponent: %u bytes\n%s\n", pad, a, pad, m->string,
-	//				pad, spki->subjectPublicKeyExponent->length - 1, e->string);
-	r_strbuf_appendf (sb, "%sAlgorithm: %s\n%sModule: %s\n%sExponent: %u bytes\n", pad, a, pad, m ? m->string : "Missing",
-		pad, spki->subjectPublicKeyExponent ? spki->subjectPublicKeyExponent->length - 1 : 0);
-	r_asn1_free_string (m);
-	//	r_asn1_free_string (e);
+	r_strbuf_appendf (sb, "%sAlgorithm: %s\n%sPublic key: %u bytes\n", pad, a, pad, pubkey? spki->subjectPublicKey->length: 0);
+	r_asn1_string_free (pubkey);
 }
 
 static void r_x509_extensions_dump(RX509Extensions *exts, const char *pad, RStrBuf *sb) {
@@ -480,7 +478,7 @@ static void r_x509_extensions_dump(RX509Extensions *exts, const char *pad, RStrB
 			e->extnID ? e->extnID->string : "Missing",
 			e->critical ? "critical" : "",
 			pad, e->extnValue ? e->extnValue->length : 0);
-		//r_asn1_free_string (s);
+		//r_asn1_string_free (s);
 	}
 }
 
@@ -519,14 +517,14 @@ static void r_x509_tbscertificate_dump(RX509TBSCertificate *tbsc, const char *pa
 		iid = r_asn1_stringify_integer (tbsc->issuerUniqueID->binary, tbsc->issuerUniqueID->length);
 		if (iid) {
 			r_strbuf_appendf (sb, "%sIssuer Unique ID:\n%s  %s", pad, pad, iid->string);
-			r_asn1_free_string (iid);
+			r_asn1_string_free (iid);
 		}
 	}
 	if (tbsc->subjectUniqueID) {
 		sid = r_asn1_stringify_integer (tbsc->subjectUniqueID->binary, tbsc->subjectUniqueID->length);
 		if (sid) {
 			r_strbuf_appendf (sb, "%sSubject Unique ID:\n%s  %s", pad, pad, sid->string);
-			r_asn1_free_string (sid);
+			r_asn1_string_free (sid);
 		}
 	}
 
@@ -535,7 +533,7 @@ static void r_x509_tbscertificate_dump(RX509TBSCertificate *tbsc, const char *pa
 	free (pad2);
 }
 
-void r_x509_certificate_dump (RX509Certificate *cert, const char *pad, RStrBuf *sb) {
+R_API void r_x509_certificate_dump(RX509Certificate *cert, const char *pad, RStrBuf *sb) {
 	RASN1String *algo = NULL;
 	char *pad2;
 	if (!cert) {
@@ -559,10 +557,10 @@ void r_x509_certificate_dump (RX509Certificate *cert, const char *pad, RStrBuf *
 	r_strbuf_appendf (sb, "%sAlgorithm:\n%s%s\n%sSignature: %u bytes\n",
 		pad, pad2, algo ? algo->string : "", pad, cert->signature->length);
 	free (pad2);
-	//	r_asn1_free_string (signature);
+	//	r_asn1_string_free (signature);
 }
 
-void r_x509_crlentry_dump (RX509CRLEntry *crle, const char *pad, RStrBuf *sb) {
+R_API void r_x509_crlentry_dump(RX509CRLEntry *crle, const char *pad, RStrBuf *sb) {
 	RASN1String *id = NULL, *utc = NULL;
 	if (!crle) {
 		return;
@@ -578,10 +576,10 @@ void r_x509_crlentry_dump (RX509CRLEntry *crle, const char *pad, RStrBuf *sb) {
 			      "%sRevocation Date:\n%s  %s\n",
 		pad, pad, id ? id->string : "Missing",
 		pad, pad, utc ? utc->string : "Missing");
-	r_asn1_free_string (id);
+	r_asn1_string_free (id);
 }
 
-R_API char *r_x509_crl_to_string(RX509CertificateRevocationList *crl, const char *pad) {
+R_API char *r_x509_crl_tostring(RX509CertificateRevocationList *crl, const char *pad) {
 	RASN1String *algo = NULL, *last = NULL, *next = NULL;
 	ut32 i;
 	char *pad2, *pad3;
@@ -643,19 +641,12 @@ R_API void r_x509_subjectpublickeyinfo_json(PJ *pj, RX509SubjectPublicKeyInfo *s
 		if (spki->algorithm.algorithm) {
 			pj_ks (pj, "Algorithm", spki->algorithm.algorithm->string);
 		}
-		if (spki->subjectPublicKeyModule) {
-			m = r_asn1_stringify_integer (spki->subjectPublicKeyModule->binary, spki->subjectPublicKeyModule->length);
+		if (spki->subjectPublicKey) {
+			m = r_asn1_stringify_integer (spki->subjectPublicKey->binary, spki->subjectPublicKey->length);
 			if (m) {
-				pj_ks (pj, "Module", m->string);
+				pj_ks (pj, "Public key", m->string);
 			}
-			r_asn1_free_string (m);
-		}
-		if (spki->subjectPublicKeyExponent) {
-			m = r_asn1_stringify_integer (spki->subjectPublicKeyExponent->binary, spki->subjectPublicKeyExponent->length);
-			if (m) {
-				pj_ks (pj, "Exponent", m->string);
-			}
-			r_asn1_free_string (m);
+			r_asn1_string_free (m);
 		}
 	}
 }
@@ -683,7 +674,7 @@ R_API void r_x509_extensions_json(PJ *pj, RX509Extensions *exts) {
 				if (m) {
 					pj_ks (pj, "Value", m->string);
 				}
-				r_asn1_free_string (m);
+				r_asn1_string_free (m);
 			}
 			pj_end (pj);
 		}
@@ -700,7 +691,7 @@ R_API void r_x509_crlentry_json(PJ *pj, RX509CRLEntry *crle) {
 			if (m) {
 				pj_ks (pj, "UserCertificate", m->string);
 			}
-			r_asn1_free_string (m);
+			r_asn1_string_free (m);
 		}
 		if (crle->revocationDate) {
 			pj_ks (pj, "RevocationDate", crle->revocationDate->string);
@@ -765,14 +756,14 @@ R_API void r_x509_tbscertificate_json(PJ *pj, RX509TBSCertificate *tbsc) {
 			if (m) {
 				pj_ks (pj, "IssuerUniqueID", m->string);
 			}
-			r_asn1_free_string (m);
+			r_asn1_string_free (m);
 		}
 		if (tbsc->subjectUniqueID) {
 			m = r_asn1_stringify_integer (tbsc->subjectUniqueID->binary, tbsc->subjectUniqueID->length);
 			if (m) {
 				pj_ks (pj, "SubjectUniqueID", m->string);
 			}
-			r_asn1_free_string (m);
+			r_asn1_string_free (m);
 		}
 		pj_k (pj, "Extensions");
 		r_x509_extensions_json (pj, &tbsc->extensions);
@@ -795,7 +786,7 @@ R_API void r_x509_certificate_json(PJ *pj, RX509Certificate *certificate) {
 		if (m) {
 			pj_ks (pj, "Signature", m->string);
 		}
-		r_asn1_free_string (m);
+		r_asn1_string_free (m);
 	}
 	pj_end (pj);
 }
