@@ -1,17 +1,20 @@
-/* sdb - MIT - Copyright 2011-2021 - pancake */
+/* sdb - MIT - Copyright 2011-2022 - pancake */
 
 #include <signal.h>
-#include <stdio.h>
-#include <string.h>
 #include <fcntl.h>
 #ifndef HAVE_SYSTEM
 #define HAVE_SYSTEM 1
 #endif
+#include <sdb/sdb.h>
 #if USE_DLSYSTEM
 #include <dlfcn.h>
 #endif
-#include "sdb.h"
 
+#if __SDB_WINDOWS__
+#define ftruncate _chsize
+#endif
+
+// TODO: enums must be uppercase
 typedef enum {
 	text,
 	zero,
@@ -21,6 +24,7 @@ typedef enum {
 	perf
 } MainFormat;
 
+// TODO: enums must be uppercase
 typedef enum {
 	nope = 0,
 	dash,
@@ -57,12 +61,15 @@ static void terminate(int sig UNUSED) {
 		s = NULL;
 		exit (1);
 	}
-	sdb_free (s);
+	sdb_gh_fini ();
+	//sdb_free (s);
 	exit (sig < 2? sig: 0);
 }
 
-static void write_null(void) {
-	(void)write (1, "", 1);
+static int write_null(void) {
+	// success = write returns 1 == 1 -> 0
+	// failure = write returns 0 != 1 -> 1
+	return write (1, "", 1) != 1;
 }
 
 #define BS 128
@@ -84,17 +91,17 @@ static char *slurp(FILE *f, size_t *sz) {
 		/* run test/add10k.sh script to benchmark */
 		const int buf_size = 96096;
 
-		buf = calloc (1, buf_size);
+		buf = (char *)sdb_gh_calloc (1, buf_size);
 		if (!buf) {
 			return NULL;
 		}
 
 		if (!fgets (buf, buf_size, f)) {
-			free (buf);
+			sdb_gh_free (buf);
 			return NULL;
 		}
 		if (feof (f)) {
-			free (buf);
+			sdb_gh_free (buf);
 			return NULL;
 		}
 
@@ -103,15 +110,11 @@ static char *slurp(FILE *f, size_t *sz) {
 			buf[buf_len - 1] = '\0';
 		}
 
-		char *newbuf = realloc (buf, buf_len + 1);
-		// realloc behaves like free if buf_len is 0
-		if (!newbuf) {
-			return buf;
-		}
-		return newbuf;
+		char *newbuf = (char *)sdb_gh_realloc (buf, buf_len + 1);
+		return newbuf? newbuf: buf;
 	}
 #endif
-	buf = calloc (BS + 1, 1);
+	buf = (char *)sdb_gh_calloc (BS + 1, 1);
 	if (!buf) {
 		return NULL;
 	}
@@ -119,7 +122,7 @@ static char *slurp(FILE *f, size_t *sz) {
 	len = 0;
 	for (;;) {
 		if (next) {
-			free (buf);
+			sdb_gh_free (buf);
 			buf = next;
 			bufsize = nextlen + blocksize;
 			//len = nextlen;
@@ -149,7 +152,7 @@ static char *slurp(FILE *f, size_t *sz) {
 				int nlen = nl - buf;
 				nextlen = len - nlen;
 				if (nextlen > 0) {
-					next = malloc (nextlen + blocksize + 1);
+					next = (char *)sdb_gh_malloc (nextlen + blocksize + 1);
 					if (!next) {
 						eprintf ("Cannot malloc %d\n", nextlen);
 						break;
@@ -169,7 +172,7 @@ static char *slurp(FILE *f, size_t *sz) {
 		}
 #endif
 		bufsize += blocksize;
-		tmp = realloc (buf, bufsize + 1);
+		tmp = (char *)sdb_gh_realloc (buf, bufsize + 1);
 		if (!tmp) {
 			bufsize -= blocksize;
 			break;
@@ -181,7 +184,7 @@ static char *slurp(FILE *f, size_t *sz) {
 		*sz = len;
 	}
 	if (len < 1) {
-		free (buf);
+		sdb_gh_free (buf);
 		return buf = NULL;
 	}
 	buf[len] = 0;
@@ -213,7 +216,7 @@ static char* get_name(const char*name) {
 		}
 		l--;
 	}
-	char *n = strdup (name);
+	char *n = sdb_strdup (name);
 	char *v, *d = n;
 	for (v = (char*)n; *v; v++) {
 		if (*v == '.') {
@@ -237,9 +240,9 @@ static char* get_cname(const char*name) {
 		}
 		l--;
 	}
-	char *n = strdup (name);
+	char *n = sdb_strdup (name);
 	char *v, *d = n;
-	for (v=(char*)n; *v; v++) {
+	for (v = (char*)n; *v; v++) {
 		if (*v == '/' || *v == '-') {
 			*d++ = '_';
 			continue;
@@ -254,7 +257,7 @@ static char* get_cname(const char*name) {
 }
 
 static char *escape(const char *b, int ch) {
-	char *a = calloc ((1 + strlen (b)), 4);
+	char *a = (char *)sdb_gh_calloc ((1 + strlen (b)), 4);
 	char *c = a;
 	while (*b) {
 		if (*b == ch) {
@@ -293,7 +296,7 @@ static void sdb_dump_cb(MainOptions *mo, const char *k, const char *v, const cha
 		if (!strcmp (v, "true") || !strcmp (v, "false")) {
 			printf ("%s\"%s\":%s", comma, k, v);
 		} else if (sdb_isnum (v)) {
-			printf ("%s\"%s\":%"ULLFMT"u", comma, k, sdb_atoi (v));
+			printf ("%s\"%s\":%" PRIu64, comma, k, sdb_atoi (v));
 		} else if (*v == '{' || *v == '[') {
 			printf ("%s\"%s\":%s", comma, k, v);
 		} else {
@@ -310,8 +313,8 @@ static void sdb_dump_cb(MainOptions *mo, const char *k, const char *v, const cha
 			} else {
 				printf ("%s,\"%s\"\n", a, b);
 			}
-			free (a);
-			free (b);
+			sdb_gh_free (a);
+			sdb_gh_free (b);
 		}
 		break;
 	case zero:
@@ -325,14 +328,14 @@ static void sdb_dump_cb(MainOptions *mo, const char *k, const char *v, const cha
 
 static void cgen_header(MainOptions *mo, const char *cname) {
 	if (mo->textmode) {
-		printf ("// SDB-CGEN V"SDB_VERSION"\n");
+		printf ("// SDB-CGEN V" SDB_VERSION "\n");
 		printf ("// gcc -DMAIN=1 %s.c ; ./a.out > %s.h\n", cname, cname);
 		printf ("#include <ctype.h>\n");
 		printf ("#include <stdio.h>\n");
 		printf ("#include <string.h>\n");
 		printf ("\n");
 		printf ("struct kv { const char *name; const char *value; };\n");
-		printf ("static struct kv kvs[] = {\n");
+		printf ("static const struct kv kvs[] = {\n");
 	} else {
 		printf ("%%{\n");
 		printf ("// gperf -aclEDCIG --null-strings -H sdb_hash_c_%s -N sdb_get_c_%s -t %s.gperf > %s.c\n", cname, cname, cname, cname);
@@ -391,7 +394,7 @@ static void cgen_footer(MainOptions *mo, const char *name, const char *cname) {
 		return;
 	}
 	printf ("%%%%\n");
-	printf ("// SDB-CGEN V"SDB_VERSION"\n");
+	printf ("// SDB-CGEN V" SDB_VERSION "\n");
 	printf ("// %p\n", cname);
 	printf ("typedef int (*GperfForeachCallback)(void *user, const char *k, const char *v);\n");
 	printf ("int gperf_%s_foreach(GperfForeachCallback cb, void *user) {\n", cname);
@@ -433,7 +436,7 @@ static void cgen_footer(MainOptions *mo, const char *name, const char *cname) {
 		"			char *comma = strchr (line, ',');\n"
 		"			if (comma) {\n"
 		"				*comma = 0;\n"
-		"				char *up = strdup (line);\n"
+		"				char *up = sdb_strdup (line);\n"
 		"				char *p = up; while (*p) { *p = toupper (*p); p++; }\n"
 		"				printf (\"#define GPERF_%s_%%s %%d\\n\",\n"
 		"					line, sdb_hash_c_%s (line, comma - line));\n"
@@ -480,24 +483,24 @@ static int sdb_dump(MainOptions *mo) {
 		break;
 	}
 
+	int ret = 0;
 	if (db->fd == -1) {
 		SdbList *l = sdb_foreach_list (db, true);
 		if (!mo->textmode && mo->format == cgen && ls_length (l) > SDB_MAX_GPERF_KEYS) {
 			ls_free (l);
 			eprintf ("Error: gperf doesn't work with datasets with more than 15.000 keys.\n");
-			free (name);
-			free (cname);
-			return -1;
+			ret = -1;
+			goto fail;
 		}
 		SdbKv *kv;
 		SdbListIter *it;
-		ls_foreach (l, it, kv) {
-			const char *k = sdbkv_key (kv);
-			const char *v = sdbkv_value (kv);
-			if (v && *v && grep && !strstr (k, expgrep) && !strstr (v, expgrep)) {
+		ls_foreach_cast (l, it, SdbKv*, kv) {
+			const char *sk = sdbkv_key (kv);
+			const char *sv = sdbkv_value (kv);
+			if (sv && *sv && grep && !strstr (sk, expgrep) && !strstr (sv, expgrep)) {
 				continue;
 			}
-			sdb_dump_cb (mo, k, v, comma);
+			sdb_dump_cb (mo, sk, sv, comma);
 			comma = ",";
 		}
 		ls_free (l);
@@ -505,59 +508,62 @@ static int sdb_dump(MainOptions *mo) {
 		int count = 0;
 		while (sdb_dump_dupnext (db, k, &v, NULL)) {
 			if (v && *v && grep && !strstr (k, expgrep) && !strstr (v, expgrep)) {
-				free (v);
+				sdb_gh_free (v);
 				continue;
 			}
 			sdb_dump_cb (mo, k, v, comma);
 			comma = ",";
-			free (v);
+			sdb_gh_free (v);
 			if (!mo->textmode && mo->format == cgen && count++ > SDB_MAX_GPERF_KEYS) {
 				eprintf ("Error: gperf doesn't work with datasets with more than 15.000 keys.\n");
-				free (name);
-				free (cname);
-				return -1;
+				ret = -1;
 			}
 		}
 	}
-	switch (mo->format) {
-	case zero:
-		fflush (stdout);
-		write_null ();
-		break;
-	case perf:
-	case cgen:
-		cgen_footer (mo, name, cname);
-		break;
-	case json:
-		printf ("}\n");
-		break;
-	default:
-		break;
+	if (ret == 0) {
+		switch (mo->format) {
+		case zero:
+			fflush (stdout);
+			ret = write_null ();
+			break;
+		case perf:
+		case cgen:
+			cgen_footer (mo, name, cname);
+			break;
+		case json:
+			printf ("}\n");
+			break;
+		default:
+			break;
+		}
 	}
+fail:
+	sdb_gh_free (cname);
+	sdb_gh_free (name);
 	sdb_free (db);
-	free (cname);
-	free (name);
-	return 0;
+	return ret;
 }
 
-static int insertkeys(Sdb *s, const char **args, int nargs, int mode) {
+static int insertkeys(Sdb *db, const char **args, int nargs, int mode) {
 	int must_save = 0;
 	if (args && nargs > 0) {
 		int i;
 		for (i = 0; i < nargs; i++) {
 			switch (mode) {
 			case '-':
-				must_save |= sdb_query (s, args[i]);
+				if (sdb_query (db, args[i])) {
+					must_save = true;
+				}
 				break;
 			case '=':
 				if (strchr (args[i], '=')) {
-					char *v, *kv = (char *) strdup (args[i]);
+					char *v, *kv = (char *) sdb_strdup (args[i]);
 					v = strchr (kv, '=');
 					if (v) {
 						*v++ = 0;
-						sdb_disk_insert (s, kv, v);
+						sdb_disk_insert (db, kv, v);
 					}
-					free (kv);
+					sdb_gh_free (kv);
 				}
 				break;
 			}
@@ -590,7 +596,7 @@ static int createdb(const char *f, const char **args, int nargs) {
 		if (!sdb_text_load_buf (s, in, len)) {
 			eprintf ("Failed to read text sdb from stdin\n");
 		}
-		free (in);
+		sdb_gh_free (in);
 	}
 	sdb_sync (s);
 	return ret;
@@ -620,7 +626,7 @@ static int showusage(int o) {
 }
 
 static int showversion(void) {
-	printf ("sdb "SDB_VERSION "\n");
+	printf ("sdb " SDB_VERSION "\n");
 	fflush (stdout);
 	return 0;
 }
@@ -634,12 +640,12 @@ static int jsonIndent(void) {
 	}
 	out = sdb_json_indent (in, "  ");
 	if (!out) {
-		free (in);
+		sdb_gh_free (in);
 		return 1;
 	}
 	puts (out);
-	free (out);
-	free (in);
+	sdb_gh_free (out);
+	sdb_gh_free (in);
 	return 0;
 }
 
@@ -652,12 +658,12 @@ static int base64encode(void) {
 	}
 	out = sdb_encode (in, (int)len);
 	if (!out) {
-		free (in);
+		sdb_gh_free (in);
 		return 1;
 	}
 	puts (out);
-	free (out);
-	free (in);
+	sdb_gh_free (out);
+	sdb_gh_free (in);
 	return 0;
 }
 
@@ -669,11 +675,10 @@ static int base64decode(void) {
 		int declen;
 		out = sdb_decode (in, &declen);
 		if (out && declen >= 0) {
-			(void)write (1, out, declen);
-			ret = 0;
+			ret = (write (1, out, declen) == declen)? 0: 1;
 		}
-		free (out);
-		free (in);
+		sdb_gh_free (out);
+		sdb_gh_free (in);
 	}
 	return ret;
 }
@@ -687,7 +692,7 @@ static void dbdiff_cb(const SdbDiff *diff, void *user) {
 	char *buf = sbuf;
 	char *hbuf = NULL;
 	if ((size_t)r >= sizeof (sbuf)) {
-		hbuf = malloc (r + 1);
+		hbuf = (char *)sdb_gh_malloc (r + 1);
 		if (!hbuf) {
 			return;
 		}
@@ -698,7 +703,7 @@ static void dbdiff_cb(const SdbDiff *diff, void *user) {
 	}
 	printf ("\x1b[%sm%s\x1b[0m\n", diff->add ? "32" : "31", buf);
 beach:
-	free (hbuf);
+	sdb_gh_free (hbuf);
 }
 
 static bool dbdiff(const char *a, const char *b) {
@@ -741,14 +746,14 @@ static int sdb_system(const char *cmd) {
 
 static int gen_gperf(MainOptions *mo, const char *file, const char *name) {
 	const size_t buf_size = 4096;
-	char *buf = malloc (buf_size);
+	char *buf = (char *)sdb_gh_malloc (buf_size);
 	if (!buf) {
 		return -1;
 	}
 	size_t out_size = strlen (file) + 32;
-	char *out = malloc (out_size);
+	char *out = (char *)sdb_gh_malloc (out_size);
 	if (!out) {
-		free (buf);
+		sdb_gh_free (buf);
 		return -1;
 	}
 	if (mo->outfile) {
@@ -760,11 +765,16 @@ static int gen_gperf(MainOptions *mo, const char *file, const char *name) {
 	if (wd == -1) {
 		wd = open (out, O_RDWR | O_CREAT, 0644);
 	} else {
-		ftruncate (wd, 0);
+		if (ftruncate (wd, 0) == -1) {
+			sdb_gh_free (out);
+			sdb_gh_free (buf);
+			close (wd);
+			return -1;
+		}
 	}
 	int rc = -1;
 	if (wd != -1) {
-#ifdef __wasi__
+#if __wasi__ || EMSCRIPTEN
 		rc = sdb_dump (mo); // file, MODE_CGEN, false, NULL);
 		fflush (stdout);
 #else
@@ -783,9 +793,12 @@ static int gen_gperf(MainOptions *mo, const char *file, const char *name) {
 	} else {
 		if (rc == 0) {
 			char *cname = get_cname (name);
+			if (!cname) {
+				return -1;
+			}
 			snprintf (buf, buf_size, "gperf -aclEDCIG --null-strings -H sdb_hash_c_%s"
 					" -N sdb_get_c_%s -t %s.gperf > %s.c\n", cname, cname, name, name);
-			free (cname);
+			sdb_gh_free (cname);
 			rc = sdb_system (buf);
 			if (rc == 0) {
 				snprintf (buf, buf_size, "gcc -DMAIN=1 %s.c ; ./a.out > %s.h\n", name, name);
@@ -794,21 +807,20 @@ static int gen_gperf(MainOptions *mo, const char *file, const char *name) {
 					eprintf ("Generated %s.c and %s.h\n", name, name);
 				}
 			} else {
-				eprintf ("Cannot run gperf\n");
-				eprintf ("%s\n", buf);
+				eprintf ("Cannot run gperf: %s\n", buf);
 			}
 		} else {
 			eprintf ("Outdated sdb binary in PATH?\n");
 		}
 	}
-	free (out);
-	free (buf);
+	sdb_gh_free (out);
+	sdb_gh_free (buf);
 	return rc;
 }
 
 static const char *main_argparse_getarg(MainOptions *mo) {
 	int cur = mo->argi;
-	if (mo->argi + 1>= mo->argc) {
+	if (mo->argi + 1 >= mo->argc) {
 		return NULL;
 	}
 	mo->argi++;
@@ -938,7 +950,7 @@ static MainOptions *main_argparse(MainOptions *mo, int argc, const char **argv)
 	return mo;
 }
 
-int main(int argc, const char **argv) {
+SDB_API int sdb_main(int argc, const char **argv) {
 	char *line;
 	int i;
 
@@ -946,7 +958,6 @@ int main(int argc, const char **argv) {
 	if (argc < 2) {
 		return showusage (1);
 	}
-
 	MainOptions _mo = {0};
 	MainOptions *mo = &_mo;
 	main_argparse (mo, argc, argv);
@@ -968,13 +979,13 @@ int main(int argc, const char **argv) {
 				return showusage (1);
 			}
 			const char *file = mo->argv[mo->db0];
-			char *name = strdup (file);
+			char *name = sdb_strdup (file);
 			char *p = strchr (name, '.');
 			if (p) {
 				*p = 0;
 			}
 			int rc = gen_gperf (mo, file, name);
-			free (name);
+			sdb_gh_free (name);
 			return rc;
 		}
 	default:
@@ -1004,25 +1015,31 @@ int main(int argc, const char **argv) {
 			int kvs = mo->db0 + 2;
 			if (mo->argi + 2 < mo->argc) {
 				for (i = mo->argi + 2; i < argc; i++) {
-					save |= sdb_query (s, mo->argv[i]);
+					if (sdb_query (s, mo->argv[i])) {
+						save = true;
+					}
 					if (mo->format) {
 						fflush (stdout);
-						write_null ();
+						ret = write_null ();
 					}
 				}
 			} else {
-				if (kvs < argc) {
-					save |= insertkeys (s, argv + mo->argi + 2, argc - kvs, '-');
+				if (kvs < argc && insertkeys (s, argv + mo->argi + 2, argc - kvs, '-')) {
+					save = true;
 				}
 				for (; (line = slurp (stdin, NULL));) {
-					save |= sdb_query (s, line);
+					if (sdb_query (s, line)) {
+						save = true;
+					}
 					if (mo->format) {
 						fflush (stdout);
-						write_null ();
+						ret = write_null ();
 					}
-					free (line);
+					sdb_gh_free (line);
 				}
 			}
+		} else {
+			eprintf ("Cannot create database\n");
 		}
 		break;
 	case eqeq: // "="
@@ -1043,10 +1060,12 @@ int main(int argc, const char **argv) {
 		sdb_config (s, options);
 		if (mo->argi + 1 < mo->argc) {
 			for (i = mo->db0 + 1; i < argc; i++) {
-				save |= sdb_query (s, mo->argv[i]);
+				if (sdb_query (s, mo->argv[i])) {
+					save = true;
+				}
 				if (mo->format) {
 					fflush (stdout);
-					write_null ();
+					ret = write_null ();
 				}
 			}
 		} else {
